/*
 * Copyright (C) 2002-2007  Auriga
 *
 * This file is part of Auriga.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

//#define DEBUG_FUNCIN
//#define DEBUG_DISP

// Ro[gꂽԌ\
//#define DEBUG_DISASM

// sߒ\
//#define DEBUG_RUN

// nbV̌vZʂo
//#define DEBUG_HASH

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#ifndef WINDOWS
	#include <sys/time.h>
#endif
#include <time.h>
#include <setjmp.h>

#include "db.h"
#include "socket.h"
#include "timer.h"
#include "malloc.h"
#include "mmo.h"
#include "lock.h"
#include "nullpo.h"
#include "utils.h"
#include "sqldbs.h"

#include "map.h"
#include "guild.h"
#include "clif.h"
#include "chrif.h"
#include "itemdb.h"
#include "pc.h"
#include "script.h"
#include "storage.h"
#include "mob.h"
#include "npc.h"
#include "pet.h"
#include "intif.h"
#include "skill.h"
#include "chat.h"
#include "battle.h"
#include "party.h"
#include "atcommand.h"
#include "status.h"
#include "itemdb.h"
#include "unit.h"
#include "homun.h"
#include "merc.h"
//  ++ NPCIX by Blaze
#include "vending.h"
//  -- NPCIX by Blaze

#define SCRIPT_BLOCK_SIZE 512

enum { LABEL_NEXTLINE = 1, LABEL_START };

static unsigned char * script_buf = NULL;
static int script_pos,script_size;

static struct script_str_data {
	int type;
	int str;
	int backpatch;
	int label;
	int (*func)(struct script_state *st);
	union {
		int val;
		char *str;
	} u;
	int next;
} *str_data;

static char *str_buf;
static int str_num = LABEL_START, str_data_size;

#define SCRIPT_HASH_SIZE 1021
int str_hash[SCRIPT_HASH_SIZE];

static struct dbt *mapreg_db    = NULL;
static struct dbt *mapregstr_db = NULL;
static int mapreg_dirty = 0;
#define MAPREG_AUTOSAVE_INTERVAL	(300*1000)

struct linkdb_node *scriptlabel_db = NULL;
static struct dbt *userfunc_db = NULL;

static char refine_posword[11][32] = { "","","","E","[u","C","ANZT[1","ANZT[2","2","3","ĂȂ" };

static char error_marker_start[16] = "";
static char error_marker_end[16]   = "";

static struct Script_Config {
	int warn_func_no_comma;
	int warn_cmd_no_comma;
	int warn_func_mismatch_paramnum;
	int warn_cmd_mismatch_paramnum;
	int check_cmdcount;
	int check_gotocount;
	int debug_vars;
} script_config;

static int parse_cmd;

// G[
static jmp_buf error_jump;
static char*   error_msg;
static char*   error_pos;
struct script_code error_code;	// G[̃_~[f[^

// if , switch ̎
enum {
	TYPE_NULL = 0,
	TYPE_IF,
	TYPE_SWITCH,
	TYPE_WHILE,
	TYPE_FOR,
	TYPE_DO,
	TYPE_USERFUNC
};

static struct {
	struct {
		int type;
		int index;
		int count;
		int flag;
		struct linkdb_node *case_label;
	} curly[256];		// EJbȐ
	int curly_count;	// EJbR̐
	int index;			// XNvgŎgp\̐
} syntax;

static int parse_syntax_for_flag = 0;

static int vars_count = 0;
static struct dbt* vars_db = NULL;

struct vars_info {
	int use_count;
	char *file;
	int   line;
};

extern struct script_function {
	int (*func)(struct script_state *st);
	const char *name;
	const char *arg;
} buildin_func[];

static struct linkdb_node *sleep_db = NULL;

enum {
	C_NOP = 0,
	C_POS,
	C_INT,
	C_PARAM,
	C_FUNC,
	C_STR,
	C_CONSTSTR,
	C_PTR,
	C_ARG,
	C_NAME,
	C_EOL,
	C_RETINFO,

	// [U`֐Q
	C_USERFUNC,
	C_USERFUNC_POS,

	// operator
	C_OP3,
	C_LOR,
	C_LAND,
	C_LE,
	C_LT,
	C_GE,
	C_GT,
	C_EQ,
	C_NE,
	C_XOR,
	C_OR,
	C_AND,
	C_ADD,
	C_SUB,
	C_MUL,
	C_DIV,
	C_MOD,
	C_NEG,
	C_LNOT,
	C_NOT,
	C_R_SHIFT,
	C_L_SHIFT,
};


/*==========================================
 * [Jvg^Cv錾 (Kvȕ̂)
 *------------------------------------------
 */
static unsigned char* parse_curly_close(unsigned char *p);
static unsigned char* parse_syntax_close(unsigned char *p);
static unsigned char* parse_syntax(unsigned char *p);

static unsigned char* parse_subexpr(unsigned char *,int);
static int get_com(unsigned char *script,int *pos);
static int get_num(unsigned char *script,int *pos);

static int mapreg_setreg(int num,int val,int eternal);
static int mapreg_setregstr(int num,const char *str,int eternal);

#ifndef NO_CSVDB_SCRIPT
static int script_csvinit( void );
static int script_csvfinal( void );
#endif

/*==========================================
 *
 *------------------------------------------
 */
struct dbt* script_get_userfunc_db(void)
{
	if(!userfunc_db)
		userfunc_db = strdb_init(50);

	return userfunc_db;
}

/*==========================================
 * G[bZ[Wo
 *------------------------------------------
 */
static void disp_error_message(const char *mes,unsigned char *pos)
{
	error_msg = (char *)aStrdup(mes);
	error_pos = (char *)pos;
	longjmp( error_jump, 1 );
}

/*==========================================
 * ̃nbVvZ
 *------------------------------------------
 */
static unsigned int calc_hash(const unsigned char *p)
{
	unsigned int h=0;

	// SDBM Algorithm
	while(*p){
		h=(h<<6)+(h<<16)-h;
		h+=(unsigned char)tolower(*p++);
	}
	return h % SCRIPT_HASH_SIZE;
}

/*==========================================
 * str_data̒ɖO邩
 * ̂łΔԍA-1
 *------------------------------------------
 */
static int search_str(const unsigned char *p)
{
	int i = str_hash[calc_hash(p)];

	while(i){
		if(strcmpi(str_buf+str_data[i].str,p)==0){
			return i;
		}
		i=str_data[i].next;
	}
	return -1;
}

/*==========================================
 * str_dataɖOo^
 * ̂łΔԍAΓo^ĐVKԍ
 *------------------------------------------
 */
static int add_str(const unsigned char *p)
{
	static int str_pos=0,str_size=0;
	int i,len;

	i=calc_hash(p);
	if(str_hash[i]==0){
		str_hash[i]=str_num;
	} else {
		i=str_hash[i];
		for(;;){
			if(strcmpi(str_buf+str_data[i].str,p)==0){
				return i;
			}
			if(str_data[i].next==0)
				break;
			i=str_data[i].next;
		}
		str_data[i].next=str_num;
	}
	if(str_num >= str_data_size){
		str_data_size += 512;
		str_data = (struct script_str_data *)aRealloc(str_data,sizeof(str_data[0])*str_data_size);
		memset(str_data + (str_data_size - 512), '\0', 512);
	}
	len=(int)strlen(p);
	while(str_pos+len+1 >= str_size){
		str_size += 4096;
		str_buf = (char *)aRealloc(str_buf,str_size);
		memset(str_buf + (str_size - 4096), '\0', 4096);
	}
	memcpy(str_buf+str_pos,p,len+1);
	str_data[str_num].type      = C_NOP;
	str_data[str_num].str       = str_pos;
	str_data[str_num].next      = 0;
	str_data[str_num].func      = NULL;
	str_data[str_num].backpatch = -1;
	str_data[str_num].label     = -1;
	str_pos += len+1;

	return str_num++;
}

/*==========================================
 * XNvgobt@TCY̊g
 *------------------------------------------
 */
static void expand_script_buf(void)
{
	script_size+=SCRIPT_BLOCK_SIZE;
	script_buf=(unsigned char *)aRealloc(script_buf,script_size);
}

/*==========================================
 * XNvgobt@ɂPoCg
 *------------------------------------------
 */
#define add_scriptb(a) { if( script_pos+1>=script_size ) expand_script_buf(); script_buf[script_pos++]=(a); }

/*==========================================
 * XNvgobt@Ƀf[^^Cv
 *------------------------------------------
 */
static void add_scriptc(int a)
{
	while(a>=0x40){
		add_scriptb((a&0x3f)|0x40);
		a=(a-0x40)>>6;
	}
	add_scriptb(a&0x3f);
}

/*==========================================
 * XNvgobt@ɐ
 *------------------------------------------
 */
static void add_scripti(int a)
{
	while(a>=0x40){
		add_scriptb(a|0xc0);
		a=(a-0x40)>>6;
	}
	add_scriptb(a|0x80);
}

/*==========================================
 * XNvgobt@Ƀx/ϐ/֐
 *------------------------------------------
 */
// ő16M܂
static void add_scriptl(int l)
{
	switch(str_data[l].type) {
	case C_POS:
	case C_USERFUNC_POS:
		add_scriptc(C_POS);
		add_scriptb(str_data[l].label);
		add_scriptb(str_data[l].label>>8);
		add_scriptb(str_data[l].label>>16);
		break;
	case C_NOP:
	case C_USERFUNC:
		// x̉\̂backpatchpf[^ߍ
		{
			int backpatch = str_data[l].backpatch;
			add_scriptc(C_NAME);
			str_data[l].backpatch=script_pos;
			add_scriptb(backpatch);
			add_scriptb(backpatch>>8);
			add_scriptb(backpatch>>16);
		}
		break;
	case C_INT:
		add_scripti(abs(str_data[l].u.val));
		if(str_data[l].u.val < 0)
			add_scriptc(C_NEG);
		break;
	case C_STR:
		{
			char *p = str_data[l].u.str;
			add_scriptc(C_STR);
			while(p && *p) {
				add_scriptb(*p++);
			}
			add_scriptb(0);
		}
		break;
	default:
		// ̗prƊm肵Ă̂Ő̂܂
		add_scriptc(C_NAME);
		add_scriptb(l);
		add_scriptb(l>>8);
		add_scriptb(l>>16);
		break;
	}
}

/*==========================================
 * x
 *------------------------------------------
 */
static void set_label(int l,int pos,unsigned char *p)
{
	int i, next;

	if(str_data[l].label != -1) {
		disp_error_message("dup label",p);
	}
	str_data[l].type  = (str_data[l].type == C_USERFUNC) ? C_USERFUNC_POS : C_POS;
	str_data[l].label = pos;

	i = str_data[l].backpatch;
	while(i >= 0 && i != 0x00ffffff) {
		next = (*(int*)(script_buf+i)) & 0x00ffffff;
		script_buf[i-1] = C_POS;
		script_buf[i]   = pos;
		script_buf[i+1] = pos>>8;
		script_buf[i+2] = pos>>16;
		i = next;
	}
}

/*==========================================
 * Xy[X/Rgǂݔ΂
 *------------------------------------------
 */
static unsigned char *skip_space(unsigned char *p)
{
	while(1){
		while(isspace(*p))
			p++;
		if(p[0]=='/' && p[1]=='/'){
			while(*p && *p!='\n')
				p++;
		} else if(p[0]=='/' && p[1]=='*'){
			p++;
			while(*p && (p[-1]!='*' || p[0]!='/'))
				p++;
			if(*p) p++;
		} else {
			break;
		}
	}
	return p;
}

/*==========================================
 * PPXLbv
 *------------------------------------------
 */
static unsigned char *skip_word(unsigned char *p)
{
	// prefix
	if(*p=='\'') p++;	// XNvgˑϐ
	if(*p=='$')  p++;	// MAPILϐp
	if(*p=='@')  p++;	// ꎞIϐp(like weiss)
	if(*p=='#')  p++;	// accountϐp
	if(*p=='#')  p++;	// [haccountϐp

	while(isalnum(*p) || *p=='_' || *p>=0x81) {
		if(*p>=0x81 && p[1])
			p+=2;
		else
			p++;
	}
	// postfix
	if(*p=='$') p++;	// ϐ

	return p;
}

/*==========================================
 * R[h
 *------------------------------------------
 */
static int set_control_code(unsigned char p)
{
	switch(p) {
		case 'n':
			add_scriptb('\r');
			add_scriptb('\n');
			break;
		case 'r':
			add_scriptb('\r');
			break;
		case 't':
			add_scriptb('\t');
			break;
		case 'a':
			add_scriptb('\a');
			break;
		case 'b':
			add_scriptb('\b');
			break;
		default:
			return 0;
	}
	return 1;
}

/*==========================================
 * ̉
 *------------------------------------------
 */
static unsigned char* parse_simpleexpr(unsigned char *p)
{
	p=skip_space(p);

#ifdef DEBUG_FUNCIN
	if(battle_config.etc_log)
		printf("parse_simpleexpr %s\n",p);
#endif
	if(*p==';' || *p==','){
		disp_error_message("unexpected expr end",p);
	}
	if(*p=='('){
		p=parse_subexpr(p+1,-1);
		p=skip_space(p);
		if((*p++)!=')'){
			disp_error_message("unmatch ')'",p-1);
		}
	} else if(isdigit(*p) || ((*p=='-' || *p=='+') && isdigit(p[1]))){
		unsigned char *p2;
		char *np;
		int i;
		p2 = p;
		if(*p2 == '-' || *p2 == '+')
			p2++;
		if(*p2 == '0' && p2[1] == 'b') {
			// 2i\L
			p2++;
			*p2 = '0';
			i = strtoul(p,&np,2);
			*p2 = 'b';
		} else {
			i = strtoul(p,&np,0);
		}
		add_scripti(i);
		p=np;
	} else if(*p=='"'){
		unsigned char *p2;
		add_scriptc(C_STR);
		while(1) {
			p++;
			while(*p && *p!='"'){
				if(*p=='\n'){
					disp_error_message("unexpected newline @ string",p);
				}
				if(p[-1]<=0x7e && *p=='\\') {
					p++;
					if(set_control_code(*p)) {
						p++;
						continue;
					}
				}
				add_scriptb(*p++);
			}
			if(!*p){
				disp_error_message("unexpected eof @ string",p);
			}
			p2 = skip_space(p+1);
			if(*p2 != '"')
				break;
			p = p2;
		}
		add_scriptb(0);
		p++;	// '"'
	} else {
		int c,l;
		unsigned char *p2 = skip_word(p);
		// label , register , function etc
		if(p2 == p) {
			disp_error_message("unexpected character",p);
		}
		c = *p2;
		*p2 = 0;
		l = add_str(p);	// Oadd_str

		parse_cmd = l;	// warn_*_mismatch_paramnum̂߂ɕKv

		*p2 = c;
		p = p2;

		if(str_data[l].type == C_USERFUNC || str_data[l].type == C_USERFUNC_POS) {
			add_scriptl(search_str("callsub"));
			add_scriptc(C_ARG);
			add_scriptl(l);
		} else if(str_data[l].type != C_FUNC && c == '[') {
			// array( name[i][j] => getelementofarray(name,i,j) )
			add_scriptl(search_str("getelementofarray"));
			add_scriptc(C_ARG);
			add_scriptl(l);
			while(*p == '[') {
				p = parse_subexpr(p+1,-1);
				p = skip_space(p);
				if((*p++) != ']') {
					disp_error_message("unmatch ']'",p-1);
				}
			}
			add_scriptc(C_FUNC);
		} else {
			add_scriptl(l);
		}
	}

#ifdef DEBUG_FUNCIN
	if(battle_config.etc_log)
		printf("parse_simpleexpr end %s\n",p);
#endif
	return p;
}

/*==========================================
 * ̉
 *------------------------------------------
 */
static unsigned char* parse_subexpr(unsigned char *p,int limit)
{
	int op;

#ifdef DEBUG_FUNCIN
	if(battle_config.etc_log)
		printf("parse_subexpr %s\n", p);
#endif
	p = skip_space(p);

	if(*p == '-') {
		unsigned char *tmpp = skip_space(p+1);
		if(*tmpp == ';' || *tmpp == ',') {
			add_scriptl(LABEL_NEXTLINE);
			return p + 1;
		}
	}

	switch(*p) {
		case '-': op = C_NEG;  break;
		case '!': op = C_LNOT; break;
		case '~': op = C_NOT;  break;
		default:  op = C_NOP;  break;
	}
	if(op == C_NOP) {
		p = parse_simpleexpr(p);
	} else {
		p = parse_subexpr(p+1, 10);
		add_scriptc(op);
	}
	p = skip_space(p);

	while(1) {
		int priority = 0, len = 0;
		switch(*p) {
			case '(': op = C_FUNC; priority = 11; len = 1; break;
			case '*': op = C_MUL;  priority =  9; len = 1; break;
			case '/': op = C_DIV;  priority =  9; len = 1; break;
			case '%': op = C_MOD;  priority =  9; len = 1; break;
			case '+': op = C_ADD;  priority =  8; len = 1; break;
			case '-': op = C_SUB;  priority =  8; len = 1; break;
			case '^': op = C_XOR;  priority =  4; len = 1; break;
			case '?': op = C_OP3;  priority =  0; len = 1; break;
			case '&':
				if(p[1] == '&') {
					op = C_LAND; priority = 2; len = 2;
				} else {
					op = C_AND;  priority = 6; len = 1;
				}
				break;
			case '|':
				if(p[1] == '|') {
					op = C_LOR; priority = 1; len = 2;
				} else {
					op = C_OR;  priority = 5; len = 1;
				}
				break;
			case '=':
				if(p[1] == '=') {
					op = C_EQ; priority = 3; len = 2;
				} else {
					op = C_NOP;
				}
				break;
			case '!':
				if(p[1] == '=') {
					op = C_NE; priority = 3; len = 2;
				} else {
					op = C_NOP;
				}
				break;
			case '>':
				if(p[1] == '>') {
					op = C_R_SHIFT; priority = 7; len = 2;
				} else if(p[1] == '=') {
					op = C_GE; priority = 3; len = 2;
				} else {
					op = C_GT; priority = 3; len = 1;
				}
				break;
			case '<':
				if(p[1] == '<') {
					op = C_L_SHIFT; priority = 7; len = 2;
				} else if(p[1] == '=') {
					op = C_LE; priority = 3; len = 2;
				} else {
					op = C_LT; priority = 3; len = 1;
				}
				break;
			default:
				op = C_NOP;
				break;
		}
		if(op == C_NOP || priority <= limit)
			break;

		p += len;
		if(op == C_FUNC) {
			int i = 0, j = 0;
			char *plist[128];
			const char *arg = NULL;

			if(str_data[parse_cmd].type == C_FUNC) {
				// ʏ̊֐
				add_scriptc(C_ARG);
			} else if(str_data[parse_cmd].type == C_USERFUNC || str_data[parse_cmd].type == C_USERFUNC_POS) {
				// [U[`֐Ăяo
				parse_cmd = search_str("callsub");
				i++;
			} else {
				disp_error_message("expect command, missing function name or calling undeclared function", p);
			}
			p = skip_space(p);

			// ֐̎̐`FbN
			if(str_data[parse_cmd].type == C_FUNC && script_config.warn_cmd_mismatch_paramnum) {
				arg = buildin_func[str_data[parse_cmd].u.val].arg;
				for(j = 0; arg[j]; j++) {
					if(arg[j] == '*')
						break;
				}
			}

			while(*p && *p != ')' && i < 128) {
				plist[i] = p;
				p = parse_subexpr(p,-1);
				p = skip_space(p);
				// ؂,
				if(*p == ',') {
					if(arg == NULL || arg[j] == '*' || i+1 < j)	// ƂȂJ}XLbv
						p++;
				}
				else if(*p != ')' && script_config.warn_func_no_comma) {
					disp_error_message("expect ',' or ')' at func params", p);
				}
				p = skip_space(p);
				i++;
			}
			plist[i] = p;
			if(*(p++) != ')') {
				disp_error_message("func request '(' ')'",p-1);
			}
			if(arg) {
				if( (arg[j] == '\0' && i != j) || (arg[j] == '*' && i < j) )
					disp_error_message("illegal number of parameters", plist[(i < j)? i: j]);
			}
		} else if(op == C_OP3) {
			p = parse_subexpr(p,-1);
			p = skip_space(p);
			if(*(p++) != ':') {
				disp_error_message("need ':'", p-1);
			}
			p = parse_subexpr(p,-1);
		} else {
			p = parse_subexpr(p, priority);
		}
		add_scriptc(op);
		p = skip_space(p);
	}
#ifdef DEBUG_FUNCIN
	if(battle_config.etc_log)
		printf("parse_subexpr end %s\n",p);
#endif
	return p;
}

/*==========================================
 * ̕]
 *------------------------------------------
 */
static unsigned char* parse_expr(unsigned char *p)
{
#ifdef DEBUG_FUNCIN
	if(battle_config.etc_log)
		printf("parse_expr %s\n",p);
#endif
	switch(*p) {
		case ')':
		case ';':
		case ':':
		case '[':
		case ']':
		case '}':
			disp_error_message("unexpected char",p);
	}
	p=parse_subexpr(p,-1);
#ifdef DEBUG_FUNCIN
	if(battle_config.etc_log)
		printf("parse_expr end %s\n",p);
#endif
	return p;
}

/*==========================================
 * s̉
 *------------------------------------------
 */
static unsigned char* parse_line(unsigned char *p)
{
	int i = 0, j = 0;
	unsigned char *plist[128];
	unsigned char *p2;
	unsigned char end;
	const char *arg = NULL;

	p = skip_space(p);
	if(*p == ';') {
		// if(); for(); while(); ̂߂ɕ
		p = parse_syntax_close(p + 1);
		return p;
	}
	if(*p == ')' && parse_syntax_for_flag)
		return p + 1;

	p = skip_space(p);
	if(*p == '{') {
		syntax.curly[syntax.curly_count].type  = TYPE_NULL;
		syntax.curly[syntax.curly_count].count = -1;
		syntax.curly[syntax.curly_count].index = -1;
		syntax.curly_count++;
		return p + 1;
	} else if(*p == '}') {
		return parse_curly_close(p);
	}

	// \֘Ȁ
	p2 = parse_syntax(p);
	if(p2 != NULL)
		return p2;

	// ŏ͊֐
	p2 = p;
	p  = parse_simpleexpr(p);

	if(str_data[parse_cmd].type == C_FUNC) {
		// ʏ̊֐
		add_scriptc(C_ARG);
	} else if(str_data[parse_cmd].type == C_USERFUNC || str_data[parse_cmd].type == C_USERFUNC_POS) {
		// [U[`֐Ăяo
		parse_cmd = search_str("callsub");
		i++;
	} else {
		disp_error_message("expect command, missing function name or calling undeclared function", p2);
	}

	if(parse_syntax_for_flag) {
		end = ')';
	} else {
		end = ';';
	}

	// ֐̎̐`FbN
	if(str_data[parse_cmd].type == C_FUNC && script_config.warn_cmd_mismatch_paramnum) {
		arg = buildin_func[str_data[parse_cmd].u.val].arg;
		for(j = 0; arg[j]; j++) {
			if(arg[j] == '*')
				break;
		}
	}

	p2 = p;
	p  = skip_space(p);

	while(p && *p && *p != end && i < 128) {
		plist[i] = p;
		if(p2 == p) {
			disp_error_message("expect space ' '", p);
		}
		p = parse_expr(p);
		p = skip_space(p);
		// ؂,
		if(*p == ',') {
			if(arg == NULL || arg[j] == '*' || i+1 < j)	// ƂȂJ}XLbv
				p++;
		}
		else if(*p != end && script_config.warn_cmd_no_comma) {
			if(parse_syntax_for_flag) {
				disp_error_message("expect ',' or ')' at cmd params", p);
			} else {
				disp_error_message("expect ',' or ';' at cmd params", p);
			}
		}
		p = skip_space(p);
		i++;
	}
	plist[i] = p;
	if(!p || *p != end) {
		if(parse_syntax_for_flag) {
			disp_error_message("need ')'", p);
		} else {
			disp_error_message("need ';'", p);
		}
	}
	add_scriptc(C_FUNC);

	// if, for, while ̕
	p = parse_syntax_close(p + 1);

	if(arg) {
		if( (arg[j] == '\0' && i != j) || (arg[j] == '*' && i < j) )
			disp_error_message("illegal number of parameters", plist[(i < j)? i: j]);
	}
	return p;
}

/*==========================================
 * { ... } ̕
 *------------------------------------------
 */
static unsigned char* parse_curly_close(unsigned char *p)
{
	if(syntax.curly_count <= 0) {
		disp_error_message("unexpected string",p);
	}

	switch(syntax.curly[syntax.curly_count-1].type) {
	case TYPE_NULL:
		syntax.curly_count--;
		// if, for, while ̕
		p = parse_syntax_close(p + 1);
		break;
	case TYPE_SWITCH:
		{
			// switch() 
			int pos = syntax.curly_count - 1;
			char label[256];
			int l;

			// ꎞϐ
			sprintf(label,"set $@__SW%x_VAL,0;",syntax.curly[pos].index);
			syntax.curly[syntax.curly_count++].type = TYPE_NULL;
			parse_line(label);
			syntax.curly_count--;

			// ŏI|C^Ɉړ
			sprintf(label,"goto __SW%x_FIN;",syntax.curly[pos].index);
			syntax.curly[syntax.curly_count++].type = TYPE_NULL;
			parse_line(label);
			syntax.curly_count--;

			// ݒñxt
			sprintf(label,"__SW%x_%x",syntax.curly[pos].index,syntax.curly[pos].count);
			l=add_str(label);
			set_label(l,script_pos,p);

			if(syntax.curly[pos].flag) {
				// default ݂
				sprintf(label,"goto __SW%x_DEF;",syntax.curly[pos].index);
				syntax.curly[syntax.curly_count++].type = TYPE_NULL;
				parse_line(label);
				syntax.curly_count--;
			}

			// Ixt
			sprintf(label,"__SW%x_FIN",syntax.curly[pos].index);
			l=add_str(label);
			set_label(l,script_pos,p);

			linkdb_final(&syntax.curly[pos].case_label);	// casex̃Xg
			syntax.curly_count--;
			p++;
		}
		break;
	default:
		disp_error_message("unexpected string",p);
	}
	return p;
}

/*==========================================
 * \֘Ȁ
 *   break, case, continue, default, do, for,
 *   function, if, switch, while ̓ŏ
 *------------------------------------------
 */
static unsigned char* parse_syntax(unsigned char *p)
{
	unsigned char *p2;

	switch(*p) {
	case 'b':
	case 'B':
		p2 = skip_word(p);
		if(p2 - p == 5 && !strncasecmp(p,"break",5)) {
			// break ̏
			char label[256];
			int pos = syntax.curly_count - 1;

			while(pos >= 0) {
				if(syntax.curly[pos].type == TYPE_DO) {
					sprintf(label,"goto __DO%x_FIN;",syntax.curly[pos].index);
					break;
				} else if(syntax.curly[pos].type == TYPE_FOR) {
					sprintf(label,"goto __FR%x_FIN;",syntax.curly[pos].index);
					break;
				} else if(syntax.curly[pos].type == TYPE_WHILE) {
					sprintf(label,"goto __WL%x_FIN;",syntax.curly[pos].index);
					break;
				} else if(syntax.curly[pos].type == TYPE_SWITCH) {
					sprintf(label,"goto __SW%x_FIN;",syntax.curly[pos].index);
					break;
				}
				pos--;
			}
			if(pos < 0) {
				disp_error_message("unexpected 'break'",p);
			}
			syntax.curly[syntax.curly_count++].type = TYPE_NULL;
			parse_line(label);
			syntax.curly_count--;
			p = skip_space(p2);
			if(*p != ';') {
				disp_error_message("need ';'",p);
			}
			// if, for, while ̕
			p = parse_syntax_close(p + 1);
			return p;
		}
		break;
	case 'c':
	case 'C':
		p2 = skip_word(p);
		if(p2 - p == 4 && !strncasecmp(p,"case",4)) {
			// case ̏
			char *np;
			char label[256];
			int l,v;
			int pos = syntax.curly_count-1;

			if(pos < 0 || syntax.curly[pos].type != TYPE_SWITCH) {
				disp_error_message("unexpected 'case'",p);
			}
			if(syntax.curly[pos].count != 1) {
				// FALLTHRU p̃Wv
				sprintf(label,"goto __SW%x_%xJ;",syntax.curly[pos].index,syntax.curly[pos].count);
				syntax.curly[syntax.curly_count++].type = TYPE_NULL;
				parse_line(label);
				syntax.curly_count--;

				// ݒñxt
				sprintf(label,"__SW%x_%x",syntax.curly[pos].index,syntax.curly[pos].count);
				l=add_str(label);
				set_label(l,script_pos,p);
			}
			// switch 蕶
			p = skip_space(p2);
			if(p == p2) {
				disp_error_message("expect space ' '",p);
			}
			p2 = p;

			// casexl^萔ł邩`FbN
			v = strtol(p,&np,0);
			if((*p == '-' || *p == '+') && isdigit(p[1]))	// '-' skip_wordoȂ̂ł炩skipĂ
				p++;
			p = skip_word(p);
			if((unsigned char *)np == p2 || (unsigned char *)np != p) {	// lł͂Ȃ悤Ȃ̂Œ萔ƂČĂ݂
				int i;
				unsigned char c = *p;

				*p = 0;
				i = search_str(p2);
				*p = c;
				if(i < 0 || str_data[i].type != C_INT) {
					disp_error_message("'case' label not integer",p2);
				}
				v = str_data[i].u.val;
			}
			p = skip_space(p);
			if(*p != ':') {
				disp_error_message("expect ':'",p);
			}

			// casexdĂȂ`FbN
			if(linkdb_search(&syntax.curly[pos].case_label, (void*)v) != NULL) {
				disp_error_message("dup 'case'",p);
			}
			linkdb_insert(&syntax.curly[pos].case_label, (void*)v, (void*)1);

			sprintf(label,"if(%d != $@__SW%x_VAL) goto __SW%x_%x;",
				v,syntax.curly[pos].index,syntax.curly[pos].index,syntax.curly[pos].count+1);
			syntax.curly[syntax.curly_count++].type = TYPE_NULL;
			// Qparse Ȃƃ_
			p2 = parse_line(label);
			parse_line(p2);
			syntax.curly_count--;
			if(syntax.curly[pos].count != 1) {
				// FALLTHRU Ĩx
				sprintf(label,"__SW%x_%xJ",syntax.curly[pos].index,syntax.curly[pos].count);
				l=add_str(label);
				set_label(l,script_pos,p);
			}
			// ꎞϐ
			sprintf(label,"set $@__SW%x_VAL,0;",syntax.curly[pos].index);
			syntax.curly[syntax.curly_count++].type = TYPE_NULL;
			parse_line(label);
			syntax.curly_count--;
			syntax.curly[pos].count++;
			return p + 1;
		}
		else if(p2 - p == 8 && !strncasecmp(p,"continue",8)) {
			// continue ̏
			char label[256];
			int pos = syntax.curly_count - 1;

			while(pos >= 0) {
				if(syntax.curly[pos].type == TYPE_DO) {
					sprintf(label,"goto __DO%x_NXT;",syntax.curly[pos].index);
					syntax.curly[pos].flag = 1; // continue p̃NtO
					break;
				} else if(syntax.curly[pos].type == TYPE_FOR) {
					sprintf(label,"goto __FR%x_NXT;",syntax.curly[pos].index);
					break;
				} else if(syntax.curly[pos].type == TYPE_WHILE) {
					sprintf(label,"goto __WL%x_NXT;",syntax.curly[pos].index);
					break;
				}
				pos--;
			}
			if(pos < 0) {
				disp_error_message("unexpected 'continue'",p);
			}
			syntax.curly[syntax.curly_count++].type = TYPE_NULL;
			parse_line(label);
			syntax.curly_count--;
			p = skip_space(p2);
			if(*p != ';') {
				disp_error_message("need ';'",p);
			}
			// if, for, while ̕
			p = parse_syntax_close(p + 1);
			return p;
		}
		break;
	case 'd':
	case 'D':
		p2 = skip_word(p);
		if(p2 - p == 7 && !strncasecmp(p,"default",7)) {
			// switch - default ̏
			char label[256];
			int l;
			int pos = syntax.curly_count-1;

			if(pos < 0 || syntax.curly[pos].type != TYPE_SWITCH) {
				disp_error_message("unexpected 'default'",p);
			}
			if(syntax.curly[pos].flag) {
				disp_error_message("dup 'default'",p);
			}
			// ݒñxt
			p = skip_space(p2);
			if(*p != ':') {
				disp_error_message("need ':'",p);
			}
			sprintf(label,"__SW%x_%x",syntax.curly[pos].index,syntax.curly[pos].count);
			l=add_str(label);
			set_label(l,script_pos,p);

			// Ŏ̃Nɔ΂
			sprintf(label,"goto __SW%x_%x;",syntax.curly[pos].index,syntax.curly[pos].count+1);
			syntax.curly[syntax.curly_count++].type = TYPE_NULL;
			parse_line(label);
			syntax.curly_count--;

			// default ̃xt
			sprintf(label,"__SW%x_DEF",syntax.curly[pos].index);
			l=add_str(label);
			set_label(l,script_pos,p);

			syntax.curly[syntax.curly_count - 1].flag = 1;
			syntax.curly[pos].count++;
			return p + 1;
		}
		else if(p2 - p == 2 && !strncasecmp(p,"do",2)) {
			char label[256];
			int l;

			p=skip_space(p2);
			syntax.curly[syntax.curly_count].type  = TYPE_DO;
			syntax.curly[syntax.curly_count].count = 1;
			syntax.curly[syntax.curly_count].index = syntax.index++;
			syntax.curly[syntax.curly_count].flag  = 0;
			// ݒñx`
			sprintf(label,"__DO%x_BGN",syntax.curly[syntax.curly_count].index);
			l=add_str(label);
			set_label(l,script_pos,p);
			syntax.curly_count++;
			return p;
		}
		break;
	case 'f':
	case 'F':
		p2 = skip_word(p);
		if(p2 - p == 3 && !strncasecmp(p,"for",3)) {
			char label[256];
			int l;
			int pos = syntax.curly_count;

			syntax.curly[syntax.curly_count].type  = TYPE_FOR;
			syntax.curly[syntax.curly_count].count = 1;
			syntax.curly[syntax.curly_count].index = syntax.index++;
			syntax.curly[syntax.curly_count].flag  = 0;
			syntax.curly_count++;

			p=skip_space(p2);
			if(*p != '(') {
				disp_error_message("need '('",p);
			}
			p++;

			// s
			syntax.curly[syntax.curly_count++].type = TYPE_NULL;
			p=parse_line(p);
			syntax.curly_count--;

			// fJñx`
			sprintf(label,"__FR%x_J",syntax.curly[pos].index);
			l=add_str(label);
			set_label(l,script_pos,p);

			p=skip_space(p);
			if(*p == ';') {
				// for(;;) ̃p^[Ȃ̂ŕK^
				;
			} else {
				// UȂIn_ɔ΂
				sprintf(label,"__FR%x_FIN",syntax.curly[pos].index);
				add_scriptl(search_str("jump_zero"));
				add_scriptc(C_ARG);
				p=parse_expr(p);
				p=skip_space(p);
				add_scriptl(add_str(label));
				add_scriptc(C_FUNC);
			}
			if(*p != ';') {
				disp_error_message("need ';'",p);
			}
			p++;

			// [vJnɔ΂
			sprintf(label,"goto __FR%x_BGN;",syntax.curly[pos].index);
			syntax.curly[syntax.curly_count++].type = TYPE_NULL;
			parse_line(label);
			syntax.curly_count--;

			// ̃[vւ̃x`
			sprintf(label,"__FR%x_NXT",syntax.curly[pos].index);
			l=add_str(label);
			set_label(l,script_pos,p);

			// ̃[vɓ鎞̏
			// for Ō ')'  ';' ƂĈtO
			parse_syntax_for_flag = 1;
			syntax.curly[syntax.curly_count++].type = TYPE_NULL;
			p=parse_line(p);
			syntax.curly_count--;
			parse_syntax_for_flag = 0;

			// 菈ɔ΂
			sprintf(label,"goto __FR%x_J;",syntax.curly[pos].index);
			syntax.curly[syntax.curly_count++].type = TYPE_NULL;
			parse_line(label);
			syntax.curly_count--;

			// [vJñxt
			sprintf(label,"__FR%x_BGN",syntax.curly[pos].index);
			l=add_str(label);
			set_label(l,script_pos,p);
			return p;
		}
		else if(p2 - p == 8 && !strncasecmp(p,"function",8)) {
			unsigned char *func_name;

			// function
			p=skip_space(p2);
			if(p == p2) {
				disp_error_message("expect space ' '",p);
			}
			// function - name
			func_name = p;
			p=skip_word(p);
			p2=skip_space(p);
			if(*p2 == ';') {
				// ֐̐錾 - Oo^ďI
				unsigned char c = *p;
				int l;
				*p = 0;
				l=add_str(func_name);
				*p = c;
				if(str_data[l].type == C_INT || str_data[l].type == C_PARAM) {
					disp_error_message("invalid label name",p);
				}
				if(str_data[l].type == C_NOP)
					str_data[l].type = C_USERFUNC;

				// if, for, while ̕
				p = parse_syntax_close(p2 + 1);
			} else if(*p2 == '{') {
				// ֐̒g
				char label[256];
				unsigned char c = *p;
				int l;
				syntax.curly[syntax.curly_count].type  = TYPE_USERFUNC;
				syntax.curly[syntax.curly_count].count = 1;
				syntax.curly[syntax.curly_count].index = syntax.index++;
				syntax.curly[syntax.curly_count].flag  = 0;
				syntax.curly_count++;

				// ֐I܂Ŕ΂
				sprintf(label,"goto __FN%x_FIN;",syntax.curly[syntax.curly_count-1].index);
				syntax.curly[syntax.curly_count++].type = TYPE_NULL;
				parse_line(label);
				syntax.curly_count--;

				// ֐̃xt
				*p = 0;
				l=add_str(func_name);
				*p = c;
				if(str_data[l].type == C_INT || str_data[l].type == C_PARAM) {
					disp_error_message("invalid label name",p);
				}
				if(str_data[l].type == C_NOP)
					str_data[l].type = C_USERFUNC;
				set_label(l,script_pos,p);
				p = skip_space(p);
			} else {
				disp_error_message("expect ';' or '{' at function syntax",p);
			}
			return p;
		}
		break;
	case 'i':
	case 'I':
		p2 = skip_word(p);
		if(p2 - p == 2 && !strncasecmp(p,"if",2)) {
			// if() ̏
			char label[256];

			p=skip_space(p2);
			if(*p != '(') {
				disp_error_message("need '('",p);
			}
			syntax.curly[syntax.curly_count].type  = TYPE_IF;
			syntax.curly[syntax.curly_count].count = 1;
			syntax.curly[syntax.curly_count].index = syntax.index++;
			syntax.curly[syntax.curly_count].flag  = 0;
			sprintf(label,"__IF%x_%x",syntax.curly[syntax.curly_count].index,syntax.curly[syntax.curly_count].count);
			syntax.curly_count++;
			add_scriptl(search_str("jump_zero"));
			add_scriptc(C_ARG);
			p=parse_expr(p);
			p=skip_space(p);
			add_scriptl(add_str(label));
			add_scriptc(C_FUNC);
			return p;
		}
		break;
	case 's':
	case 'S':
		p2 = skip_word(p);
		if(p2 - p == 6 && !strncasecmp(p,"switch",6)) {
			// switch() ̏
			char label[256];

			p=skip_space(p2);
			if(*p != '(') {
				disp_error_message("need '('",p);
			}
			syntax.curly[syntax.curly_count].type  = TYPE_SWITCH;
			syntax.curly[syntax.curly_count].count = 1;
			syntax.curly[syntax.curly_count].index = syntax.index++;
			syntax.curly[syntax.curly_count].flag  = 0;
			sprintf(label,"$@__SW%x_VAL",syntax.curly[syntax.curly_count].index);
			syntax.curly_count++;
			add_scriptl(search_str("set"));
			add_scriptc(C_ARG);
			add_scriptl(add_str(label));
			p=parse_expr(p);
			p=skip_space(p);
			if(*p != '{') {
				disp_error_message("need '{'",p);
			}
			add_scriptc(C_FUNC);
			return p + 1;
		}
		break;
	case 'w':
	case 'W':
		p2 = skip_word(p);
		if(p2 - p == 5 && !strncasecmp(p,"while",5)) {
			char label[256];
			int l;

			p=skip_space(p2);
			if(*p != '(') {
				disp_error_message("need '('",p);
			}
			syntax.curly[syntax.curly_count].type  = TYPE_WHILE;
			syntax.curly[syntax.curly_count].count = 1;
			syntax.curly[syntax.curly_count].index = syntax.index++;
			syntax.curly[syntax.curly_count].flag  = 0;
			// fJñx`
			sprintf(label,"__WL%x_NXT",syntax.curly[syntax.curly_count].index);
			l=add_str(label);
			set_label(l,script_pos,p);

			// UȂIn_ɔ΂
			sprintf(label,"__WL%x_FIN",syntax.curly[syntax.curly_count].index);
			add_scriptl(search_str("jump_zero"));
			add_scriptc(C_ARG);
			p=parse_expr(p);
			p=skip_space(p);
			add_scriptl(add_str(label));
			add_scriptc(C_FUNC);
			syntax.curly_count++;
			return p;
		}
		break;
	}
	return NULL;
}

/*==========================================
 * if, for, while, do ̕
 *   flag == 1 : ꂽ
 *   flag == 0 : Ȃ
 *------------------------------------------
 */
static unsigned char* parse_syntax_close_sub(unsigned char *p,int *flag)
{
	char label[256];
	int l, pos;

	if(syntax.curly_count <= 0) {
		*flag = 0;
		return p;
	}
	pos = syntax.curly_count - 1;

	switch(syntax.curly[pos].type) {
	case TYPE_IF:
		{
			unsigned char *bp = p;
			unsigned char *p2;

			// if ŏIꏊ֔΂
			sprintf(label,"goto __IF%x_FIN;",syntax.curly[pos].index);
			syntax.curly[syntax.curly_count++].type = TYPE_NULL;
			parse_line(label);
			syntax.curly_count--;

			// ݒñxt
			sprintf(label,"__IF%x_%x",syntax.curly[pos].index,syntax.curly[pos].count);
			l=add_str(label);
			set_label(l,script_pos,p);

			syntax.curly[pos].count++;
			p = skip_space(p);
			p2 = skip_word(p);
			if(!syntax.curly[pos].flag && p2 - p == 4 && !strncasecmp(p,"else",4)) {
				// else  or else - if
				p = skip_space(p2);
				p2 = skip_word(p);
				if(p2 - p == 2 && !strncasecmp(p,"if",2)) {
					// else - if
					p=skip_space(p2);
					if(*p != '(') {
						disp_error_message("need '('",p);
					}
					sprintf(label,"__IF%x_%x",syntax.curly[pos].index,syntax.curly[pos].count);
					add_scriptl(search_str("jump_zero"));
					add_scriptc(C_ARG);
					p=parse_expr(p);
					p=skip_space(p);
					add_scriptl(add_str(label));
					add_scriptc(C_FUNC);
					*flag = 0;
					return p;
				} else {
					// else
					if(!syntax.curly[pos].flag) {
						syntax.curly[pos].flag = 1;
						*flag = 0;
						return p;
					}
				}
			}
			// ŏIñxt
			sprintf(label,"__IF%x_FIN",syntax.curly[pos].index);
			l=add_str(label);
			set_label(l,script_pos,p);
			if(syntax.curly[pos].flag == 1) {
				// ifɑ΂elseȂ̂Ń|C^̈ʒu͓
				p = bp;
			}
			syntax.curly_count--;
			*flag = 1;
		}
		break;
	case TYPE_DO:
		{
			unsigned char *p2;

			if(syntax.curly[pos].flag) {
				// ݒñx`(continue łɗ)
				sprintf(label,"__DO%x_NXT",syntax.curly[pos].index);
				l=add_str(label);
				set_label(l,script_pos,p);
			}

			// UȂIn_ɔ΂
			p = skip_space(p);
			p2 = skip_word(p);
			if(p2 - p != 5 || strncasecmp(p,"while",5)) {
				disp_error_message("need 'while'",p);
			}

			p = skip_space(p2);
			if(*p != '(') {
				disp_error_message("need '('",p);
			}
			sprintf(label,"__DO%x_FIN",syntax.curly[pos].index);
			add_scriptl(search_str("jump_zero"));
			add_scriptc(C_ARG);
			p=parse_expr(p);
			p=skip_space(p);
			add_scriptl(add_str(label));
			add_scriptc(C_FUNC);

			// Jnn_ɔ΂
			sprintf(label,"goto __DO%x_BGN;",syntax.curly[pos].index);
			syntax.curly[syntax.curly_count++].type = TYPE_NULL;
			parse_line(label);
			syntax.curly_count--;

			// In_̃x`
			sprintf(label,"__DO%x_FIN",syntax.curly[pos].index);
			l=add_str(label);
			set_label(l,script_pos,p);
			if(*p != ';') {
				disp_error_message("need ';'",p);
			}
			p++;
			syntax.curly_count--;
			*flag = 1;
		}
		break;
	case TYPE_FOR:
		// ̃[vɔ΂
		sprintf(label,"goto __FR%x_NXT;",syntax.curly[pos].index);
		syntax.curly[syntax.curly_count++].type = TYPE_NULL;
		parse_line(label);
		syntax.curly_count--;

		// for Ĩxt
		sprintf(label,"__FR%x_FIN",syntax.curly[pos].index);
		l=add_str(label);
		set_label(l,script_pos,p);
		syntax.curly_count--;
		*flag = 1;
		break;
	case TYPE_WHILE:
		// while f֔΂
		sprintf(label,"goto __WL%x_NXT;",syntax.curly[pos].index);
		syntax.curly[syntax.curly_count++].type = TYPE_NULL;
		parse_line(label);
		syntax.curly_count--;

		// while Ĩxt
		sprintf(label,"__WL%x_FIN",syntax.curly[pos].index);
		l=add_str(label);
		set_label(l,script_pos,p);
		syntax.curly_count--;
		*flag = 1;
		break;
	case TYPE_USERFUNC:
		// ߂
		add_scriptl(search_str("return"));
		add_scriptc(C_ARG);
		add_scriptc(C_FUNC);

		// ݒñxt
		sprintf(label,"__FN%x_FIN",syntax.curly[pos].index);
		l=add_str(label);
		set_label(l,script_pos,p);
		syntax.curly_count--;
		*flag = 1;
		break;
	default:
		*flag = 0;
		break;
	}
	return p;
}

/*==========================================
 * if(...) for(...) hoge(); ̂悤
 * Pxꂽēx邩mF
 *------------------------------------------
 */
static unsigned char* parse_syntax_close(unsigned char *p)
{
	int flag;

	do {
		p = parse_syntax_close_sub(p,&flag);
	} while(flag);

	return p;
}

/*==========================================
 * gݍ݊֐̒ǉ
 *------------------------------------------
 */
static void add_buildin_func(void)
{
	int i,n;

	for(i=0;buildin_func[i].func;i++){
		n=add_str(buildin_func[i].name);
		str_data[n].type=C_FUNC;
		str_data[n].u.val=i;
		str_data[n].func=buildin_func[i].func;
	}
}

/*==========================================
 * 萔f[^x[X̓ǂݍ
 *------------------------------------------
 */
static void read_constdb(void)
{
	FILE *fp;
	char line[1024], name[1024], val[1024];
	int n, type;

	fp = fopen("db/const.txt","r");
	if(fp == NULL) {
		printf("can't read db/const.txt\n");
		return;
	}
	while(fgets(line,1020,fp)) {
		if(line[0] == '\0' || line[0] == '\r' || line[0] == '\n')
			continue;
		if(line[0] == '/' && line[1] == '/')
			continue;
		type = 0;
		if(sscanf(line, "%[A-Za-z0-9_],%[-0-9xXA-Fa-f],%d", name, val, &type) >= 2 ||
		   sscanf(line, "%[A-Za-z0-9_] %[-0-9xXA-Fa-f] %d", name, val, &type) >= 2)
		{
			n = add_str(name);
			str_data[n].type  = (type == 0)? C_INT: C_PARAM;
			str_data[n].u.val = (int)strtol(val,NULL,0);
		} else {
			if(sscanf(line, "%[A-Za-z0-9_]$,%[^\r\n]", name, val) == 2 ||
			   sscanf(line, "%[A-Za-z0-9_]$ %[^\r\n]", name, val) == 2)
			{
				n = add_str(strcat(name, "$"));
				str_data[n].type  = C_STR;
				str_data[n].u.str = (char *)aStrdup(val);
			}
		}
	}
	fclose(fp);
}

/*==========================================
 * G[\
 *------------------------------------------
 */
static const char* script_print_line(const char *p, const char *mark, int line)
{
	int i;

	if(p == NULL || !p[0])
		return NULL;

	if(line < 0)
		printf("*% 5d : ", -line);
	else
		printf(" % 5d : ", line);

	for(i = 0; p[i] && p[i] != '\n'; i++) {
		if(p + i != mark)
			printf("%c", p[i]);
		else
			printf("%s%c%s", error_marker_start, p[i], error_marker_end);
	}
	printf("\n");

	return p + i + (p[i] == '\n' ? 1 : 0);
}

void script_error(const char *src, const char *file, int start_line, const char *msg, const char *pos)
{
	const char *linestart[5] = { NULL, NULL, NULL, NULL, NULL };
	const char *p;
	int j, line = start_line;

	// G[s߂
	for(p = src; p && *p; line++) {
		const char *lineend = strchr(p, '\n');
		if(lineend == NULL || pos < lineend) {
			break;
		}
		for(j = 0; j < 4; j++) {
			linestart[j] = linestart[j+1];
		}
		linestart[4] = p;
		p = lineend + 1;
	}

	printf("\a\n");
	printf("script error on %s line %d\n", file, line);
	printf("    %s\n", msg);

	for(j = 0; j < 5; j++) {
		script_print_line(linestart[j], NULL, line + j - 5);
	}
	p = script_print_line(p, pos, -line);
	for(j = 0; j < 5; j++) {
		p = script_print_line(p, NULL, line + j + 1);
	}
}

/*==========================================
 * XNvg̉
 *------------------------------------------
 */
struct script_code* parse_script(unsigned char *src,const char *file,int line)
{
	unsigned char *p,*tmpp;
	int i;
	struct script_code *code;
	static int first=1;

	memset(&syntax,0,sizeof(syntax));
	if(first){
		add_buildin_func();
		read_constdb();
	}
	first=0;

	p=src;
	p=skip_space(p);
	if(*p!='{'){
		script_error(src,file,line,"not found '{'",p);
		return &error_code;
	}
	p++;
	p=skip_space(p);
	if(p && *p == '}') {
		// R[hȂXNvg͐
		return NULL;
	}

	script_buf=(unsigned char *)aMalloc(SCRIPT_BLOCK_SIZE*sizeof(unsigned char));
	script_pos=0;
	script_size=SCRIPT_BLOCK_SIZE;
	str_data[LABEL_NEXTLINE].type=C_NOP;
	str_data[LABEL_NEXTLINE].backpatch=-1;
	str_data[LABEL_NEXTLINE].label=-1;
	for(i=LABEL_START;i<str_num;i++){
		if(
			str_data[i].type==C_POS || str_data[i].type==C_NAME ||
			str_data[i].type==C_USERFUNC || str_data[i].type == C_USERFUNC_POS
		){
			str_data[i].type=C_NOP;
			str_data[i].backpatch=-1;
			str_data[i].label=-1;
		}
	}

	// Oplabel db̏
	linkdb_final(&scriptlabel_db);

	// O
	if( setjmp( error_jump ) != 0 ) {
		int j;
		script_error(src,file,line,error_msg,error_pos);
		// n
		aFree( error_msg );
		aFree( script_buf );
		script_pos  = 0;
		script_size = 0;
		script_buf  = NULL;
		for(j=LABEL_START; j<str_num; j++) {
			if(str_data[j].type == C_NOP)
				str_data[j].type = C_NAME;
		}
		for(j=0; j<sizeof(syntax.curly)/sizeof(syntax.curly[0]); j++) {
			linkdb_final(&syntax.curly[j].case_label);
		}
		linkdb_final(&scriptlabel_db);
		return &error_code;
	}

	while (p && *p && (*p!='}' || syntax.curly_count != 0)) {
		p=skip_space(p);
		// labelꏈ
		tmpp=skip_space(skip_word(p));
		if(*tmpp==':' && !(!strncasecmp(p,"default:",8) && p + 7 == tmpp)){
			int l,c;
			unsigned char *p2 = skip_word(p);

			c = *p2;
			*p2 = 0;
			if(*p == 0) {
				*p2 = c;
				disp_error_message("label length 0",p);
			}
			l=add_str(p);
			*p2 = c;
			if(str_data[l].type == C_INT || str_data[l].type == C_PARAM) {
				disp_error_message("invalid label name",p);
			}
			set_label(l,script_pos,p);

			// GNX|[g郉xȂ`FbN
			*p2 = 0;
			if((p[0] == 'O' || p[0] == 'o') && (p[1] == 'N' || p[1] == 'n')) {
				for(i=2; p[i]; i++) {
					if(i >= 23 || (!isalnum(p[i]) && p[i] != '_')) {
						*p2 = c;
						disp_error_message("invalid label name",p);
					}
				}
				linkdb_insert(&scriptlabel_db,p,(void*)script_pos);	// Oplabel dbo^
			}
			*p2 = c;
			p=tmpp+1;
			continue;
		}

		// ͑Sꏏ
		p=parse_line(p);
		p=skip_space(p);
		add_scriptc(C_EOL);

		set_label(LABEL_NEXTLINE,script_pos,p);
		str_data[LABEL_NEXTLINE].type=C_NOP;
		str_data[LABEL_NEXTLINE].backpatch=-1;
		str_data[LABEL_NEXTLINE].label=-1;
	}

	add_scriptc(C_NOP);
	script_size = script_pos;

	// ̃x
	for(i=LABEL_START;i<str_num;i++){
		if(str_data[i].type==C_NOP){
			int j,next;
			str_data[i].type=C_NAME;
			str_data[i].label=i;

			if(vars_db) {
				if( str_data[i].backpatch != -1 ) {
					struct vars_info *v = (struct vars_info *)numdb_search(vars_db, i);
					if( v == NULL ) {
						v            = (struct vars_info *)aMalloc( sizeof( struct vars_info ) );
						v->file      = (char *)aStrdup( file );
						v->line      = line;
						v->use_count = 1;
						numdb_insert(vars_db, i, v);
						vars_count++;
					} else {
						v->use_count++;
					}
				}
			}
			for(j=str_data[i].backpatch;j>=0 && j!=0x00ffffff;){
				next=(*(int*)(script_buf+j)) & 0x00ffffff;
				script_buf[j]=i;
				script_buf[j+1]=i>>8;
				script_buf[j+2]=i>>16;
				j=next;
			}
		}
	}

#ifdef DEBUG_DISP
	for(i=0;i<script_pos;i++){
		if((i&15)==0) printf("%04x : ",i);
		printf("%02x ",script_buf[i]);
		if((i&15)==15) printf("\n");
	}
	printf("\n");
#endif

#ifdef DEBUG_DISASM
	printf("------------------------------\n");
	i = 0;
	while(i < script_pos) {
		int j = i;
		printf("%06x ",i);

		switch(get_com(script_buf,&i)) {
		case C_EOL:	 printf("C_EOL"); break;
		case C_INT:	 printf("C_INT %d",get_num(script_buf,&i)); break;
		case C_POS:
			printf("C_POS 0x%06x",*(int*)(script_buf+i)&0xffffff);
			i += 3;
			break;
		case C_NAME:
			j = (*(int*)(script_buf+i)&0xffffff);
			printf("C_NAME %s",j == 0xffffff ? "?? unknown ??" : str_buf + str_data[j].str);
			i += 3;
			break;
		case C_ARG:     printf("C_ARG");     break;
		case C_FUNC:    printf("C_FUNC");    break;
		case C_ADD:     printf("C_ADD");     break;
		case C_SUB:     printf("C_SUB");     break;
		case C_MUL:     printf("C_MUL");     break;
		case C_DIV:     printf("C_DIV");     break;
		case C_MOD:     printf("C_MOD");     break;
		case C_EQ:      printf("C_EQ");      break;
		case C_NE:      printf("C_NE");      break;
		case C_GT:      printf("C_GT");      break;
		case C_GE:      printf("C_GE");      break;
		case C_LT:      printf("C_LT");      break;
		case C_LE:      printf("C_LE");      break;
		case C_AND:     printf("C_AND");     break;
		case C_OR:      printf("C_OR");      break;
		case C_XOR:     printf("C_XOR");     break;
		case C_LAND:    printf("C_LAND");    break;
		case C_LOR:     printf("C_LOR");     break;
		case C_R_SHIFT: printf("C_R_SHIFT"); break;
		case C_L_SHIFT: printf("C_L_SHIFT"); break;
		case C_NEG:     printf("C_NEG");     break;
		case C_NOT:     printf("C_NOT");     break;
		case C_LNOT:    printf("C_LNOT");    break;
		case C_NOP:     printf("C_NOP");     break;
		case C_OP3:     printf("C_OP3");     break;
		case C_STR:
			j = strlen(script_buf + i);
			printf("C_STR %s",script_buf + i);
			i += j+1;
			break;
		default:
			printf("unknown");
		}
		printf("\n");
	}
	printf("\n");
#endif

	code = (struct script_code *)aMalloc(sizeof(struct script_code) + script_size);
	code->script_vars = NULL;
	code->script_size = script_size;
	memcpy(code->script_buf, script_buf, script_size);
	aFree(script_buf);

	return code;
}

//
// sn
//
enum {
	RUN = 0,
	STOP,
	END,
	RERUNLINE,
	GOTO,
	RETFUNC
};

/*==========================================
 * ridsdւ̉
 *------------------------------------------
 */
static struct map_session_data *script_rid2sd(struct script_state *st)
{
	struct map_session_data *sd=map_id2sd(st->rid);

	if(!sd){
		printf("script_rid2sd: fatal error ! player not attached!\n");
	}
	return sd;
}

/*==========================================
 * ϐ̓ǂݎ
 *------------------------------------------
 */
static void get_val(struct script_state *st,struct script_data *data)
{
	struct map_session_data *sd = NULL;
	char *name;
	char prefix, postfix;

	if(data->type != C_NAME)
		return;

	name    = str_buf+str_data[data->u.num&0x00ffffff].str;
	prefix  = *name;
	postfix = name[strlen(name)-1];

	if(postfix == '$') {
		// ^
		data->type = C_CONSTSTR;
		if(str_data[data->u.num&0x00ffffff].type == C_STR) {
			data->u.str = str_data[data->u.num&0x00ffffff].u.str;
		} else {
			switch(prefix) {
			case '@':
				if((sd = script_rid2sd(st)) == NULL) {
					printf("get_val error name?: %s\n", name);
					data->u.str = "";
				} else {
					data->u.str = pc_readregstr(sd,data->u.num);
				}
				break;
			case '$':
				data->u.str = (char *)numdb_search(mapregstr_db,data->u.num);
				break;
			case '\'':
				{
					struct linkdb_node **n;
					if( data->ref ) {
						n = data->ref;
					} else if( name[1] == '@' ) {
						n = st->stack->var_function;
					} else {
						n = &st->script->script_vars;
					}
					data->u.str = (char *)linkdb_search(n, (void*)data->u.num);
				}
				break;
			default:
				printf("script: get_val: illegal scope string variable %s.\n", name);
				data->u.str = "!!ERROR!!";
				break;
			}
		}
		if(data->u.str == NULL) {
			data->u.str = "";
		}
	} else {
		// l^
		data->type = C_INT;
		if(str_data[data->u.num&0x00ffffff].type == C_INT) {
			data->u.num = str_data[data->u.num&0x00ffffff].u.val;
		} else if(str_data[data->u.num&0x00ffffff].type == C_PARAM) {
			if((sd = script_rid2sd(st)) == NULL) {
				printf("get_val error name?: %s\n", name);
				data->u.num = 0;
			} else {
				data->u.num = pc_readparam(sd,str_data[data->u.num&0x00ffffff].u.val);
			}
		} else {
			switch(prefix) {
			case '@':
				if((sd = script_rid2sd(st)) == NULL) {
					printf("get_val error name?: %s\n", name);
					data->u.num = 0;
				} else {
					data->u.num = pc_readreg(sd,data->u.num);
				}
				break;
			case '$':
				data->u.num = (int)numdb_search(mapreg_db,data->u.num);
				break;
			case '#':
				if((sd = script_rid2sd(st)) == NULL) {
					printf("get_val error name?: %s\n", name);
					data->u.num = 0;
				} else {
					if(name[1] == '#')
						data->u.num = pc_readaccountreg2(sd,name);
					else
						data->u.num = pc_readaccountreg(sd,name);
				}
				break;
			case '\'':
				{
					struct linkdb_node **n;
					if( data->ref ) {
						n = data->ref;
					} else if( name[1] == '@' ) {
						n = st->stack->var_function;
					} else {
						n = &st->script->script_vars;
					}
					data->u.num = (int)linkdb_search(n, (void*)data->u.num);
				}
				break;
			default:
				if((sd = script_rid2sd(st)) == NULL) {
					printf("get_val error name?: %s\n", name);
					data->u.num = 0;
				} else {
					data->u.num = pc_readglobalreg(sd,name);
				}
				break;
			}
		}
	}

	return;
}

/*==========================================
 * ϐ̓ǂݎ2
 *------------------------------------------
 */
static void* get_val2(struct script_state *st,int num,struct linkdb_node **ref)
{
	struct script_data dat;

	dat.type  = C_NAME;
	dat.u.num = num;
	dat.ref   = ref;
	get_val(st,&dat);

	return (dat.type == C_INT)? (void*)dat.u.num: (void*)dat.u.str;
}

/*==========================================
 * ϐݒp
 *------------------------------------------
 */
static int set_reg(struct script_state *st,struct map_session_data *sd,int num,const char *name,const void *v,struct linkdb_node** ref)
{
	char prefix  = *name;
	char postfix = name[strlen(name)-1];

	if(postfix == '$') {
		// ^
		const char *str = (const char *)v;
		switch(prefix) {
		case '@':
			pc_setregstr(sd,num,str);
			break;
		case '$':
			mapreg_setregstr(num,str,(name[1] == '@')? 0: 1);
			break;
		case '\'':
			{
				char *old_str;
				struct linkdb_node **n;
				if( ref ) {
					n = ref;
				} else if( name[1] == '@' ) {
					n = st->stack->var_function;
				} else {
					n = &st->script->script_vars;
				}
				if( str[0] ) {
					old_str = (char *)linkdb_replace(n, (void*)num, aStrdup(str));
				} else {
					old_str = (char *)linkdb_erase(n, (void*)num);
				}
				if(old_str)
					aFree(old_str);
			}
			break;
		default:
			printf("script: set_reg: illegal scope string variable !");
			break;
		}
	} else {
		// l^
		int val = (int)v;
		if(str_data[num&0x00ffffff].type == C_PARAM) {
			pc_setparam(sd,str_data[num&0x00ffffff].u.val,val);
		} else {
			switch(prefix) {
			case '@':
				pc_setreg(sd,num,val);
				break;
			case '$':
				mapreg_setreg(num,val,(name[1] == '@')? 0: 1);
				break;
			case '#':
				if(name[1] == '#')
					pc_setaccountreg2(sd,name,val);
				else
					pc_setaccountreg(sd,name,val);
				break;
			case '\'':
				{
					struct linkdb_node **n;
					if( ref ) {
						n = ref;
					} else if( name[1] == '@' ) {
						n = st->stack->var_function;
					} else {
						n = &st->script->script_vars;
					}
					if( val != 0 ) {
						linkdb_replace(n, (void*)num, (void*)val);
					} else {
						linkdb_erase(n, (void*)num);
					}
				}
				break;
			default:
				pc_setglobalreg(sd,name,val);
				break;
			}
		}
	}
	return 0;
}

/*==========================================
 * ւ̕ϊ
 *------------------------------------------
 */
static char* conv_str(struct script_state *st,struct script_data *data)
{
	get_val(st,data);

	switch(data->type) {
	case C_INT:
		{
			char *buf = (char *)aCalloc(16, sizeof(char));
			sprintf(buf, "%d", data->u.num);
			data->type  = C_STR;
			data->u.str = buf;
		}
		break;
	case C_POS:
		// XNvg̃oOixɓnꍇj
		// : mes -;
		data->type  = C_CONSTSTR;
		data->u.str = "** SCRIPT ERROR **";
		printf("script: conv_str: label has used as argument !\n");
		break;
	case C_NAME:
		// e|B{͂
		data->type  = C_CONSTSTR;
		data->u.str = str_buf+str_data[data->u.num].str;
		break;
	}
	return data->u.str;
}

/*==========================================
 * l֕ϊ
 *------------------------------------------
 */
static int conv_num(struct script_state *st,struct script_data *data)
{
	get_val(st,data);
	if(data->type == C_STR || data->type == C_CONSTSTR) {
		char *p = data->u.str;
		data->u.num = atoi(p);
		if(data->type == C_STR) {
			aFree(p);
		}
		data->type = C_INT;
	}
	return data->u.num;
}

/*==========================================
 * X^bN̊g
 *------------------------------------------
 */
static void expand_stack(struct script_stack *stack)
{
	stack->sp_max += 64;
	stack->stack_data = (struct script_data *)aRealloc(stack->stack_data, sizeof(stack->stack_data[0]) * stack->sp_max);
}

/*==========================================
 * X^bN֐lvbV
 *------------------------------------------
 */
static void push_val(struct script_stack *stack,int type,int val)
{
	if(stack->sp >= stack->sp_max)
		expand_stack(stack);

	stack->stack_data[stack->sp].type  = type;
	stack->stack_data[stack->sp].u.num = val;
	stack->stack_data[stack->sp].ref   = NULL;
	stack->sp++;
}

/*==========================================
 * X^bN֐l{t@XvbV
 *------------------------------------------
 */
static void push_val2(struct script_stack *stack,int type,int val,struct linkdb_node** ref)
{
	push_val(stack,type,val);
	stack->stack_data[stack->sp-1].ref = ref;
}

/*==========================================
 * X^bN֕vbV
 *------------------------------------------
 */
static void push_str(struct script_stack *stack,int type,unsigned char *str)
{
	if(stack->sp >= stack->sp_max)
		expand_stack(stack);

	stack->stack_data[stack->sp].type  = type;
	stack->stack_data[stack->sp].u.str = str;
	stack->stack_data[stack->sp].ref   = NULL;
	stack->sp++;
}

/*==========================================
 * X^bNփ|C^vbV
 *------------------------------------------
 */
static void push_ptr(struct script_stack *stack,int type,void *ptr)
{
	if(stack->sp >= stack->sp_max)
		expand_stack(stack);

	stack->stack_data[stack->sp].type  = type;
	stack->stack_data[stack->sp].u.ptr = ptr;
	stack->stack_data[stack->sp].ref   = NULL;
	stack->sp++;
}

/*==========================================
 * X^bN֕vbV
 *------------------------------------------
 */
static void push_copy(struct script_stack *stack,int pos)
{
	switch(stack->stack_data[pos].type){
	case C_CONSTSTR:
		push_str(stack,C_CONSTSTR,stack->stack_data[pos].u.str);
		break;
	case C_STR:
		push_str(stack,C_STR,(unsigned char *)aStrdup(stack->stack_data[pos].u.str));
		break;
	case C_PTR:
		push_ptr(stack,C_PTR,stack->stack_data[pos].u.ptr);
		break;
	default:
		push_val2(
			stack,stack->stack_data[pos].type,stack->stack_data[pos].u.num,
			stack->stack_data[pos].ref
		);
		break;
	}
}

/*==========================================
 * X^bN|bv
 *------------------------------------------
 */
static void pop_stack(struct script_stack* stack,int start,int end)
{
	int i;

	for(i=start;i<end;i++){
		if(stack->stack_data[i].type==C_STR){
			aFree(stack->stack_data[i].u.str);
		}
	}
	if(stack->sp>end){
		memmove(&stack->stack_data[start],&stack->stack_data[end],sizeof(stack->stack_data[0])*(stack->sp-end));
	}
	stack->sp-=end-start;
}

/*==========================================
 * XNvgˑϐA֐ˑϐ̉
 *------------------------------------------
 */
static void script_free_vars(struct linkdb_node **node)
{
	struct linkdb_node *n = *node;

	while(n) {
		char *name   = str_buf + str_data[(int)(n->key)&0x00ffffff].str;
		char postfix = name[strlen(name)-1];
		if( postfix == '$' ) {
			// ^ϐȂ̂ŁAf[^폜
			aFree(n->data);
		}
		n = n->next;
	}
	linkdb_final( node );
}

/*==========================================
 * X^bN̉
 *------------------------------------------
 */
void script_free_stack(struct script_stack *stack)
{
	int i;

	for(i = 0; i < stack->sp; i++) {
		if( stack->stack_data[i].type == C_STR ) {
			aFree( stack->stack_data[i].u.str );
		} else if( i > 0 && stack->stack_data[i].type == C_RETINFO ) {
			struct linkdb_node** n = (struct linkdb_node**)stack->stack_data[i-1].u.ptr;
			script_free_vars( n );
			aFree( n );
		}
	}
	script_free_vars( stack->var_function );
	aFree(stack->var_function);
	aFree(stack->stack_data);
	aFree(stack);
}

/*==========================================
 * XNvgR[h̉
 *------------------------------------------
 */
void script_free_code(struct script_code* code)
{
	script_free_vars( &code->script_vars );
	aFree( code );
}

//
// smain
//
/*==========================================
 * R}h̓ǂݎ
 *------------------------------------------
 */
static int unget_com_data = -1;

static int get_com(unsigned char *script,int *pos)
{
	int i,j;

	if(unget_com_data>=0){
		i=unget_com_data;
		unget_com_data=-1;
		return i;
	}
	if(script[*pos]>=0x80){
		return C_INT;
	}
	i=0; j=0;
	while(script[*pos]>=0x40){
		i=script[(*pos)++]<<j;
		j+=6;
	}
	return i+(script[(*pos)++]<<j);
}

/*==========================================
 * R}h̃vbVobNigpj
 *------------------------------------------
 */
/*
static void unget_com(int c)
{
	if(unget_com_data!=-1){
		if(battle_config.error_log)
			printf("unget_com can back only 1 data\n");
	}
	unget_com_data=c;
}
*/

/*==========================================
 * l̎擾
 *------------------------------------------
 */
static int get_num(unsigned char *script,int *pos)
{
	int i=0,j=0;

	while(script[*pos]>=0xc0){
		i+=(script[(*pos)++]&0x7f)<<j;
		j+=6;
	}
	return i+((script[(*pos)++]&0x7f)<<j);
}

/*==========================================
 * X^bNlo
 *------------------------------------------
 */
static int pop_val(struct script_state* st)
{
	if(st->stack->sp<=0)
		return 0;
	st->stack->sp--;
	get_val(st,&(st->stack->stack_data[st->stack->sp]));
	if(st->stack->stack_data[st->stack->sp].type==C_INT)
		return st->stack->stack_data[st->stack->sp].u.num;
	return 0;
}

/*==========================================
 * ^ǂ
 *------------------------------------------
 */
static int isstr(struct script_data *c)
{
	if( c->type == C_STR || c->type == C_CONSTSTR )
		return 1;
	if( c->type == C_NAME ) {
		char *p = str_buf + str_data[c->u.num & 0xffffff].str;
		char postfix = p[strlen(p)-1];
		return (postfix == '$');
	}
	return 0;
}

/*==========================================
 * ZZq
 *------------------------------------------
 */
static void op_add(struct script_state* st)
{
	struct script_data *s1, *s2;

	st->stack->sp--;
	s1 = &st->stack->stack_data[st->stack->sp-1];
	s2 = &st->stack->stack_data[st->stack->sp];

	get_val(st,s1);
	get_val(st,s2);

	if(isstr(s1) || isstr(s2)) {
		conv_str(st,s1);
		conv_str(st,s2);
	}
	if(s2->type == C_INT) { // ii
		int *i1 = &s1->u.num;
		int *i2 = &s2->u.num;
		int ret = *i1 + *i2;
		atn_bignumber ret_bignum = (atn_bignumber)*i1 + (atn_bignumber)*i2;
		if(ret_bignum > 0x7FFFFFFF || ret_bignum < -1 * 0x7FFFFFFF) {
			printf("script::op_add overflow detected op:%d rid:%d\n",C_ADD,st->rid);
			ret = (ret_bignum > 0x7FFFFFFF ? 0x7FFFFFFF : -1 * 0x7FFFFFFF);
		}
		*i1 = ret;
	} else { // ss̗\
		char *buf = (char *)aCalloc(strlen(s1->u.str) + strlen(s2->u.str) + 1, sizeof(char));
		strcpy(buf,s1->u.str);
		strcat(buf,s2->u.str);
		if(s1->type == C_STR)
			aFree(s1->u.str);
		if(s2->type == C_STR)
			aFree(s2->u.str);
		s1->type  = C_STR;
		s1->u.str = buf;
	}
	s1->ref = NULL;
}

/*==========================================
 * OZq
 *------------------------------------------
 */
static void op_3(struct script_state *st)
{
	int flag = 0;

	if( isstr(&st->stack->stack_data[st->stack->sp-3])) {
		char *str = conv_str(st,& (st->stack->stack_data[st->stack->sp-3]));
		flag = str[0];
	} else {
		flag = conv_num(st,& (st->stack->stack_data[st->stack->sp-3]));
	}
	if( flag ) {
		push_copy(st->stack, st->stack->sp-2 );
	} else {
		push_copy(st->stack, st->stack->sp-1 );
	}
	pop_stack(st->stack,st->stack->sp-4,st->stack->sp-1);
}

/*==========================================
 * 񍀉Zq()
 *------------------------------------------
 */
static void op_2str(struct script_state *st,int op,int sp1,int sp2)
{
	char *s1=st->stack->stack_data[sp1].u.str;
	char *s2=st->stack->stack_data[sp2].u.str;
	int a=0;

	switch(op){
	case C_EQ:
		a= (strcmp(s1,s2)==0);
		break;
	case C_NE:
		a= (strcmp(s1,s2)!=0);
		break;
	case C_GT:
		a= (strcmp(s1,s2)> 0);
		break;
	case C_GE:
		a= (strcmp(s1,s2)>=0);
		break;
	case C_LT:
		a= (strcmp(s1,s2)< 0);
		break;
	case C_LE:
		a= (strcmp(s1,s2)<=0);
		break;
	default:
		printf("Illegal string operator\n");
		break;
	}
	if(st->stack->stack_data[sp1].type==C_STR)
		aFree(s1);
	if(st->stack->stack_data[sp2].type==C_STR)
		aFree(s2);

	push_val(st->stack,C_INT,a);
}

/*==========================================
 * 񍀉Zq(l)
 *------------------------------------------
 */
static void op_2num(struct script_state *st,int op,int i1,int i2)
{
	int ret = 0;
	atn_bignumber ret_bignum = 0;

	switch(op) {
	case C_MOD:     ret = i1 % i2;    break;
	case C_AND:     ret = i1 & i2;    break;
	case C_OR:      ret = i1 | i2;    break;
	case C_XOR:     ret = i1 ^ i2;    break;
	case C_LAND:    ret = (i1 && i2); break;
	case C_LOR:     ret = (i1 || i2); break;
	case C_EQ:      ret = (i1 == i2); break;
	case C_NE:      ret = (i1 != i2); break;
	case C_GT:      ret = (i1 >  i2); break;
	case C_GE:      ret = (i1 >= i2); break;
	case C_LT:      ret = (i1 <  i2); break;
	case C_LE:      ret = (i1 <= i2); break;
	case C_R_SHIFT: ret = i1>>i2;     break;
	case C_L_SHIFT: ret = i1<<i2;     break;
	default:
		switch(op) {
		case C_SUB:
			ret = i1 - i2;
			ret_bignum = (atn_bignumber)i1 - (atn_bignumber)i2;
			break;
		case C_MUL:
			ret = i1 * i2;
			ret_bignum = (atn_bignumber)i1 * (atn_bignumber)i2;
			break;
		case C_DIV:
			if(i2 == 0) {
				printf("script::op_2num division by zero.\n");
				ret = 0x7FFFFFFF;
				ret_bignum = 0;	// I[o[t[΍΂
			} else {
				ret = i1 / i2;
				ret_bignum = (atn_bignumber)i1 / (atn_bignumber)i2;
			}
			break;
		}
		if(ret_bignum > 0x7FFFFFFF || ret_bignum < -1 * 0x7FFFFFFF) {
			printf("script::op_2num overflow detected op:%d rid:%d\n",op,st->rid);
			ret = (ret_bignum > 0x7FFFFFFF ? 0x7FFFFFFF : -1 * 0x7FFFFFFF);
		}
	}
	push_val(st->stack,C_INT,ret);
}

/*==========================================
 * 񍀉Zq
 *------------------------------------------
 */
static void op_2(struct script_state *st,int op)
{
	int i1,i2;
	char *s1=NULL,*s2=NULL;

	i2=pop_val(st);
	if( isstr(&st->stack->stack_data[st->stack->sp]) )
		s2=st->stack->stack_data[st->stack->sp].u.str;

	i1=pop_val(st);
	if( isstr(&st->stack->stack_data[st->stack->sp]) )
		s1=st->stack->stack_data[st->stack->sp].u.str;

	if( s1!=NULL && s2!=NULL ){
		// ss => op_2str
		op_2str(st,op,st->stack->sp,st->stack->sp+1);
	}else if( s1==NULL && s2==NULL ){
		// ii => op_2num
		op_2num(st,op,i1,i2);
	}else{
		// si,is => error
		printf("script: op_2: int&str, str&int not allow.\n");
		if(s1 && st->stack->stack_data[st->stack->sp].type == C_STR)
			aFree(s1);
		if(s2 && st->stack->stack_data[st->stack->sp+1].type == C_STR)
			aFree(s2);
		push_val(st->stack,C_INT,0);
	}
}

/*==========================================
 * PZq
 *------------------------------------------
 */
static void op_1num(struct script_state *st,int op)
{
	int i1;

	i1=pop_val(st);
	switch(op){
	case C_NEG:
		i1=-i1;
		break;
	case C_NOT:
		i1=~i1;
		break;
	case C_LNOT:
		i1=!i1;
		break;
	}
	push_val(st->stack,C_INT,i1);
}

/*==========================================
 * ֐̎s
 *------------------------------------------
 */
static int run_func(struct script_state *st)
{
	int i,start_sp,end_sp,func;

	end_sp=st->stack->sp;

#ifdef DEBUG_RUN
	if(battle_config.etc_log) {
		printf("stack dump :");
		for(i=0;i<end_sp;i++){
			switch(st->stack->stack_data[i].type){
			case C_INT:
				printf(" int(%d)",st->stack->stack_data[i].u.num);
				break;
			case C_NAME:
				printf(" name(%s)",str_buf+str_data[st->stack->stack_data[i].u.num & 0xffffff].str);
				break;
			case C_ARG:
				printf(" arg");
				break;
			case C_POS:
				printf(" pos(0x%x)",st->stack->stack_data[i].u.num);
				break;
			case C_STR:
				printf(" str(%s)",st->stack->stack_data[i].u.str);
				break;
			case C_CONSTSTR:
				printf(" cstr(%s)",st->stack->stack_data[i].u.str);
				break;
			case C_RETINFO:
				printf(" retinfo(0x%x)",st->stack->stack_data[i].u.num);
				break;
			default:
				printf(" etc(%d,%d)",st->stack->stack_data[i].type,st->stack->stack_data[i].u.num);
				break;
			}
		}
		printf("\n");
	}
#endif
	for(i=end_sp-1;i>=0 && st->stack->stack_data[i].type!=C_ARG;i--);
	if(i<=0) {
		if(battle_config.error_log)
			printf("function not found\n");
		st->state=END;
		return 0;
	}
	start_sp=i-1;
	st->start=i-1;
	st->end=end_sp;

	func=st->stack->stack_data[st->start].u.num;
	if(str_data[func].type!=C_FUNC ){
		printf("run_func: not function and command! \n");
		st->state=END;
		return 0;
	}
#ifdef DEBUG_RUN
	printf("run_func : %s (func_no : %d , func_type : %d pos : 0x%x)\n",
		str_buf+str_data[func].str,func,str_data[func].type,st->pos-1
	);
#endif
	if(str_data[func].func) {
		str_data[func].func(st);
	} else {
		if(battle_config.error_log)
			printf("run_func : %s? (%d(%d))\n",str_buf+str_data[func].str,func,str_data[func].type);
		push_val(st->stack,C_INT,0);
	}

	if(st->state != RERUNLINE) {
		pop_stack(st->stack,start_sp,end_sp);
	}

	if(st->state==RETFUNC) {
		// [U[`֐̕A
		int olddefsp=st->stack->defsp;
		int j;

		pop_stack(st->stack,st->stack->defsp,start_sp);	// AɎזȃX^bN폜
		if(st->stack->defsp<5 || st->stack->stack_data[st->stack->defsp-1].type!=C_RETINFO){
			printf("script:run_func(return) return without callfunc or callsub!\n");
			st->state=END;
			return 0;
		}
		// ֐ˑϐ̍폜
		script_free_vars( st->stack->var_function );
		aFree(st->stack->var_function);

		j = conv_num(st,& (st->stack->stack_data[st->stack->defsp-5]));									// ̐擾
		st->pos = conv_num(st,& (st->stack->stack_data[st->stack->defsp-1]));								// XNvgʒu̕
		st->script = (struct script_code*)st->stack->stack_data[st->stack->defsp-3].u.ptr;		// XNvg𕜌
		st->stack->var_function = (struct linkdb_node**)st->stack->stack_data[st->stack->defsp-2].u.ptr; // ֐ˑϐ

		st->stack->defsp=conv_num(st,& (st->stack->stack_data[st->stack->defsp-4]));	// X^bN|C^𕜌
		pop_stack(st->stack,olddefsp-5-j,olddefsp);		// vȂȂX^bN(ƕApf[^)폜

		st->state=GOTO;
	}

	return 0;
}

/*==========================================
 * XNvg̎s
 *------------------------------------------
 */
static void run_script_main(struct script_state *st);

void run_script(struct script_code *rootscript,int pos,int rid,int oid)
{
	struct script_state *st;
	struct map_session_data *sd = NULL;

	if(rootscript == NULL || pos < 0)
		return;

	sd = map_id2sd(rid);
	st = (struct script_state *)aCalloc(1, sizeof(struct script_state));

	if(sd && sd->stack && sd->npc_scriptroot == rootscript) {
		// ÕX^bN𕜋A
		st->script = sd->npc_script;
		st->stack  = sd->stack;
		st->state  = sd->npc_scriptstate;
		sd->stack           = NULL; // vC[f^b`Ăǂ悤ɃNA
		sd->npc_script	    = NULL;
		sd->npc_scriptroot  = NULL;
		sd->npc_scriptstate = 0;
	} else {
		// X^bN
		st->stack = (struct script_stack *)aMalloc(sizeof(struct script_stack));
		st->stack->sp     = 0;
		st->stack->defsp  = 0;
		st->stack->sp_max = 64;
		st->stack->stack_data   = (struct script_data *)aCalloc(st->stack->sp_max,sizeof(st->stack->stack_data[0]));
		st->stack->var_function = (struct linkdb_node **)aCalloc(1, sizeof(struct linkdb_node*));
		st->state  = RUN;
		st->script = rootscript;
	}
	st->pos = pos;
	st->rid = rid;
	st->oid = oid;
	st->scriptroot = rootscript;
	st->sleep.timer = -1;
	run_script_main(st);	// stfree܂߂Ă̓ŏ
}

/*==========================================
 * wm[hsleep_db폜
 *------------------------------------------
 */
static struct linkdb_node* script_erase_sleepdb(struct linkdb_node *n)
{
	struct linkdb_node *retnode;

	if( n == NULL)
		return NULL;
	if( n->prev == NULL )
		sleep_db = n->next;
	else
		n->prev->next = n->next;
	if( n->next )
		n->next->prev = n->prev;
	retnode = n->next;
	aFree( n );
	return retnode;		// ̃m[hԂ
}

/*==========================================
 * sleepp^C}[֐
 *------------------------------------------
 */
static int run_script_timer(int tid, unsigned int tick, int id, void *data)
{
	struct script_state *st     = (struct script_state *)data;
	struct linkdb_node *node    = sleep_db;
	struct map_session_data *sd = map_id2sd(st->rid);

	if( sd && sd->status.char_id != id ) {
		st->rid = 0;
	}
	while( node && st->sleep.timer != -1 ) {
		if( (int)node->key == st->oid && ((struct script_state *)node->data)->sleep.timer == st->sleep.timer ) {
			script_erase_sleepdb(node);
			st->sleep.timer = -1;
			break;
		}
		node = node->next;
	}
	run_script_main(st);
	return 0;
}

/*==========================================
 * XNvg̎sC
 *------------------------------------------
 */
static void run_script_main(struct script_state *st)
{
	int c;
	int cmdcount  = script_config.check_cmdcount;
	int gotocount = script_config.check_gotocount;
	struct map_session_data *sd;

	if(st->state == RERUNLINE) {
		st->state = RUN;
		run_func(st);
		if(st->state == GOTO){
			st->state = RUN;
		}
	} else {
		st->state = RUN;
	}
	while(st->state == RUN) {
		switch(c = get_com(st->script->script_buf,&st->pos)) {
		case C_EOL:
			if(st->stack->sp != st->stack->defsp) {
				if(battle_config.error_log)
					printf("stack.sp(%d) != default(%d)\n",st->stack->sp,st->stack->defsp);
				pop_stack(st->stack, st->stack->defsp, st->stack->sp);
			}
			break;
		case C_INT:
			push_val(st->stack,C_INT,get_num(st->script->script_buf,&st->pos));
			break;
		case C_POS:
		case C_NAME:
			push_val(st->stack,c,(*(int*)(st->script->script_buf+st->pos))&0xffffff);
			st->pos += 3;
			break;
		case C_ARG:
			push_val(st->stack,c,0);
			break;
		case C_STR:
			push_str(st->stack,C_CONSTSTR,st->script->script_buf+st->pos);
			while(st->script->script_buf[st->pos++]);
			break;
		case C_FUNC:
			run_func(st);
			if(st->state == GOTO) {
				st->state = RUN;
				if(gotocount > 0 && (--gotocount) <= 0) {
					printf("run_script: infinity loop ! gotocount %d\n", script_config.check_gotocount);
					st->state = END;
				}
			}
			break;
		case C_ADD:
			op_add(st);
			break;
		case C_SUB:
		case C_MUL:
		case C_DIV:
		case C_MOD:
		case C_EQ:
		case C_NE:
		case C_GT:
		case C_GE:
		case C_LT:
		case C_LE:
		case C_AND:
		case C_OR:
		case C_XOR:
		case C_LAND:
		case C_LOR:
		case C_R_SHIFT:
		case C_L_SHIFT:
			op_2(st,c);
			break;
		case C_NEG:
		case C_NOT:
		case C_LNOT:
			op_1num(st,c);
			break;
		case C_OP3:
			op_3(st);
			break;
		case C_NOP:
			st->state = END;
			break;
		default:
			if(battle_config.error_log)
				printf("unknown command : %d @ 0x%06x\n",c,st->pos);
			st->state = END;
			break;
		}
		if(cmdcount > 0 && (--cmdcount) <= 0) {
			printf("run_script: infinity loop ! cmdcount %d\n", script_config.check_cmdcount);
			st->state = END;
		}
	}

	sd = map_id2sd(st->rid);
	if(st->sleep.tick > 0) {
		// X^bNsleep_dbɕۑ
		st->sleep.charid = (sd) ? sd->status.char_id : 0;
		st->sleep.timer  = add_timer(gettick()+st->sleep.tick, run_script_timer, st->sleep.charid, st);
		linkdb_insert(&sleep_db, (void*)st->oid, st);
		return;
	}

	if(st->state != END && sd) {
		if(sd->stack) {
			// ̃X^bNj
			script_free_stack(sd->stack);
			sd->state.menu_or_input = 0;
		}
		// ĊJ邽߂ɃX^bNۑ
		sd->npc_script      = st->script;
		sd->npc_scriptroot  = st->scriptroot;
		sd->npc_scriptstate = st->state;
		sd->stack           = st->stack;
		sd->npc_id          = st->oid;
		sd->npc_pos         = st->pos;
	} else {
		// sI or ss\Ȃ̂ŃX^bNNA
		if(sd && st->oid == sd->npc_id) {
			sd->npc_pos = -1;
			sd->npc_id  = 0;
			npc_event_dequeue(sd);
		}
		st->pos = -1;
		script_free_stack(st->stack);
	}
	aFree(st);

	return;
}

/*==========================================
 * }bvϐ̕ύX
 *------------------------------------------
 */
static int mapreg_setreg(int num,int val,int eternal)
{
	if(val != 0)
		numdb_insert(mapreg_db,num,val);
	else
		numdb_erase(mapreg_db,num);

	if(eternal)
		mapreg_dirty = 1;

	return 0;
}

/*==========================================
 * ^}bvϐ̕ύX
 *------------------------------------------
 */
static int mapreg_setregstr(int num,const char *str,int eternal)
{
	char *old_str;

	if(str && *str)
		old_str = (char *)numdb_insert(mapregstr_db,num,aStrdup(str));
	else
		old_str = (char *)numdb_erase(mapregstr_db,num);

	if(old_str)
		aFree(old_str);

	if(eternal)
		mapreg_dirty = 1;

	return 0;
}

#ifdef TXT_ONLY

char mapreg_txt[256] = "save/mapreg.txt";

/*==========================================
 * iI}bvϐ̓ǂݍ(TXT)
 *------------------------------------------
 */
static int script_txt_load_mapreg(void)
{
	FILE *fp;
	char line[2048];

	if( (fp=fopen(mapreg_txt,"rt"))==NULL )
		return -1;

	while(fgets(line,sizeof(line),fp)){
		char buf[256];
		int n,v,s,i;
		if( sscanf(line,"%255[^,],%d\t%n",buf,&i,&n)!=2 &&
		    (i=0,sscanf(line,"%255[^\t]\t%n",buf,&n)!=1) )
			continue;
		if(i < 0 || i >= 128) {
			printf("%s: %s broken data !\n",mapreg_txt,buf);
			continue;
		}
		if( buf[strlen(buf)-1]=='$' ){
			char buf2[2048];
			if( sscanf(line+n,"%[^\n\r]",buf2)!=1 ){
				printf("%s: %s broken data !\n",mapreg_txt,buf);
				continue;
			}
			s=add_str(buf);
			numdb_insert(mapregstr_db,(i<<24)|s,aStrdup(buf2));
		}else{
			if( sscanf(line+n,"%d",&v)!=1 ){
				printf("%s: %s broken data !\n",mapreg_txt,buf);
				continue;
			}
			s=add_str(buf);
			numdb_insert(mapreg_db,(i<<24)|s,v);
		}
	}
	fclose(fp);
	return 0;
}

/*==========================================
 * iI}bvϐ̏(TXT)
 *------------------------------------------
 */
static int script_txt_save_mapreg_intsub(void *key,void *data,va_list ap)
{
	FILE *fp=va_arg(ap,FILE*);
	int num=((int)key)&0x00ffffff, i=((int)key)>>24;
	char *name=str_buf+str_data[num].str;

	if( name[1]!='@' ){
		if(i==0)
			fprintf(fp,"%s\t%d" RETCODE, name, (int)data);
		else
			fprintf(fp,"%s,%d\t%d" RETCODE, name, i, (int)data);
	}
	return 0;
}

static int script_txt_save_mapreg_strsub(void *key,void *data,va_list ap)
{
	FILE *fp=va_arg(ap,FILE*);
	int num=((int)key)&0x00ffffff, i=((int)key)>>24;
	char *name=str_buf+str_data[num].str;

	if( name[1]!='@' ){
		if(i==0)
			fprintf(fp,"%s\t%s" RETCODE, name, (char *)data);
		else
			fprintf(fp,"%s,%d\t%s" RETCODE, name, i, (char *)data);
	}
	return 0;
}

static int script_txt_save_mapreg(void)
{
	FILE *fp;
	int lock;

	if( (fp=lock_fopen(mapreg_txt,&lock))==NULL )
		return -1;
	numdb_foreach(mapreg_db,script_txt_save_mapreg_intsub,fp);
	numdb_foreach(mapregstr_db,script_txt_save_mapreg_strsub,fp);
	lock_fclose(fp,mapreg_txt,&lock);
	mapreg_dirty=0;
	return 0;
}

#define script_load_mapreg script_txt_load_mapreg
#define script_save_mapreg script_txt_save_mapreg

#else /* TXT_ONLY */

MYSQL mysql_handle_script;

/*==========================================
 * iI}bvϐ̓ǂݍ(SQL)
 *------------------------------------------
 */
static int script_sql_load_mapreg(void)
{
	MYSQL_RES* sql_res;
	MYSQL_ROW  sql_row = NULL;
	char buf[64];

	sqldbs_query(&mysql_handle, "SELECT `reg`,`index`,`value` FROM `" MAPREG_TABLE "` WHERE `server_tag` = '%s'", strecpy(buf,map_server_tag));

	sql_res = sqldbs_store_result(&mysql_handle);

	if(sql_res) {
		int i,s;
		char name[256];
		while((sql_row = sqldbs_fetch(sql_res)) != NULL) {
			i = atoi(sql_row[1]);
			if(i < 0 || i >= 128)
				continue;
			strncpy(name, sql_row[0], 256);
			name[255] = '\0';	// force \0 terminal
			s = add_str(name);

			if(name[strlen(name)-1] == '$') {
				numdb_insert(mapregstr_db,(i<<24)|s,aStrdup(sql_row[2]));
			} else {
				numdb_insert(mapreg_db,(i<<24)|s,atoi(sql_row[2]));
			}
		}
		sqldbs_free_result(sql_res);
	}
	return 0;
}

/*==========================================
 * iI}bvϐ̏(SQL)
 *------------------------------------------
 */
static int script_sql_save_mapreg_intsub(void *key,void *data,va_list ap)
{
	int num=((int)key)&0x00ffffff, i=((int)key)>>24;
	char *name=str_buf+str_data[num].str;

	if( name[1]!='@' ){
		char buf1[64], buf2[1024];
		sqldbs_query(
			&mysql_handle,
			"INSERT INTO `" MAPREG_TABLE "` (`server_tag`,`reg`,`index`,`value`) VALUES ('%s','%s','%d','%d')",
			strecpy(buf1,map_server_tag), strecpy(buf2,name), i, (int)data
		);
	}
	return 0;
}

static int script_sql_save_mapreg_strsub(void *key,void *data,va_list ap)
{
	int num=((int)key)&0x00ffffff, i=((int)key)>>24;
	char *name=str_buf+str_data[num].str;

	if( name[1]!='@' ){
		char buf1[64], buf2[1024], buf3[4096];
		sqldbs_query(
			&mysql_handle,
			"INSERT INTO `" MAPREG_TABLE"` (`server_tag`,`reg`,`index`,`value`) VALUES ('%s','%s','%d','%s')",
			strecpy(buf1,map_server_tag), strecpy(buf2,name), i, strecpy(buf3,(char*)data)
		);
	}
	return 0;
}

static int script_sql_save_mapreg(void)
{
	char buf[64];

	sqldbs_query(&mysql_handle, "DELETE FROM `" MAPREG_TABLE "` WHERE `server_tag` = '%s'", strecpy(buf,map_server_tag));

	numdb_foreach(mapreg_db,script_sql_save_mapreg_intsub);
	numdb_foreach(mapregstr_db,script_sql_save_mapreg_strsub);
	mapreg_dirty=0;
	return 0;
}

#define script_load_mapreg script_sql_load_mapreg
#define script_save_mapreg script_sql_save_mapreg

#endif /* TXT_ONLY */

/*==========================================
 * iI}bvϐ̎Z[u
 *------------------------------------------
 */
static int script_autosave_mapreg(int tid,unsigned int tick,int id,void *data)
{
	if(mapreg_dirty)
		script_save_mapreg();
	return 0;
}

/*==========================================
 * posword̐ݒ
 *------------------------------------------
 */
static void set_posword(char *p)
{
	char *np;
	int i;

	for(i = 0; i < 11; i++) {
		if((np = strchr(p,',')) != NULL) {
			*np = 0;
		}
		strncpy(refine_posword[i], p, 32);
		refine_posword[i][31] = '\0';	// force \0 terminal

		if(np == NULL)
			break;
		p = np + 1;
	}
}

/*==========================================
 * configǂݍ
 *------------------------------------------
 */
int script_config_read(const char *cfgName)
{
	char line[1024], w1[1024], w2[1024];
	FILE *fp;
	static int count = 0;

	if(count++ == 0) {
		script_config.debug_vars = 0;
		script_config.warn_func_no_comma = 1;
		script_config.warn_cmd_no_comma  = 1;
		script_config.warn_func_mismatch_paramnum = 1;
		script_config.warn_cmd_mismatch_paramnum  = 1;
		script_config.check_cmdcount  = 65536;
		script_config.check_gotocount = 16384;
	}

	fp = fopen(cfgName,"r");
	if(fp == NULL) {
		printf("file not found: %s\n", cfgName);
		return 1;
	}
	while(fgets(line, 1020, fp)) {
		if(line[0] == '\0' || line[0] == '\r' || line[0] == '\n')
			continue;
		if(line[0] == '/' && line[1] == '/')
			continue;
		if(sscanf(line, "%[^:]: %[^\r\n]", w1, w2) != 2)
			continue;

		if(strcmpi(w1, "debug_vars") == 0) {
			script_config.debug_vars = atoi(w2);
		}
		else if(strcmpi(w1, "refine_posword") == 0) {
			set_posword(w2);
		}
		else if(strcmpi(w1, "check_cmdcount") == 0) {
			script_config.check_cmdcount = atoi(w2);
		}
		else if(strcmpi(w1, "check_gotocount") == 0) {
			script_config.check_gotocount = atoi(w2);
		}
		else if(strcmpi(w1, "error_marker_start") == 0) {
			strncpy(error_marker_start, w2, 16);
			error_marker_start[15] = '\0';
		}
		else if(strcmpi(w1, "error_marker_end") == 0) {
			strncpy(error_marker_end, w2, 16);
			error_marker_end[15] = '\0';
		}
		else if(strcmpi(w1,"import") == 0) {
			script_config_read(w2);
		}
	}
	fclose(fp);

	return 0;
}

/*==========================================
 * "this" lmapmapԍ֕ϊ
 *------------------------------------------
 */
static int script_mapname2mapid(struct script_state *st,const char *mapname)
{
	if(strcmp(mapname,"this")==0)
	{
		struct npc_data *nd = map_id2nd(st->oid);
		if(nd) {
			return nd->bl.m;
		} else {
			struct map_session_data *sd = script_rid2sd(st);
			return (sd != NULL)? sd->bl.m: -1;
		}
	}
	return map_mapname2mapid(mapname);
}

/*==========================================
 * @readvars, @writevarsp֐
 *------------------------------------------
 */
int script_check_variable(const char *name,int array_flag,int read_only)
{
	int i = search_str(name);

	if(i < 0)
		return 0;

	switch(str_data[i].type) {
		case C_NAME:		// ϐ͖ŋ
			return 1;
		case C_PARAM:		// ߍݕϐ[]Ȃ΋
			if(!array_flag)
				return 1;
			break;
		case C_INT:		// 萔[]Ȃēǂݎ莞̂݋
		case C_STR:
			if(!array_flag && read_only)
				return 1;
			break;
	}
	return 0;
}

void* script_read_vars(struct map_session_data *sd,const char *var,int elem,struct linkdb_node **ref)
{
	if(sd) {
		// vC[ɃA^b`KvȂstp
		struct script_state st;

		memset(&st, 0, sizeof(st));
		st.rid = sd->bl.id;
		return get_val2(&st, (elem<<24) | add_str(var), ref);
	}

	return get_val2(NULL, (elem<<24) | add_str(var), ref);
}

void script_write_vars(struct map_session_data *sd,const char *var,int elem,void *v,struct linkdb_node **ref)
{
	set_reg(NULL, sd, (elem<<24) | add_str(var), var, v, ref);

	return;
}

/*==========================================
 * gpꂽϐ̈ꗗo
 *------------------------------------------
 */
static int varlist_sort1(void *key,void *data,va_list ap)
{
	int *count  = va_arg(ap, int*);
	int *v      = va_arg(ap, int*);

	v[(*count)++] = (int)key;
	return 0;
}

static int varlist_sort2(const void *a, const void *b)
{
	char *name1 = str_buf + str_data[*(const int*)a].str;
	char *name2 = str_buf + str_data[*(const int*)b].str;

	return strcmp(name1, name2);
}

static int varsdb_final(void *key,void *data,va_list ap)
{
	struct vars_info *v = (struct vars_info*)data;

	aFree( v->file );
	aFree( v );
	return 0;
}

static int varsdb_output(void)
{
	if( vars_db ) {
		FILE *fp = fopen("variables.txt", "wt");
		if( fp ) {
			int i;
			int *vlist = (int *)aMalloc( vars_count * sizeof(int) );
			int count  = 0;
			numdb_foreach(vars_db, varlist_sort1, &count, vlist);
			qsort( vlist, count, sizeof(int), varlist_sort2 );

			printf("do_final_script: enum used variables\n");
			for(i = 0; i < count; i++) {
				char *name = str_buf + str_data[ vlist[i] ].str;
				struct vars_info *v = (struct vars_info *)numdb_search( vars_db, (void*)vlist[i] );
				if( strncmp(name, "$@__", 4) != 0 ) {
					// switch ̓ϐ͏O
					fprintf(fp, "%-20s % 4d %s line %d" RETCODE, name, v->use_count, v->file, v->line);
				}
			}
			aFree( vlist );
			fclose(fp);
		}
		numdb_final(vars_db, varsdb_final);
	}
	return 0;
}

/*==========================================
 * nbV֘Ao
 *------------------------------------------
 */
#ifdef DEBUG_HASH

static int debug_hash_output(void)
{
	FILE *fp = fopen("hash_dump.txt", "wt");

	if(fp) {
		int i,count[SCRIPT_HASH_SIZE];
		int max=0;
		int *buckets;

		printf("do_final_script: dumping script str hash information\n");
		memset(count, 0, sizeof(count));

		fprintf(fp, "#1 dump all data" RETCODE);
		fprintf(fp, "-------+--------+----------------------" RETCODE);
		fprintf(fp, "  num  |  hash  | data_name"             RETCODE);
		fprintf(fp, "-------+--------+----------------------" RETCODE);
		for(i=LABEL_START; i<str_num; i++) {
			unsigned int h = calc_hash(str_buf+str_data[i].str);
			fprintf(fp, " %05d | %6u | %s" RETCODE, i, h, str_buf+str_data[i].str);
			if(++count[h] > max)
				max = count[h];
		}
		fprintf(fp, "-------+--------+----------------------" RETCODE RETCODE);

		buckets = (int *)aCalloc((max+1),sizeof(int));

		fprintf(fp, "#2 how many data is there in one hash" RETCODE);
		fprintf(fp, "--------+-------" RETCODE);
		fprintf(fp, "  hash  | count"  RETCODE);
		fprintf(fp, "--------+-------" RETCODE);
		for(i=0; i<SCRIPT_HASH_SIZE; i++) {
			fprintf(fp, " %6d | %5d" RETCODE, i, count[i]);
			buckets[count[i]]++;
		}
		fprintf(fp, "--------+-------" RETCODE RETCODE);

		fprintf(fp, "#3 distribution of hashed data" RETCODE);
		fprintf(fp, "-------+---------+---------" RETCODE);
		fprintf(fp, " items | buckets | percent"  RETCODE);
		fprintf(fp, "-------+---------+---------" RETCODE);
		for(i=0; i<=max; i++) {
			fprintf(fp, " %5d | %7d | %6.2lf%%" RETCODE, i, buckets[i], (double)buckets[i]/SCRIPT_HASH_SIZE*100.);
		}
		aFree(buckets);
		fclose(fp);
	}
	return 0;
}

#endif

/*==========================================
 * I
 *------------------------------------------
 */
static int mapregstr_db_final(void *key,void *data,va_list ap)
{
	aFree(data);
	return 0;
}

static int userfunc_db_final(void *key,void *data,va_list ap)
{
	aFree(key);
	script_free_code((struct script_code *)data);
	return 0;
}

int do_final_script(void)
{
	int i;

	if(mapreg_dirty)
		script_save_mapreg();

	if(mapreg_db)
		numdb_final(mapreg_db,NULL);
	if(mapregstr_db)
		strdb_final(mapregstr_db,mapregstr_db_final);
	if(userfunc_db)
		strdb_final(userfunc_db,userfunc_db_final);
	if(scriptlabel_db)
		linkdb_final(&scriptlabel_db);
	if(sleep_db) {
		struct linkdb_node *n = sleep_db;
		while(n) {
			struct script_state *st = (struct script_state *)n->data;
			script_free_stack(st->stack);
			aFree(st);
			n = n->next;
		}
		linkdb_final(&sleep_db);
	}
	varsdb_output();

#ifdef DEBUG_HASH
	debug_hash_output();
#endif

#ifndef NO_CSVDB_SCRIPT
	script_csvfinal();
#endif

	for(i = LABEL_START; i < str_num; i++) {
		if(str_data[i].type == C_STR)
			aFree(str_data[i].u.str);
	}
	aFree(str_buf);
	aFree(str_data);

	return 0;
}

/*==========================================
 * 
 *------------------------------------------
 */
int do_init_script(void)
{
	mapreg_db=numdb_init();
	mapregstr_db=numdb_init();
	script_load_mapreg();

	add_timer_func_list(run_script_timer);
	add_timer_func_list(script_autosave_mapreg);

	add_timer_interval(gettick()+MAPREG_AUTOSAVE_INTERVAL,
		script_autosave_mapreg,0,NULL,MAPREG_AUTOSAVE_INTERVAL);

#ifndef NO_CSVDB_SCRIPT
	script_csvinit();
#endif

	if(script_config.debug_vars)
		vars_db = numdb_init();

	return 0;
}

//
// ߍ݊֐
//

int buildin_mes(struct script_state *st);
int buildin_goto(struct script_state *st);
int buildin_callsub(struct script_state *st);
int buildin_callfunc(struct script_state *st);
int buildin_return(struct script_state *st);
int buildin_getarg(struct script_state *st);
int buildin_getargcount(struct script_state *st);
int buildin_next(struct script_state *st);
int buildin_close(struct script_state *st);
int buildin_close2(struct script_state *st);
int buildin_menu(struct script_state *st);
int buildin_rand(struct script_state *st);
int buildin_warp(struct script_state *st);
int buildin_mapwarp(struct script_state *st);
int buildin_areawarp(struct script_state *st);
int buildin_heal(struct script_state *st);
int buildin_itemheal(struct script_state *st);
int buildin_percentheal(struct script_state *st);
int buildin_jobchange(struct script_state *st);
int buildin_input(struct script_state *st);
int buildin_setlook(struct script_state *st);
int buildin_getlook(struct script_state *st);
int buildin_set(struct script_state *st);
int buildin_setarray(struct script_state *st);
int buildin_cleararray(struct script_state *st);
int buildin_copyarray(struct script_state *st);
int buildin_getarraysize(struct script_state *st);
int buildin_deletearray(struct script_state *st);
int buildin_printarray(struct script_state *st);
int buildin_getelementofarray(struct script_state *st);
int buildin_getitem(struct script_state *st);
int buildin_getitem2(struct script_state *st);
int buildin_delitem(struct script_state *st);
int buildin_delcartitem(struct script_state *st);
int buildin_delitem2(struct script_state *st);
int buildin_delcartitem2(struct script_state *st);
int buildin_viewpoint(struct script_state *st);
int buildin_countitem(struct script_state *st);
int buildin_countcartitem(struct script_state *st);
int buildin_checkweight(struct script_state *st);
int buildin_readparam(struct script_state *st);
int buildin_getcharid(struct script_state *st);
int buildin_getcharname(struct script_state *st);
int buildin_getpartyname(struct script_state *st);
int buildin_getpartymember(struct script_state *st);
int buildin_getguildname(struct script_state *st);
int buildin_getguildmaster(struct script_state *st);
int buildin_getguildmasterid(struct script_state *st);
int buildin_strcharinfo(struct script_state *st);
int buildin_getequipid(struct script_state *st);
int buildin_getequipname(struct script_state *st);
int buildin_getequipisequiped(struct script_state *st);
int buildin_getequipisenableref(struct script_state *st);
int buildin_getequipisidentify(struct script_state *st);
int buildin_getequiprefinerycnt(struct script_state *st);
int buildin_getequipweaponlv(struct script_state *st);
int buildin_getequippercentrefinery(struct script_state *st);
int buildin_successrefitem(struct script_state *st);
int buildin_failedrefitem(struct script_state *st);
int buildin_cutin(struct script_state *st);
int buildin_cutincard(struct script_state *st);
int buildin_statusup(struct script_state *st);
int buildin_statusup2(struct script_state *st);
int buildin_bonus(struct script_state *st);
int buildin_bonus2(struct script_state *st);
int buildin_bonus3(struct script_state *st);
int buildin_bonus4(struct script_state *st);
int buildin_skill(struct script_state *st);
int buildin_guildskill(struct script_state *st);
int buildin_getskilllv(struct script_state *st);
int buildin_getgdskilllv(struct script_state *st);
int buildin_basicskillcheck(struct script_state *st);
int buildin_getgmlevel(struct script_state *st);
int buildin_end(struct script_state *st);
int buildin_checkoption(struct script_state *st);
int buildin_setoption(struct script_state *st);
int buildin_setcart(struct script_state *st);
int buildin_setfalcon(struct script_state *st);
int buildin_setriding(struct script_state *st);
int buildin_savepoint(struct script_state *st);
int buildin_gettimetick(struct script_state *st);
int buildin_gettime(struct script_state *st);
int buildin_gettimestr(struct script_state *st);
int buildin_openstorage(struct script_state *st);
int buildin_guildopenstorage(struct script_state *st);
int buildin_itemskill(struct script_state *st);
int buildin_produce(struct script_state *st);
int buildin_monster(struct script_state *st);
int buildin_areamonster(struct script_state *st);
int buildin_killmonster(struct script_state *st);
int buildin_killmonsterall(struct script_state *st);
int buildin_areakillmonster(struct script_state *st);
int buildin_doevent(struct script_state *st);
int buildin_donpcevent(struct script_state *st);
int buildin_addtimer(struct script_state *st);
int buildin_deltimer(struct script_state *st);
int buildin_addtimercount(struct script_state *st);
int buildin_gettimerdiff(struct script_state *st);
int buildin_initnpctimer(struct script_state *st);
int buildin_stopnpctimer(struct script_state *st);
int buildin_startnpctimer(struct script_state *st);
int buildin_setnpctimer(struct script_state *st);
int buildin_getnpctimer(struct script_state *st);
int buildin_announce(struct script_state *st);
int buildin_mapannounce(struct script_state *st);
int buildin_areaannounce(struct script_state *st);
int buildin_getusers(struct script_state *st);
int buildin_getmapusers(struct script_state *st);
int buildin_getareausers(struct script_state *st);
int buildin_getareadropitem(struct script_state *st);
int buildin_enablenpc(struct script_state *st);
int buildin_disablenpc(struct script_state *st);
int buildin_hideoffnpc(struct script_state *st);
int buildin_hideonnpc(struct script_state *st);
int buildin_sc_start(struct script_state *st);
int buildin_sc_start2(struct script_state *st);
int buildin_sc_starte(struct script_state *st);
int buildin_sc_start3(struct script_state *st);
int buildin_sc_end(struct script_state *st);
int buildin_sc_ison(struct script_state *st);
int buildin_getscrate(struct script_state *st);
int buildin_debugmes(struct script_state *st);
int buildin_catchpet(struct script_state *st);
int buildin_birthpet(struct script_state *st);
int buildin_resetstatus(struct script_state *st);
int buildin_resetskill(struct script_state *st);
int buildin_changebase(struct script_state *st);
int buildin_changesex(struct script_state *st);
int buildin_waitingroom(struct script_state *st);
int buildin_delwaitingroom(struct script_state *st);
int buildin_kickwaitingroom(struct script_state *st);
int buildin_kickwaitingroomall(struct script_state *st);
int buildin_enablewaitingroomevent(struct script_state *st);
int buildin_disablewaitingroomevent(struct script_state *st);
int buildin_getwaitingroomstate(struct script_state *st);
int buildin_warpwaitingpc(struct script_state *st);
int buildin_getwaitingpcid(struct script_state *st);
int buildin_attachrid(struct script_state *st);
int buildin_detachrid(struct script_state *st);
int buildin_isloggedin(struct script_state *st);
int buildin_setmapflag(struct script_state *st);
int buildin_removemapflag(struct script_state *st);
int buildin_checkmapflag(struct script_state *st);
int buildin_pvpon(struct script_state *st);
int buildin_pvpoff(struct script_state *st);
int buildin_gvgon(struct script_state *st);
int buildin_gvgoff(struct script_state *st);
int buildin_emotion(struct script_state *st);
int buildin_maprespawnguildid(struct script_state *st);
int buildin_agitstart(struct script_state *st);
int buildin_agitend(struct script_state *st);
int buildin_agitcheck(struct script_state *st);
int buildin_flagemblem(struct script_state *st);
int buildin_getcastlename(struct script_state *st);
int buildin_getcastledata(struct script_state *st);
int buildin_setcastledata(struct script_state *st);
int buildin_requestguildinfo(struct script_state *st);
int buildin_getequipcardcnt(struct script_state *st);
int buildin_successremovecards(struct script_state *st);
int buildin_failedremovecards(struct script_state *st);
int buildin_marriage(struct script_state *st);
int buildin_wedding_effect(struct script_state *st);
int buildin_divorce(struct script_state *st);
int buildin_getitemname(struct script_state *st);
int buildin_makepet(struct script_state *st);
int buildin_getinventorylist(struct script_state *st);
int buildin_getcartlist(struct script_state *st);
int buildin_getskilllist(struct script_state *st);
int buildin_clearitem(struct script_state *st);
int buildin_clearcartitem(struct script_state *st);
int buildin_getrepairableitemcount(struct script_state *st);
int buildin_repairitem(struct script_state *st);
int buildin_classchange(struct script_state *st);
int buildin_misceffect(struct script_state *st);
int buildin_areamisceffect(struct script_state *st);
int buildin_soundeffect(struct script_state *st);
int buildin_areasoundeffect(struct script_state *st);
int buildin_gmcommand(struct script_state *st);
int buildin_getusersname(struct script_state *st);
int buildin_dispbottom(struct script_state *st);
int buildin_recovery(struct script_state *st);
int buildin_getpetinfo(struct script_state *st);
int buildin_gethomuninfo(struct script_state *st);
int buildin_checkequipedcard(struct script_state *st);
int buildin_globalmes(struct script_state *st);
int buildin_jump_zero(struct script_state *st);
int buildin_select(struct script_state *st);
int buildin_getmapmobs(struct script_state *st);
int buildin_getareamobs(struct script_state *st);
int buildin_getguildrelation(struct script_state *st);
int buildin_unequip(struct script_state *st);
int buildin_allowuseitem(struct script_state *st);
int buildin_equippeditem(struct script_state *st);
int buildin_getmapname(struct script_state *st);
int buildin_summon(struct script_state *st);
int buildin_getmapxy(struct script_state *st);
int buildin_checkcart(struct script_state *st);
int buildin_checkfalcon(struct script_state *st);
int buildin_checkriding(struct script_state *st);
int buildin_checksit(struct script_state *st);
int buildin_checkdead(struct script_state *st);
int buildin_checkcasting(struct script_state *st);
int buildin_adoption(struct script_state *st);
int buildin_breakadoption(struct script_state *st);
int buildin_petskillattack(struct script_state *st);
int buildin_petskillsupport(struct script_state *st);
int buildin_changepettype(struct script_state *st);
int buildin_making(struct script_state *st);
int buildin_getpkflag(struct script_state *st);
int buildin_guildgetexp(struct script_state *st);
int buildin_flagname(struct script_state *st);
int buildin_getnpcposition(struct script_state *st);
#ifndef NO_CSVDB_SCRIPT
int buildin_csvgetrows(struct script_state *st);
int buildin_csvgetcols(struct script_state *st);
int buildin_csvread(struct script_state *st);
int buildin_csvreadarray(struct script_state *st);
int buildin_csvfind(struct script_state *st);
int buildin_csvwrite(struct script_state *st);
int buildin_csvwritearray(struct script_state *st);
int buildin_csvreload(struct script_state *st);
int buildin_csvinsert(struct script_state *st);
int buildin_csvdelete(struct script_state *st);
int buildin_csvsort(struct script_state *st);
int buildin_csvflush(struct script_state *st);
#endif
int buildin_sleep(struct script_state *st);
int buildin_sleep2(struct script_state *st);
int buildin_awake(struct script_state *st);
int buildin_getvariableofnpc(struct script_state *st);
int buildin_changeviewsize(struct script_state *st);
int buildin_usediteminfo(struct script_state *st);
int buildin_getitemid(struct script_state *st);
int buildin_getguildmembers(struct script_state *st);
int buildin_npcskillattack(struct script_state *st);
int buildin_npcskillsupport(struct script_state *st);
int buildin_npcskillpos(struct script_state *st);
int buildin_strnpcinfo(struct script_state *st);
int buildin_getpartyleader(struct script_state *st);
int buildin_getstrlen(struct script_state *st);
int buildin_substr(struct script_state *st);
int buildin_distance(struct script_state *st);
int buildin_homundel(struct script_state *st);
int buildin_homunrename(struct script_state *st);
int buildin_homunevolution(struct script_state *st);
int buildin_recalcstatus(struct script_state *st);
int buildin_sqlquery(struct script_state *st);
int buildin_strescape(struct script_state *st);
int buildin_dropitem(struct script_state *st);
int buildin_getexp(struct script_state *st);
int buildin_getiteminfo(struct script_state *st);
int buildin_getonlinepartymember(struct script_state *st);
int buildin_getonlineguildmember(struct script_state *st);
int buildin_makemerc(struct script_state *st);
int buildin_openbook(struct script_state *st);
int buildin_pushpc(struct script_state *st);
int buildin_setcell(struct script_state *st);
int buildin_areasetcell(struct script_state *st);
int buildin_callguardian(struct script_state *st);
int buildin_getguardianinfo(struct script_state *st);
int buildin_getmobname(struct script_state *st);
int buildin_checkactiveitem(struct script_state *st);
//  ++ NPCIX by Blaze
int buildin_setvending(struct script_state *st);
int buildin_delvending(struct script_state *st);
int buildin_getvending(struct script_state *st);
int buildin_openvending(struct script_state *st);
int buildin_closevending(struct script_state *st);
int buildin_checkvending(struct script_state *st);
//  -- NPCIX by Blaze

struct script_function buildin_func[] = {
	{buildin_mes,"mes","s"},
	{buildin_next,"next",""},
	{buildin_close,"close",""},
	{buildin_close2,"close2",""},
	{buildin_menu,"menu","*"},
	{buildin_goto,"goto","l"},
	{buildin_callsub,"callsub","i*"},
	{buildin_callfunc,"callfunc","s*"},
	{buildin_return,"return","*"},
	{buildin_getarg,"getarg","i"},
	{buildin_getargcount,"getargcount",""},
	{buildin_jobchange,"jobchange","i*"},
	{buildin_input,"input","s"},
	{buildin_warp,"warp","sii"},
	{buildin_mapwarp,"mapwarp","ssii"},
	{buildin_areawarp,"areawarp","siiiisii"},
	{buildin_setlook,"setlook","ii"},
	{buildin_getlook,"getlook","i"},
	{buildin_set,"set","ii"},
	{buildin_setarray,"setarray","ii*"},
	{buildin_cleararray,"cleararray","iii"},
	{buildin_copyarray,"copyarray","iii"},
	{buildin_getarraysize,"getarraysize","i"},
	{buildin_deletearray,"deletearray","ii"},
	{buildin_printarray,"printarray","s*"},
	{buildin_getelementofarray,"getelementofarray","ii*"},
	{buildin_getitem,"getitem","ii**"},
	{buildin_getitem2,"getitem2","iiiiiiiii*"},
	{buildin_delitem,"delitem","ii*"},
	{buildin_delcartitem,"delcartitem","ii*"},
	{buildin_delitem2,"delitem2","ii*"},
	{buildin_delcartitem2,"delcartitem2","ii*"},
	{buildin_cutin,"cutin","si"},
	{buildin_cutincard,"cutincard","i"},
	{buildin_viewpoint,"viewpoint","iiiii"},
	{buildin_heal,"heal","ii"},
	{buildin_itemheal,"itemheal","ii"},
	{buildin_percentheal,"percentheal","ii"},
	{buildin_rand,"rand","i*"},
	{buildin_countitem,"countitem","i"},
	{buildin_countcartitem,"countcartitem","i"},
	{buildin_checkweight,"checkweight","ii"},
	{buildin_readparam,"readparam","i*"},
	{buildin_getcharid,"getcharid","i*"},
	{buildin_getcharname,"getcharname","i"},
	{buildin_getpartyname,"getpartyname","i"},
	{buildin_getpartymember,"getpartymember","i"},
	{buildin_getguildname,"getguildname","i"},
	{buildin_getguildmaster,"getguildmaster","i"},
	{buildin_getguildmasterid,"getguildmasterid","i"},
	{buildin_strcharinfo,"strcharinfo","i"},
	{buildin_getequipid,"getequipid","i"},
	{buildin_getequipname,"getequipname","i"},
	{buildin_getequipisequiped,"getequipisequiped","i"},
	{buildin_getequipisenableref,"getequipisenableref","i"},
	{buildin_getequipisidentify,"getequipisidentify","i"},
	{buildin_getequiprefinerycnt,"getequiprefinerycnt","i"},
	{buildin_getequipweaponlv,"getequipweaponlv","i"},
	{buildin_getequippercentrefinery,"getequippercentrefinery","i"},
	{buildin_successrefitem,"successrefitem","i"},
	{buildin_failedrefitem,"failedrefitem","i"},
	{buildin_statusup,"statusup","i"},
	{buildin_statusup2,"statusup2","ii"},
	{buildin_bonus,"bonus","ii"},
	{buildin_bonus2,"bonus2","iii"},
	{buildin_bonus3,"bonus3","iiii"},
	{buildin_bonus4,"bonus4","iiiii"},
	{buildin_skill,"skill","ii*"},
	{buildin_guildskill,"guildskill","ii*"},
	{buildin_getskilllv,"getskilllv","i"},
	{buildin_getgdskilllv,"getgdskilllv","ii"},
	{buildin_basicskillcheck,"basicskillcheck",""},
	{buildin_getgmlevel,"getgmlevel",""},
	{buildin_end,"end",""},
	{buildin_checkoption,"checkoption","i"},
	{buildin_setoption,"setoption","i"},
	{buildin_setcart,"setcart",""},
	{buildin_setfalcon,"setfalcon",""},
	{buildin_setriding,"setriding",""},
	{buildin_savepoint,"savepoint","sii"},
	{buildin_gettimetick,"gettimetick","i"},
	{buildin_gettime,"gettime","i"},
	{buildin_gettimestr,"gettimestr","si"},
	{buildin_openstorage,"openstorage",""},
	{buildin_guildopenstorage,"guildopenstorage",""},
	{buildin_itemskill,"itemskill","iis*"},
	{buildin_produce,"produce","i"},
	{buildin_monster,"monster","siissi*"},
	{buildin_areamonster,"areamonster","siiiissi*"},
	{buildin_killmonster,"killmonster","ss"},
	{buildin_killmonsterall,"killmonsterall","s"},
	{buildin_areakillmonster,"areakillmonster","siiii"},
	{buildin_doevent,"doevent","s"},
	{buildin_donpcevent,"donpcevent","s"},
	{buildin_addtimer,"addtimer","is"},
	{buildin_deltimer,"deltimer","s"},
	{buildin_addtimercount,"addtimercount","si"},
	{buildin_gettimerdiff,"gettimerdiff","s"},
	{buildin_initnpctimer,"initnpctimer","*"},
	{buildin_stopnpctimer,"stopnpctimer","*"},
	{buildin_startnpctimer,"startnpctimer","*"},
	{buildin_setnpctimer,"setnpctimer","*"},
	{buildin_getnpctimer,"getnpctimer","i*"},
	{buildin_announce,"announce","si*"},
	{buildin_mapannounce,"mapannounce","ssi*"},
	{buildin_areaannounce,"areaannounce","siiiisi*"},
	{buildin_getusers,"getusers","i"},
	{buildin_getmapusers,"getmapusers","s"},
	{buildin_getareausers,"getareausers","siiii"},
	{buildin_getareadropitem,"getareadropitem","siiiii"},
	{buildin_enablenpc,"enablenpc","*"},
	{buildin_disablenpc,"disablenpc","*"},
	{buildin_hideoffnpc,"hideoffnpc","*"},
	{buildin_hideonnpc,"hideonnpc","*"},
	{buildin_sc_start,"sc_start","iii*"},
	{buildin_sc_start2,"sc_start2","iiii*"},
	{buildin_sc_starte,"sc_starte","iii*"},
	{buildin_sc_start3,"sc_start3","iiiiiii*"},
	{buildin_sc_end,"sc_end","i*"},
	{buildin_sc_ison,"sc_ison","i"},
	{buildin_getscrate,"getscrate","ii*"},
	{buildin_debugmes,"debugmes","s"},
	{buildin_catchpet,"pet","i"},
	{buildin_birthpet,"bpet",""},
	{buildin_resetstatus,"resetstatus",""},
	{buildin_resetskill,"resetskill","*"},
	{buildin_changebase,"changebase","i*"},
	{buildin_changesex,"changesex",""},
	{buildin_waitingroom,"waitingroom","si*"},
	{buildin_delwaitingroom,"delwaitingroom","*"},
	{buildin_kickwaitingroom,"kickwaitingroom","i"},
	{buildin_kickwaitingroomall,"kickwaitingroomall","*"},
	{buildin_enablewaitingroomevent,"enablewaitingroomevent","*"},
	{buildin_disablewaitingroomevent,"disablewaitingroomevent","*"},
	{buildin_getwaitingroomstate,"getwaitingroomstate","i*"},
	{buildin_warpwaitingpc,"warpwaitingpc","sii*"},
	{buildin_getwaitingpcid,"getwaitingpcid","i*"},
	{buildin_attachrid,"attachrid","i"},
	{buildin_detachrid,"detachrid",""},
	{buildin_isloggedin,"isloggedin","i"},
	{buildin_setmapflag,"setmapflag","si*"},
	{buildin_removemapflag,"removemapflag","si"},
	{buildin_checkmapflag,"checkmapflag","si"},
	{buildin_pvpon,"pvpon","s"},
	{buildin_pvpoff,"pvpoff","s"},
	{buildin_gvgon,"gvgon","s"},
	{buildin_gvgoff,"gvgoff","s"},
	{buildin_emotion,"emotion","i*"},
	{buildin_maprespawnguildid,"maprespawnguildid","sii"},
	{buildin_agitstart,"agitstart",""},
	{buildin_agitend,"agitend",""},
	{buildin_agitcheck,"agitcheck",""},
	{buildin_flagemblem,"flagemblem","i"},
	{buildin_getcastlename,"getcastlename","s*"},
	{buildin_getcastledata,"getcastledata","si*"},
	{buildin_setcastledata,"setcastledata","sii"},
	{buildin_requestguildinfo,"requestguildinfo","i*"},
	{buildin_getequipcardcnt,"getequipcardcnt","i"},
	{buildin_successremovecards,"successremovecards","i*"},
	{buildin_failedremovecards,"failedremovecards","ii*"},
	{buildin_marriage,"marriage","s"},
	{buildin_wedding_effect,"wedding",""},
	{buildin_divorce,"divorce","i"},
	{buildin_getitemname,"getitemname","i"},
	{buildin_makepet,"makepet","i"},
	{buildin_getinventorylist,"getinventorylist",""},
	{buildin_getcartlist,"getcartlist",""},
	{buildin_getskilllist,"getskilllist",""},
	{buildin_clearitem,"clearitem",""},
	{buildin_clearcartitem,"clearcartitem",""},
	{buildin_getrepairableitemcount,"getrepairableitemcount",""},
	{buildin_repairitem,"repairitem",""},
	{buildin_classchange,"classchange","ii"},
	{buildin_misceffect,"misceffect","i*"},
	{buildin_areamisceffect,"areamisceffect","siiiii"},
	{buildin_soundeffect,"soundeffect","si*"},
	{buildin_areasoundeffect,"areasoundeffect","siiiisi*"},
	{buildin_gmcommand,"gmcommand","s"},
	{buildin_dispbottom,"dispbottom","s"},
	{buildin_getusersname,"getusersname",""},
	{buildin_recovery,"recovery",""},
	{buildin_getpetinfo,"getpetinfo","i"},
	{buildin_gethomuninfo,"gethomuninfo","i"},
	{buildin_checkequipedcard,"checkequipedcard","i"},
	{buildin_jump_zero,"jump_zero","ii"},
	{buildin_select,"select","*"},
	{buildin_globalmes,"globalmes","s*"},
	{buildin_getmapmobs,"getmapmobs","s"},
	{buildin_getareamobs,"getareamobs","siiii"},
	{buildin_getguildrelation,"getguildrelation","i*"},
	{buildin_unequip,"unequip","*"},
	{buildin_allowuseitem,"allowuseitem","*"},
	{buildin_equippeditem,"equippeditem","i*"},
	{buildin_getmapname,"getmapname","s"},
	{buildin_summon,"summon","si*"},
	{buildin_getmapxy,"getmapxy","siii*"},
	{buildin_checkcart,"checkcart",""},
	{buildin_checkfalcon,"checkfalcon",""},
	{buildin_checkriding,"checkriding",""},
	{buildin_checksit,"checksit",""},
	{buildin_checkdead,"checkdead",""},
	{buildin_checkcasting,"checkcasting",""},
	{buildin_adoption,"adoption","s*"},
	{buildin_breakadoption,"breakadoption","*"},
	{buildin_petskillattack,"petskillattack","iiii"},
	{buildin_petskillsupport,"petskillsupport","iiiii"},
	{buildin_changepettype,"changepettype","i"},
	{buildin_making,"making","ii"},
	{buildin_getpkflag,"getpkflag","s"},
	{buildin_guildgetexp,"guildgetexp","i"},
	{buildin_flagname,"flagname","s"},
	{buildin_getnpcposition,"getnpcposition","s"},
	{buildin_homundel,"homundel",""},
	{buildin_homunrename,"homunrename","s*"},
	{buildin_homunevolution,"homunevolution",""},
#ifndef NO_CSVDB_SCRIPT
	{buildin_csvgetrows,"csvgetrows","s"},
	{buildin_csvgetcols,"csvgetcols","si"},
	{buildin_csvread,"csvread","sii"},
	{buildin_csvreadarray,"csvreadarray","sii"},
	{buildin_csvfind,"csvfind","sii"},
	{buildin_csvwrite,"csvwrite","siis"},
	{buildin_csvwritearray,"csvwritearray","sii"},
	{buildin_csvreload,"csvreload","s"},
	{buildin_csvinsert,"csvinsert","si"},
	{buildin_csvdelete,"csvdelete","si"},
	{buildin_csvsort,"csvsort","sii"},
	{buildin_csvflush,"csvflush","s"},
#endif
	{buildin_sleep,"sleep","i"},
	{buildin_sleep2,"sleep2","i"},
	{buildin_awake,"awake","s"},
	{buildin_getvariableofnpc,"getvariableofnpc","is"},
	{buildin_changeviewsize,"changeviewsize","ii"},
	{buildin_usediteminfo,"usediteminfo","i"},
	{buildin_getitemid,"getitemid","s"},
	{buildin_getguildmembers,"getguildmembers","i"},
	{buildin_npcskillattack,"npcskillattack","iii"},
	{buildin_npcskillsupport,"npcskillsupport","ii"},
	{buildin_npcskillpos,"npcskillpos","iiii"},
	{buildin_strnpcinfo,"strnpcinfo","i"},
	{buildin_getpartyleader,"getpartyleader","i"},
	{buildin_getstrlen,"getstrlen","s"},
	{buildin_substr,"substr","si*"},
	{buildin_distance,"distance","i*"},
	{buildin_recalcstatus,"recalcstatus","*"},
	{buildin_sqlquery,"sqlquery","s*"},
	{buildin_strescape,"strescape","s"},
	{buildin_dropitem,"dropitem","siiiii*"},
	{buildin_getexp,"getexp","ii"},
	{buildin_getiteminfo,"getiteminfo","si"},
	{buildin_getonlinepartymember,"getonlinepartymember","*"},
	{buildin_getonlineguildmember,"getonlineguildmember","*"},
	{buildin_makemerc,"makemerc","ii"},
	{buildin_openbook,"openbook","i*"},
	{buildin_pushpc,"pushpc","ii"},
	{buildin_setcell,"setcell","siii"},
	{buildin_areasetcell,"areasetcell","siiiii"},
	{buildin_callguardian,"callguardian","siissi*"},
	{buildin_getguardianinfo,"getguardianinfo","sii"},
	{buildin_getmobname,"getmobname","i"},
	{buildin_checkactiveitem,"checkactiveitem","*"},
//  ++ NPCIX by Blaze
	{buildin_setvending,"setvending","iiii*"},
	{buildin_delvending,"delvending","*"},
	{buildin_getvending,"getvending","*"},
	{buildin_openvending,"openvending","s*"},
	{buildin_closevending,"closevending","*"},
	{buildin_checkvending,"checkvending","*"},
//  -- NPCIX by Blaze
	{NULL,NULL,NULL}
};

/*==========================================
 *
 *------------------------------------------
 */
int buildin_mes(struct script_state *st)
{
	char *str;

	str = conv_str(st,& (st->stack->stack_data[st->start+2]));
	if(*str)
		clif_scriptmes(script_rid2sd(st),st->oid,str);
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_goto(struct script_state *st)
{
	if( st->stack->stack_data[st->start+2].type!=C_POS ){
		printf("buildin_goto: not label !\n");
		st->state=END;
		return 0;
	}

	st->pos = conv_num(st,& (st->stack->stack_data[st->start+2]));
	st->state = GOTO;
	return 0;
}

/*==========================================
 * [U[`֐̌Ăяo
 *------------------------------------------
 */
int buildin_callfunc(struct script_state *st)
{
	struct script_code *scr, *oldscr;
	char *str=conv_str(st,& (st->stack->stack_data[st->start+2]));

	if( (scr = (struct script_code *)strdb_search(script_get_userfunc_db(),str)) ){
		int i,j;
		struct linkdb_node **oldval = st->stack->var_function;

		for(i=st->start+3,j=0;i<st->end;i++,j++) {
			push_copy(st->stack,i);
		}

		push_val(st->stack,C_INT,j);				// ̐vbV
		push_val(st->stack,C_INT,st->stack->defsp);		// ݂̊X^bN|C^vbV
		push_ptr(st->stack,C_PTR,st->script);		// ݂̃XNvgvbV
		push_ptr(st->stack,C_PTR,st->stack->var_function);	// ݂̊֐ˑϐvbV
		push_val(st->stack,C_RETINFO,st->pos);			// ݂̃XNvgʒuvbV

		oldscr = st->script;
		st->pos=0;
		st->script=scr;
		st->stack->defsp=st->start+5+j;
		st->state=GOTO;
		st->stack->var_function = (struct linkdb_node**)aCalloc(1, sizeof(struct linkdb_node*));

		// ' ϐ̈p
		for(i = 0; i < j; i++) {
			struct script_data *s = &st->stack->stack_data[st->stack->sp-6-i];
			if( s->type == C_NAME && !s->ref ) {
				char *name = str_buf+str_data[s->u.num&0x00ffffff].str;
				// '@ ϐ̈p
				if( name[0] == '\'' && name[1] == '@' ) {
					s->ref = oldval;
				} else if( name[0] == '\'' ) {
					s->ref = &oldscr->script_vars;
				}
			}
		}

	}else{
		printf("buildin_callfunc: function not found! [%s]\n",str);
		st->state=END;
	}
	return 0;
}

/*==========================================
 * Tu[eB̌Ăяo
 *------------------------------------------
 */
int buildin_callsub(struct script_state *st)
{
	int pos = conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(st->stack->stack_data[st->start+2].type != C_POS) {
		printf("buildin_callsub: not label !\n");
		st->state=END;
	} else {
		int i,j;
		struct linkdb_node **oldval = st->stack->var_function;

		for(i=st->start+3,j=0;i<st->end;i++,j++) {
			push_copy(st->stack,i);
		}

		push_val(st->stack,C_INT,j);				// ̐vbV
		push_val(st->stack,C_INT,st->stack->defsp);		// ݂̊X^bN|C^vbV
		push_ptr(st->stack,C_PTR,st->script);		// ݂̃XNvgvbV
		push_ptr(st->stack,C_PTR,st->stack->var_function);	// ݂̊֐ˑϐvbV
		push_val(st->stack,C_RETINFO,st->pos);			// ݂̃XNvgʒuvbV

		st->pos=pos;
		st->stack->defsp=st->start+5+j;
		st->state=GOTO;
		st->stack->var_function = (struct linkdb_node**)aCalloc(1, sizeof(struct linkdb_node*));

		// ' ϐ̈p
		for(i = 0; i < j; i++) {
			struct script_data *s = &st->stack->stack_data[st->stack->sp-6-i];
			if( s->type == C_NAME && !s->ref ) {
				char *name = str_buf+str_data[s->u.num&0x00ffffff].str;
				// '@ ϐ̈p
				if( name[0] == '\'' && name[1] == '@' ) {
					s->ref = oldval;
				}
			}
		}
	}
	return 0;
}

/*==========================================
 * ̎擾
 *------------------------------------------
 */
int buildin_getarg(struct script_state *st)
{
	int num=conv_num(st,& (st->stack->stack_data[st->start+2]));
	int max,stsp;

	if( st->stack->defsp<5 || st->stack->stack_data[st->stack->defsp-1].type!=C_RETINFO ){
		printf("buildin_getarg: without callfunc or callsub!\n");
		st->state=END;
		return 0;
	}
	max=conv_num(st,& (st->stack->stack_data[st->stack->defsp-5]));
	stsp=st->stack->defsp - max - 5;
	if( num >= max ){
		printf("buildin_getarg: arg1(%d) out of range(%d) !\n",num,max);
		st->state=END;
		return 0;
	}
	push_copy(st->stack,stsp+num);
	return 0;
}

/*==========================================
 * ̑擾
 *------------------------------------------
 */
int buildin_getargcount(struct script_state *st)
{
	int max;

	if( st->stack->defsp<5 || st->stack->stack_data[st->stack->defsp-1].type!=C_RETINFO ){
		printf("buildin_getargcount: without callfunc or callsub!\n");
		st->state=END;
		return 0;
	}
	max=conv_num(st,& (st->stack->stack_data[st->stack->defsp-5]));

	push_val(st->stack,C_INT,max);
	return 0;
}

/*==========================================
 * Tu[`/[U[`֐̏I
 *------------------------------------------
 */
int buildin_return(struct script_state *st)
{
	if(st->end>st->start+2){	// ߂lL
		struct script_data *sd;

		push_copy(st->stack,st->start+2);
		sd = &st->stack->stack_data[st->stack->sp-1];

		if(sd->type == C_NAME) {
			char *name = str_buf + str_data[sd->u.num&0x00ffffff].str;
			if( name[0] == '\'' ) {
				if( name[1] == '@' ) {
					// '@ ϐQƓnɂƊ댯Ȃ̂Œlnɂ
					get_val(st,sd);
					if(isstr(sd)) {		// ̏ꍇaStrdupȂƂȂ
						sd->type  = C_STR;
						sd->u.str = (char *)aStrdup(sd->u.str);
					}
				} else if( !sd->ref ) {
					// ' ϐ͎QƓnłǂAQƌݒ肳ĂȂ
					// ̃XNvg̒lĂ܂̂ŕ␳
					sd->ref = &st->script->script_vars;
				}
			}
		}
	}
	st->state=RETFUNC;
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_next(struct script_state *st)
{
	st->state=STOP;
	clif_scriptnext(script_rid2sd(st),st->oid);
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_close(struct script_state *st)
{
	st->state=END;
	clif_scriptclose(script_rid2sd(st),st->oid);
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_close2(struct script_state *st)
{
	st->state=STOP;
	clif_scriptclose(script_rid2sd(st),st->oid);
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_menu(struct script_state *st)
{
	int i;
	struct map_session_data *sd = script_rid2sd(st);

	if(sd == NULL) {	// LZɂ
		st->state = END;
		return 0;
	}

	if(sd->state.menu_or_input == 0) {
		char *buf;
		st->state = RERUNLINE;
		sd->state.menu_or_input = 1;

		if((st->end - st->start - 2) % 2 == 1) {
			// ̐Ȃ̂ŃG[
			printf("buildin_menu: illigal argument count(%d).\n", st->end - st->start - 2);
			sd->state.menu_or_input = 0;
			st->state = END;
			return 0;
		}
		if(st->end > st->start+3) {
			size_t len = 0;
			for(i=st->start+2; i<st->end; i+=2) {
				conv_str(st,& (st->stack->stack_data[i]));
				len += strlen(st->stack->stack_data[i].u.str) + 1;
			}
			buf = (char *)aCalloc(len + 1, sizeof(char));
			for(i=st->start+2; i<st->end; i+=2) {
				if(st->stack->stack_data[i].u.str[0]) {
					if(buf[0]) {
						strcat(buf,":");
					}
					strcat(buf,st->stack->stack_data[i].u.str);
				}
			}
			clif_scriptmenu(sd,st->oid,buf);
			aFree(buf);
		} else {
			buf = conv_str(st,& (st->stack->stack_data[st->start+2]));
			clif_scriptmenu(sd,st->oid,buf);
		}
	} else if(sd->npc_menu <= 0 || sd->npc_menu == 0xff) {	// invalid or cancel
		sd->state.menu_or_input = 0;
		st->state = END;
	} else {	// goto
		int count = 0;
		sd->state.menu_or_input = 0;
		// 󕶎 : ̃j[␳
		for(i=st->start+2; count < sd->npc_menu && i<st->end; i+=2) {
			char *p  = conv_str(st,& (st->stack->stack_data[i]));
			char *np = NULL;
			count++;
			if(*p == '\0') {
				sd->npc_menu++;
				continue;
			}
			while(count <= sd->npc_menu && (p = strchr(p,':')) != NULL) {
				count++;
				if(np == p) {
					sd->npc_menu++;
				}
				p = np + 1;
			}
		}
		if(count >= sd->npc_menu) {
			pc_setreg(sd,add_str("@menu"),sd->npc_menu);
			if(st->stack->stack_data[i-1].type != C_POS) {
				printf("buildin_menu: not label !\n");
				st->state = END;
				return 0;
			}
			st->pos   = conv_num(st,& (st->stack->stack_data[i-1]));
			st->state = GOTO;
		} else {
			// sȒlȂ̂ŃLZ
			st->state = END;
		}
	}
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_rand(struct script_state *st)
{
	int range;

	if(st->end>st->start+3){
		int min=conv_num(st,& (st->stack->stack_data[st->start+2]));
		int max=conv_num(st,& (st->stack->stack_data[st->start+3]));
		if(max<min){
			int tmp;
			tmp=min;
			min=max;
			max=tmp;
		}
		range=max-min+1;
		push_val(st->stack,C_INT,atn_rand()%range+min);
	} else {
		range=conv_num(st,& (st->stack->stack_data[st->start+2]));
		if(range != 0)
			push_val(st->stack,C_INT,atn_rand()%range);
		else
			push_val(st->stack,C_INT,atn_rand());
	}
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
static int script_warp(struct map_session_data *sd, const char *mapname, int x, int y)
{
	nullpo_retr(0, sd);

	if(strcmp(mapname, "Random") == 0) {
		pc_randomwarp(sd, 3);
	} else if(strcmp(mapname, "SavePoint") == 0) {
		pc_setpos(sd, sd->status.save_point.map, sd->status.save_point.x, sd->status.save_point.y, 3);
	} else {
		if(pc_setpos(sd, mapname, x, y, 0)) {
			// ŝ .gat tăgC
			char *str = (char *)aMalloc(strlen(mapname) + 5);
			memcpy(str, mapname, strlen(mapname) + 1);
			strcat(str, ".gat");
			pc_setpos(sd, str, x, y, 0);
			aFree(str);
		}
	}

	if(unit_isdead(&sd->bl)) {
		pc_setstand(sd);
		pc_setrestartvalue(sd, 3);
	}

	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_warp(struct script_state *st)
{
	int x,y;
	char *str;
	struct block_list *bl = map_id2bl(st->rid);

	nullpo_retr(0, bl);

	str=conv_str(st,& (st->stack->stack_data[st->start+2]));
	x=conv_num(st,& (st->stack->stack_data[st->start+3]));
	y=conv_num(st,& (st->stack->stack_data[st->start+4]));

	if(bl->type == BL_PC) {
		script_warp((struct map_session_data *)bl, str, x, y);
	} else if(bl->type == BL_MOB) {
		struct mob_data *md = (struct mob_data *)bl;

		if(strcmp(str, "Random") == 0) {
			mob_warp(md, -1, -1, -1, 3);
		} else {
			int m = map_mapname2mapid(str);
			if(m >= 0)
				mob_warp(md, m, x, y, 3);
		}
	}
	return 0;
}

/*==========================================
 * MAPw胏[v
 *------------------------------------------
 */
static int buildin_mapwarp_sub(struct block_list *bl,va_list ap)
{
	int x,y;
	char *mapname;
	struct map_session_data *sd;

	nullpo_retr(0, bl);
	nullpo_retr(0, sd = (struct map_session_data *)bl);

	mapname = va_arg(ap, char *);
	x = va_arg(ap,int);
	y = va_arg(ap,int);

	script_warp(sd, mapname, x, y);

	return 0;
}

int buildin_mapwarp(struct script_state *st)
{
	int x,y,m;
	char *mapname, *str;

	mapname=conv_str(st,& (st->stack->stack_data[st->start+2]));
	str=conv_str(st,& (st->stack->stack_data[st->start+3]));
	x=conv_num(st,& (st->stack->stack_data[st->start+4]));
	y=conv_num(st,& (st->stack->stack_data[st->start+5]));

	m = script_mapname2mapid(st,mapname);
	if(m >= 0)
		map_foreachinarea(buildin_mapwarp_sub,m,0,0,map[m].xs,map[m].ys,BL_PC,str,x,y);
	return 0;
}

/*==========================================
 * GAw胏[v
 *------------------------------------------
 */
int buildin_areawarp(struct script_state *st)
{
	int x,y,m;
	char *mapname, *str;
	int x0,y0,x1,y1;

	mapname=conv_str(st,& (st->stack->stack_data[st->start+2]));
	x0=conv_num(st,& (st->stack->stack_data[st->start+3]));
	y0=conv_num(st,& (st->stack->stack_data[st->start+4]));
	x1=conv_num(st,& (st->stack->stack_data[st->start+5]));
	y1=conv_num(st,& (st->stack->stack_data[st->start+6]));
	str=conv_str(st,& (st->stack->stack_data[st->start+7]));
	x=conv_num(st,& (st->stack->stack_data[st->start+8]));
	y=conv_num(st,& (st->stack->stack_data[st->start+9]));

	m = script_mapname2mapid(st,mapname);
	if(m >= 0)
		map_foreachinarea(buildin_mapwarp_sub,m,x0,y0,x1,y1,BL_PC,str,x,y);
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_heal(struct script_state *st)
{
	int hp,sp;

	hp=conv_num(st,& (st->stack->stack_data[st->start+2]));
	sp=conv_num(st,& (st->stack->stack_data[st->start+3]));

	pc_heal(script_rid2sd(st),hp,sp);
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_itemheal(struct script_state *st)
{
	int hp,sp;

	hp=conv_num(st,& (st->stack->stack_data[st->start+2]));
	sp=conv_num(st,& (st->stack->stack_data[st->start+3]));

	pc_itemheal(script_rid2sd(st),hp,sp);
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_percentheal(struct script_state *st)
{
	int hp,sp;

	hp=conv_num(st,& (st->stack->stack_data[st->start+2]));
	sp=conv_num(st,& (st->stack->stack_data[st->start+3]));

	pc_percentheal(script_rid2sd(st),hp,sp);
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_jobchange(struct script_state *st)
{
	int job, upper=-1;

	job=conv_num(st,& (st->stack->stack_data[st->start+2]));
	if( st->end>st->start+3 )
		upper=conv_num(st,& (st->stack->stack_data[st->start+3]));

	if(job >= 0 && job < MAX_VALID_PC_CLASS)
		pc_jobchange(script_rid2sd(st),job, upper);

	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_input(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	int num;
	char *name;
	char postfix;

	if(sd == NULL) {	// G[ɂ
		st->state = END;
		return 0;
	}
	if(st->stack->stack_data[st->start+2].type != C_NAME) {
		// ϐł͂Ȃ̂ŃG[
		printf("buildin_input: param not name\n");
		st->state = END;
		return 0;
	}
	num     = st->stack->stack_data[st->start+2].u.num;
	name    = str_buf+str_data[num&0x00ffffff].str;
	postfix = name[strlen(name)-1];

	if(sd->state.menu_or_input) {
		sd->state.menu_or_input=0;
		if(postfix == '$')
			set_reg(st,sd,num,name,(void*)sd->npc_str,st->stack->stack_data[st->start+2].ref);
		else
			set_reg(st,sd,num,name,(void*)sd->npc_amount,st->stack->stack_data[st->start+2].ref);
	} else {
		st->state=RERUNLINE;
		if(postfix == '$')
			clif_scriptinputstr(sd,st->oid);
		else
			clif_scriptinput(sd,st->oid);
		sd->state.menu_or_input=1;
	}
	return 0;
}

/*==========================================
 * ϐݒ
 *------------------------------------------
 */
int buildin_set(struct script_state *st)
{
	struct map_session_data *sd = NULL;
	int num;
	char *name;
	char prefix, postfix;

	if( st->stack->stack_data[st->start+2].type != C_NAME ) {
		printf("buildin_set: param not name\n");
		return 0;
	}
	num     = st->stack->stack_data[st->start+2].u.num;
	name    = str_buf+str_data[num&0x00ffffff].str;
	prefix  = *name;
	postfix = name[strlen(name)-1];

	if( prefix != '$' && prefix != '\'' )
		sd = script_rid2sd(st);

	if( postfix == '$' ) {
		// 
		char *str = conv_str(st,& (st->stack->stack_data[st->start+3]));
		set_reg(st,sd,num,name,(void*)str,st->stack->stack_data[st->start+2].ref);
	} else {
		// l
		int val = conv_num(st,& (st->stack->stack_data[st->start+3]));
		set_reg(st,sd,num,name,(void*)val,st->stack->stack_data[st->start+2].ref);
	}

	return 0;
}

/*==========================================
 * zϐݒ
 *------------------------------------------
 */
int buildin_setarray(struct script_state *st)
{
	struct map_session_data *sd = NULL;
	int num;
	char *name;
	char prefix, postfix;
	int i,j,max;

	if( st->stack->stack_data[st->start+2].type != C_NAME ) {
		printf("buildin_setarray: param not name\n");
		return 0;
	}
	num     = st->stack->stack_data[st->start+2].u.num;
	name    = str_buf+str_data[num&0x00ffffff].str;
	prefix  = *name;
	postfix = name[strlen(name)-1];

	if( prefix != '$' && prefix != '@' && prefix != '\'' ) {
		printf("buildin_setarray: illegal scope !\n");
		return 0;
	}
	if( prefix != '$' && prefix != '\'' )
		sd = script_rid2sd(st);

	max = 128 - (num>>24);
	for(j=0,i=st->start+3; i<st->end && j<max; i++,j++){
		void *v;
		if( postfix == '$' )
			v = (void*)conv_str(st,& (st->stack->stack_data[i]));
		else
			v = (void*)conv_num(st,& (st->stack->stack_data[i]));
		set_reg( st, sd, num+(j<<24), name, v, st->stack->stack_data[st->start+2].ref);
	}
	return 0;
}

/*==========================================
 * zϐNA
 *------------------------------------------
 */
int buildin_cleararray(struct script_state *st)
{
	struct map_session_data *sd = NULL;
	int num;
	char *name;
	char prefix, postfix;
	int i, sz;
	void *v;

	if( st->stack->stack_data[st->start+2].type != C_NAME ) {
		printf("buildin_cleararray: param not name\n");
		return 0;
	}
	num     = st->stack->stack_data[st->start+2].u.num;
	name    = str_buf+str_data[num&0x00ffffff].str;
	prefix  = *name;
	postfix = name[strlen(name)-1];

	if( prefix != '$' && prefix != '@' && prefix != '\'' ) {
		printf("buildin_cleararray: illegal scope !\n");
		return 0;
	}
	if( prefix != '$' && prefix != '\'' )
		sd = script_rid2sd(st);

	sz = conv_num(st,& (st->stack->stack_data[st->start+4]));
	if(sz > 128 - (num>>24))
		sz = 128 - (num>>24);

	if( postfix == '$' )
		v = (void*)conv_str(st,& (st->stack->stack_data[st->start+3]));
	else
		v = (void*)conv_num(st,& (st->stack->stack_data[st->start+3]));

	for(i=0;i<sz;i++)
		set_reg(st,sd,num+(i<<24),name,v,st->stack->stack_data[st->start+2].ref);
	return 0;
}

/*==========================================
 * zϐRs[
 *------------------------------------------
 */
int buildin_copyarray(struct script_state *st)
{
	struct map_session_data *sd = NULL;
	int num, num2;
	char *name, *name2;
	char prefix, postfix, prefix2, postfix2;
	int i, sz;

	if( st->stack->stack_data[st->start+2].type != C_NAME ||
	    st->stack->stack_data[st->start+3].type != C_NAME )
	{
		printf("buildin_copyarray: param not name\n");
		return 0;
	}
	num      = st->stack->stack_data[st->start+2].u.num;
	name     = str_buf+str_data[num&0x00ffffff].str;
	prefix   = *name;
	postfix  = name[strlen(name)-1];
	num2     = st->stack->stack_data[st->start+3].u.num;
	name2    = str_buf+str_data[num2&0x00ffffff].str;
	prefix2  = *name2;
	postfix2 = name2[strlen(name2)-1];

	if( prefix != '$' && prefix != '@' && prefix != '\'' ) {
		printf("buildin_copyarray: illegal scope !\n");
		return 0;
	}
	if( prefix2 != '$' && prefix2 != '@' && prefix2 != '\'' ) {
		printf("buildin_copyarray: illegal scope !\n");
		return 0;
	}
	if( postfix != postfix2 && (postfix == '$' || postfix2 == '$') ) {
		printf("buildin_copyarray: type mismatch !\n");
		return 0;
	}
	if( (prefix != '$' && prefix != '\'') || (prefix2 != '$' && prefix2 != '\'') )
		sd = script_rid2sd(st);

	sz = conv_num(st,& (st->stack->stack_data[st->start+4]));
	if(sz > 128 - (num>>24))
		sz = 128 - (num>>24);

	if((num & 0x00FFFFFF) == (num2 & 0x00FFFFFF) && (num & 0xFF000000) > (num2 & 0xFF000000)) {
		// zŁAnum > num2 ̏ꍇ傫Rs[ȂƂȂ
		for(i=sz-1; i>=0; i--)
			set_reg(
				st,sd,num+(i<<24),name,
				get_val2(st,num2+(i<<24),st->stack->stack_data[st->start+3].ref),
				st->stack->stack_data[st->start+2].ref
			);
	} else {
		for(i=0; i<sz; i++) {
			if((num2>>24)+i < 128) {
				set_reg(
					st,sd,num+(i<<24),name,
					get_val2(st,num2+(i<<24),st->stack->stack_data[st->start+3].ref),
					st->stack->stack_data[st->start+2].ref
				);
			} else {	// num2z𒴂c0Ŗ߂
				set_reg(
					st,sd,num+(i<<24),name,
					(postfix == '$')? "": 0,
					st->stack->stack_data[st->start+2].ref
				);
			}
		}
	}
	return 0;
}

/*==========================================
 * zϐ̃TCY擾
 *------------------------------------------
 */
static int getarraysize(struct script_state *st,int num,char postfix,struct linkdb_node** ref)
{
	int i = num>>24;
	int c = (i == 0)? -1: i;

	for(; i<128; i++) {
		void *v = get_val2(st,(num & 0x00FFFFFF)+(i<<24),ref);
		if( postfix == '$' && *((char*)v) )
			c = i;
		if( postfix != '$' && (int)v )
			c = i;
	}
	return c+1;
}

int buildin_getarraysize(struct script_state *st)
{
	int num;
	char *name;
	char prefix, postfix;

	if( st->stack->stack_data[st->start+2].type != C_NAME ) {
		printf("buildin_getarraysize: param not name\n");
		push_val(st->stack,C_INT,0);
		return 0;
	}
	num     = st->stack->stack_data[st->start+2].u.num;
	name    = str_buf+str_data[num&0x00ffffff].str;
	prefix  = *name;
	postfix = name[strlen(name)-1];

	if( prefix != '$' && prefix != '@' && prefix != '\'' ) {
		printf("buildin_getarraysize: illegal scope !\n");
		push_val(st->stack,C_INT,0);
		return 0;
	}

	push_val(st->stack,C_INT,getarraysize(st,num,postfix,st->stack->stack_data[st->start+2].ref));
	return 0;
}

/*==========================================
 * zϐvf폜
 *------------------------------------------
 */
int buildin_deletearray(struct script_state *st)
{
	struct map_session_data *sd = NULL;
	int num;
	char *name;
	char prefix, postfix;
	int i,j,count,sz;

	if( st->stack->stack_data[st->start+2].type != C_NAME ) {
		printf("buildin_deletearray: param not name\n");
		return 0;
	}
	num     = st->stack->stack_data[st->start+2].u.num;
	name    = str_buf+str_data[num&0x00ffffff].str;
	prefix  = *name;
	postfix = name[strlen(name)-1];

	if( prefix != '$' && prefix != '@' && prefix != '\'' ) {
		printf("buildin_deletearray: illegal scope !\n");
		return 0;
	}
	if( prefix != '$' && prefix != '\'' )
		sd = script_rid2sd(st);

	count = conv_num(st,& (st->stack->stack_data[st->start+3]));
	sz = getarraysize(st,num,postfix,st->stack->stack_data[st->start+2].ref)-(num>>24)-count;

	for(i=0;i<sz;i++){
		set_reg(
			st,sd,num+(i<<24),name,
			get_val2(st,num+((i+count)<<24),st->stack->stack_data[st->start+2].ref),
			st->stack->stack_data[st->start+2].ref
		);
	}
	if(count > 128 - (num>>24))
		count = 128 - (num>>24);
	for(j=0;j<count;j++){
		set_reg(st,sd,num+((i+j)<<24),name, (postfix == '$')? "": 0, st->stack->stack_data[st->start+2].ref);
	}
	return 0;
}

/*==========================================
 * z𕶎Ƃďo
 *------------------------------------------
 */
int buildin_printarray(struct script_state *st)
{
	struct map_session_data *sd = NULL;
	int num;
	char *name;
	char prefix, postfix;
	int i, count = 0;
	size_t len = 0;
	char *buf;
	const char *delimiter = ":";
	void *list[128];

	if( st->stack->stack_data[st->start+2].type != C_NAME ) {
		printf("buildin_printarray: param not name\n");
		return 0;
	}
	num     = st->stack->stack_data[st->start+2].u.num;
	name    = str_buf+str_data[num&0x00ffffff].str;
	prefix  = *name;
	postfix = name[strlen(name)-1];

	if( prefix != '$' && prefix != '@' && prefix != '\'' ) {
		printf("buildin_printarray: illegal scope !\n");
		return 0;
	}
	if( prefix != '$' && prefix != '\'' )
		sd = script_rid2sd(st);

	if(st->end > st->start+3)
		delimiter = conv_str(st,& (st->stack->stack_data[st->start+3]));

	// getarraysize
	for(i = 0; i < 128 - (num>>24); i++) {
		void *v = get_val2(st, num+(i<<24), st->stack->stack_data[st->start+2].ref);

		list[i] = v;
		if( postfix == '$' && *(char*)v ) {
			len += strlen((char*)v);
			count = i + 1;
		}
		if( postfix != '$' && (int)v ) {
			len += 16;
			count = i + 1;
		}
	}

	if(count > 0) {
		len += (count - 1) * strlen(delimiter);
	}
	buf = (char *)aCalloc(len + 1, sizeof(char));

	for(i = 0; i < count; i++) {
		if(i > 0) {
			strcat(buf, delimiter);
		}
		if( postfix == '$' ) {
			strcat(buf, (char*)list[i]);
		} else {
			char val[16];
			sprintf(val, "%d", (int)list[i]);
			strcat(buf, val);
		}
	}

	push_str(st->stack, C_STR, buf);
	return 0;
}

/*==========================================
 * wvf\l(L[)擾
 *------------------------------------------
 */
int buildin_getelementofarray(struct script_state *st)
{
	int i,j=0,count=0;
	int list[128];

	if( st->stack->stack_data[st->start+2].type != C_NAME ) {
		printf("buildin_getelementofarray: (operator[]) param1 not name !\n");
		push_val(st->stack,C_INT,0);
		return 0;
	}

	for(i=st->start+3; i<st->end && count < 128; i++) {
		j = conv_num(st,& (st->stack->stack_data[i]));
		if(j < 0 || j >= 128) {
			printf("buildin_getelementofarray: (operator[]) param illegal number %d\n",j);
			push_val(st->stack,C_INT,0);
			return 0;
		}
		if(j == 0 && count == 0)	// @hoge[0][1]  @hoge[1] Ƃēo^邽߂̏
			continue;

		if(i < st->end-1)
			list[count++] = j;
	}

	if(count == 0) {
		push_val2(st->stack,C_NAME,
			(j<<24) | (st->stack->stack_data[st->start+2].u.num&0x00ffffff),
			st->stack->stack_data[st->start+2].ref);
	} else {
		int num      = st->stack->stack_data[st->start+2].u.num;
		char *name   = str_buf+str_data[num&0x00ffffff].str;
		size_t len   = strlen(name);
		char postfix = name[len-1];
		char *var    = (char *)aCalloc(1+len+count*5, sizeof(char));	// 1ő [xxx] 5
		char *p;

		// postfix[]ϐ̂̂ɂ
		strcpy(var,name);
		if(postfix == '$')
			var[len-1] = 0;
		if((p = strchr(var,'[')) != NULL)
			*p = 0;

		p = var + strlen(var);
		for(i=0; i<count; i++) {
			p += sprintf(p,"[%d]",list[i]);
		}
		if(postfix == '$')
			strcat(var,"$");

		push_val2(st->stack,C_NAME,(j<<24) | add_str(var),st->stack->stack_data[st->start+2].ref);
		aFree(var);
	}
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_setlook(struct script_state *st)
{
	int type,val;

	type=conv_num(st,& (st->stack->stack_data[st->start+2]));
	val=conv_num(st,& (st->stack->stack_data[st->start+3]));

	pc_changelook(script_rid2sd(st),type,val);

	return 0;
}

/*==========================================
 * ڂ̃p[^Ԃ
 *   nF1,^A2,A3,iA4,iA5,iA6,FA7,FA8,
 *------------------------------------------
 */
int buildin_getlook(struct script_state *st)
{
	int type,val= -1;
	struct map_session_data *sd=script_rid2sd(st);
	type=conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(sd==NULL){
		push_val(st->stack,C_INT,-1);
		return 0;
	}
	switch(type){
		case LOOK_HAIR:
			val=sd->status.hair;
			break;
		case LOOK_WEAPON:
			val=sd->status.weapon;
			break;
		case LOOK_HEAD_BOTTOM:
			val=sd->status.head_bottom;
			break;
		case LOOK_HEAD_TOP:
			val=sd->status.head_top;
			break;
		case LOOK_HEAD_MID:
			val=sd->status.head_mid;
			break;
		case LOOK_HAIR_COLOR:
			val=sd->status.hair_color;
			break;
		case LOOK_CLOTHES_COLOR:
			val=sd->status.clothes_color;
			break;
		case LOOK_SHIELD:
			val=sd->status.shield;
			break;
		case LOOK_SHOES:
			break;
		case LOOK_MOB:
			break;
	}
	push_val(st->stack,C_INT,val);
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_cutin(struct script_state *st)
{
	char *str = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int type  = conv_num(st,& (st->stack->stack_data[st->start+3]));

	clif_cutin(script_rid2sd(st),str,type);

	return 0;
}

/*==========================================
 * J[h̃CXg\
 *------------------------------------------
 */
int buildin_cutincard(struct script_state *st)
{
	int itemid = conv_num(st,& (st->stack->stack_data[st->start+2]));

	clif_cutin(script_rid2sd(st),itemdb_search(itemid)->cardillustname,4);

	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_viewpoint(struct script_state *st)
{
	int type,x,y,id,color;

	type=conv_num(st,& (st->stack->stack_data[st->start+2]));
	x=conv_num(st,& (st->stack->stack_data[st->start+3]));
	y=conv_num(st,& (st->stack->stack_data[st->start+4]));
	id=conv_num(st,& (st->stack->stack_data[st->start+5]));
	color=conv_num(st,& (st->stack->stack_data[st->start+6]));

	clif_viewpoint(script_rid2sd(st),st->oid,type,x,y,id,color);

	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_countitem(struct script_state *st)
{
	int nameid=0, count=0, i;
	struct map_session_data *sd = script_rid2sd(st);
	struct script_data *data;

	data = &(st->stack->stack_data[st->start+2]);
	get_val(st,data);
	if( isstr(data) ) {
		const char *name = conv_str(st,data);
		struct item_data *item_data = itemdb_searchname(name);
		if(item_data)
			nameid = item_data->nameid;
	} else {
		nameid = conv_num(st,data);
	}

	if(sd && nameid > 0) {
		for(i=0; i<MAX_INVENTORY; i++) {
			if(sd->status.inventory[i].nameid == nameid)
				count += sd->status.inventory[i].amount;
		}
	}
	push_val(st->stack,C_INT,count);

	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_countcartitem(struct script_state *st)
{
	int nameid=0, count=0, i;
	struct map_session_data *sd = script_rid2sd(st);
	struct script_data *data;

	data = &(st->stack->stack_data[st->start+2]);
	get_val(st,data);
	if( isstr(data) ) {
		const char *name = conv_str(st,data);
		struct item_data *item_data = itemdb_searchname(name);
		if(item_data)
			nameid = item_data->nameid;
	} else {
		nameid = conv_num(st,data);
	}

	if(sd && nameid > 0) {
		for(i=0; i<MAX_CART; i++) {
			if(sd->status.cart[i].nameid == nameid)
				count += sd->status.cart[i].amount;
		}
	}
	push_val(st->stack,C_INT,count);

	return 0;
}

/*==========================================
 * dʃ`FbN
 *------------------------------------------
 */
int buildin_checkweight(struct script_state *st)
{
	int nameid=0, amount;
	struct map_session_data *sd = script_rid2sd(st);
	struct script_data *data;

	data = &(st->stack->stack_data[st->start+2]);
	get_val(st,data);
	if( isstr(data) ) {
		const char *name = conv_str(st,data);
		struct item_data *item_data = itemdb_searchname(name);
		if(item_data)
			nameid = item_data->nameid;
	} else {
		nameid = conv_num(st,data);
	}

	amount = conv_num(st,& (st->stack->stack_data[st->start+3]));

	if(sd && nameid > 0) {
		if(amount <= MAX_AMOUNT &&
		   itemdb_weight(nameid)*amount + sd->weight <= sd->max_weight &&
		   pc_search_inventory(sd,0) >= 0)
		{
			push_val(st->stack,C_INT,1);
			return 0;
		}
	}
	push_val(st->stack,C_INT,0);
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_getitem(struct script_state *st)
{
	int nameid=0, flag=0, ret=0, i=0, amount;
	struct map_session_data *sd;
	struct script_data *data;
	struct item item_tmp;

	sd = script_rid2sd(st);

	data = &(st->stack->stack_data[st->start+2]);
	get_val(st,data);
	if( isstr(data) ) {
		const char *name = conv_str(st,data);
		struct item_data *item_data = itemdb_searchname(name);
		if(item_data)
			nameid = item_data->nameid;
	} else {
		nameid = conv_num(st,data);
	}

	amount = conv_num(st,& (st->stack->stack_data[st->start+3]));
	if(st->end > st->start+4)	// Ӓ肵Ԃœnǂ
		flag = (conv_num(st,& (st->stack->stack_data[st->start+4])) == 0)? 1: 0;
	if(st->end > st->start+5)	// ACew肵IDɓn
		sd = map_id2sd(conv_num(st,& (st->stack->stack_data[st->start+5])));
	if(sd == NULL)			// ACen肪Ȃ炨A
		return 0;

	do {
		memset(&item_tmp,0,sizeof(item_tmp));

		if(nameid < 0)		// _
			item_tmp.nameid = itemdb_searchrandomid(-nameid);
		else
			item_tmp.nameid = nameid;

		if(!itemdb_exists(item_tmp.nameid))
			break;

		if(!flag || battle_config.itemidentify)
			item_tmp.identify = 1;
		else
			item_tmp.identify = !itemdb_isequip3(item_tmp.nameid);

		if((ret = pc_additem(sd,&item_tmp,(nameid < 0)? 1: amount))) {
			clif_additem(sd,0,0,ret);
			if(!pc_candrop(sd,item_tmp.nameid))
				map_addflooritem(&item_tmp,(nameid < 0)? 1: amount,sd->bl.m,sd->bl.x,sd->bl.y,0,0,0,0);
		}
	} while(nameid < 0 && ++i < amount);	// _n̓ACe̍ĒI

	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_getitem2(struct script_state *st)
{
	int nameid=0, ret=0, i=0, amount;
	int iden,ref,attr,c1,c2,c3,c4;
	unsigned int limit=0;
	struct map_session_data *sd;
	struct script_data *data;
	struct item_data *item_data;
	struct item item_tmp;

	sd = script_rid2sd(st);

	data = &(st->stack->stack_data[st->start+2]);
	get_val(st,data);
	if( isstr(data) ) {
		const char *name = conv_str(st,data);
		struct item_data *item_data = itemdb_searchname(name);
		if(item_data)
			nameid = item_data->nameid;
	} else {
		nameid = conv_num(st,data);
	}

	amount = conv_num(st,& (st->stack->stack_data[st->start+3]));
	iden   = conv_num(st,& (st->stack->stack_data[st->start+4]));
	ref    = conv_num(st,& (st->stack->stack_data[st->start+5]));
	attr   = conv_num(st,& (st->stack->stack_data[st->start+6]));
	c1     = conv_num(st,& (st->stack->stack_data[st->start+7]));
	c2     = conv_num(st,& (st->stack->stack_data[st->start+8]));
	c3     = conv_num(st,& (st->stack->stack_data[st->start+9]));
	c4     = conv_num(st,& (st->stack->stack_data[st->start+10]));
	if(st->end > st->start+11)
		limit = (unsigned int)conv_num(st,& (st->stack->stack_data[st->start+11]));
	if(st->end > st->start+12)	// ACew肵IDɓn
		sd = map_id2sd(conv_num(st,& (st->stack->stack_data[st->start+12])));
	if(sd == NULL)			// ACen肪Ȃ炨A
		return 0;

	do {
		memset(&item_tmp,0,sizeof(item_tmp));

		if(nameid < 0)		// _
			item_tmp.nameid = itemdb_searchrandomid(-nameid);
		else
			item_tmp.nameid = nameid;

		if(!itemdb_exists(item_tmp.nameid))
			break;

		item_data = itemdb_search(item_tmp.nameid);

		if(item_data->type == 4 || item_data->type == 5) {
			if(ref > MAX_REFINE)
				ref = MAX_REFINE;
		} else if(item_data->type == 7) {
			iden = 1;
			ref = 0;
		} else {
			iden = 1;
			ref = attr = 0;
		}

		if(iden || battle_config.itemidentify)
			item_tmp.identify = 1;
		else
			item_tmp.identify = !itemdb_isequip3(item_tmp.nameid);

		item_tmp.refine    = ref;
		item_tmp.attribute = attr;
		item_tmp.card[0]   = c1;
		item_tmp.card[1]   = c2;
		item_tmp.card[2]   = c3;
		item_tmp.card[3]   = c4;
		item_tmp.limit     = (limit > 0)? (unsigned int)time(NULL) + limit: 0;

		if((ret = pc_additem(sd,&item_tmp,(nameid < 0)? 1: amount))) {
			clif_additem(sd,0,0,ret);
			if(!pc_candrop(sd,item_tmp.nameid))
				map_addflooritem(&item_tmp,(nameid < 0)? 1: amount,sd->bl.m,sd->bl.x,sd->bl.y,0,0,0,0);
		}
	} while(nameid < 0 && ++i < amount);	// _n̓ACe̍ĒI

	return 0;
}

/*==========================================
 * wID̃ACe폜
 *------------------------------------------
 */
int buildin_delitem(struct script_state *st)
{
	int nameid=0,amount,i;
	int skip_egg = 0;
	struct map_session_data *sd = script_rid2sd(st);
	struct script_data *data;

	nullpo_retr(0, sd);

	data = &(st->stack->stack_data[st->start+2]);
	get_val(st,data);
	if( isstr(data) ) {
		const char *name = conv_str(st,data);
		struct item_data *item_data = itemdb_searchname(name);
		if(item_data)
			nameid = item_data->nameid;
	} else {
		nameid = conv_num(st,data);
	}
	if(nameid <= 0)
		return 0;

	amount = conv_num(st,& (st->stack->stack_data[st->start+3]));
	if(st->end > st->start+4)
		skip_egg = conv_num(st,& (st->stack->stack_data[st->start+4]));

	for(i=0; i<MAX_INVENTORY; i++) {
		if(sd->status.inventory[i].nameid == nameid) {
			if(!skip_egg &&
			   sd->inventory_data[i]->type == 7 &&
			   sd->status.inventory[i].amount > 0 &&
			   sd->status.inventory[i].card[0] == (short)0xff00 &&
			   search_petDB_index(nameid, PET_EGG) >= 0)
			{
				intif_delete_petdata(*((long *)(&sd->status.inventory[i].card[1])));
			}

			if(sd->status.inventory[i].amount >= amount) {
				pc_delitem(sd,i,amount,0);
				break;
			} else {
				amount -= sd->status.inventory[i].amount;
				pc_delitem(sd,i,sd->status.inventory[i].amount,0);
			}
		}
	}

	return 0;
}

/*==========================================
 * wID̃ACeJ[g폜
 *------------------------------------------
 */
int buildin_delcartitem(struct script_state *st)
{
	int nameid=0,amount,i;
	int skip_egg = 0;
	struct map_session_data *sd = script_rid2sd(st);
	struct script_data *data;

	nullpo_retr(0, sd);

	data = &(st->stack->stack_data[st->start+2]);
	get_val(st,data);
	if( isstr(data) ) {
		const char *name = conv_str(st,data);
		struct item_data *item_data = itemdb_searchname(name);
		if(item_data)
			nameid = item_data->nameid;
	} else {
		nameid = conv_num(st,data);
	}
	if(nameid <= 0)
		return 0;

	amount = conv_num(st,& (st->stack->stack_data[st->start+3]));
	if(st->end > st->start+4)
		skip_egg = conv_num(st,& (st->stack->stack_data[st->start+4]));

	for(i=0; i<MAX_CART; i++) {
		if(sd->status.cart[i].nameid == nameid) {
			if(!skip_egg &&
			   itemdb_type(nameid) == 7 &&
			   sd->status.cart[i].amount > 0 &&
			   sd->status.cart[i].card[0] == (short)0xff00 &&
			   search_petDB_index(nameid, PET_EGG) >= 0)
			{
				intif_delete_petdata(*((long *)(&sd->status.cart[i].card[1])));
			}

			if(sd->status.cart[i].amount >= amount) {
				pc_cart_delitem(sd,i,amount,!pc_iscarton(sd));
				break;
			} else {
				amount -= sd->status.cart[i].amount;
				pc_cart_delitem(sd,i,sd->status.cart[i].amount,!pc_iscarton(sd));
			}
		}
	}

	return 0;
}

/*==========================================
 * indexԖڂ̃ACe폜
 *------------------------------------------
 */
int buildin_delitem2(struct script_state *st)
{
	int idx,amount;
	int skip_egg = 0;
	struct map_session_data *sd = script_rid2sd(st);

	nullpo_retr(0, sd);

	idx = conv_num(st,& (st->stack->stack_data[st->start+2]));
	amount = conv_num(st,& (st->stack->stack_data[st->start+3]));
	if(st->end > st->start+4)
		skip_egg = conv_num(st,& (st->stack->stack_data[st->start+4]));

	if(idx < 0 || idx >= MAX_INVENTORY || amount <= 0)
		return 0;

	if(!skip_egg &&
	   sd->status.inventory[idx].nameid > 0 &&
	   sd->inventory_data[idx]->type == 7 &&
	   sd->status.inventory[idx].amount > 0 &&
	   sd->status.inventory[idx].card[0] == (short)0xff00 &&
	   search_petDB_index(sd->status.inventory[idx].nameid, PET_EGG) >= 0)
	{
		intif_delete_petdata(*((long *)(&sd->status.inventory[idx].card[1])));
	}

	if(sd->status.inventory[idx].amount >= amount)
		pc_delitem(sd,idx,amount,0);
	else
		pc_delitem(sd,idx,sd->status.inventory[idx].amount,0);

	return 0;
}

/*==========================================
 * indexԖڂ̃ACeJ[g폜
 *------------------------------------------
 */
int buildin_delcartitem2(struct script_state *st)
{
	int idx,amount;
	int skip_egg = 0;
	struct map_session_data *sd = script_rid2sd(st);

	nullpo_retr(0, sd);

	idx = conv_num(st,& (st->stack->stack_data[st->start+2]));
	amount = conv_num(st,& (st->stack->stack_data[st->start+3]));
	if(st->end > st->start+4)
		skip_egg = conv_num(st,& (st->stack->stack_data[st->start+4]));

	if(idx < 0 || idx >= MAX_CART || amount <= 0)
		return 0;

	if(!skip_egg &&
	   sd->status.cart[idx].nameid > 0 &&
	   itemdb_type(sd->status.cart[idx].nameid) == 7 &&
	   sd->status.cart[idx].amount > 0 &&
	   sd->status.cart[idx].card[0] == (short)0xff00 &&
	   search_petDB_index(sd->status.cart[idx].nameid, PET_EGG) >= 0)
	{
		intif_delete_petdata(*((long *)(&sd->status.cart[idx].card[1])));
	}

	if(sd->status.cart[idx].amount >= amount)
		pc_cart_delitem(sd,idx,amount,!pc_iscarton(sd));
	else
		pc_cart_delitem(sd,idx,sd->status.cart[idx].amount,!pc_iscarton(sd));

	return 0;
}

/*==========================================
 * L֌W̑擾
 *------------------------------------------
 */
int buildin_equippeditem(struct script_state *st)
{
	int id;
	struct map_session_data *sd;

	id=conv_num(st,& (st->stack->stack_data[st->start+2]));
	if( st->end>st->start+3 )
		sd=map_nick2sd(conv_str(st,& (st->stack->stack_data[st->start+3])));
	else
		sd=script_rid2sd(st);

	if(sd==NULL){
		push_val(st->stack,C_INT,0);
		return 0;
	}

	push_val(st->stack,C_INT,pc_equippeditem(sd,id));

	return 0;
}

/*==========================================
 * L֌W̃p[^擾
 *------------------------------------------
 */
int buildin_readparam(struct script_state *st)
{
	int type;
	struct map_session_data *sd;

	type=conv_num(st,& (st->stack->stack_data[st->start+2]));
	if( st->end>st->start+3 )
		sd=map_nick2sd(conv_str(st,& (st->stack->stack_data[st->start+3])));
	else
		sd=script_rid2sd(st);

	if(sd==NULL){
		push_val(st->stack,C_INT,-1);
		return 0;
	}

	push_val(st->stack,C_INT,pc_readparam(sd,type));

	return 0;
}

/*==========================================
 * L֌WID擾
 *------------------------------------------
 */
int buildin_getcharid(struct script_state *st)
{
	int num, id = -1;
	struct map_session_data *sd;

	num = conv_num(st,& (st->stack->stack_data[st->start+2]));
	if(st->end > st->start+3)
		sd = map_nick2sd(conv_str(st,& (st->stack->stack_data[st->start+3])));
	else
		sd = script_rid2sd(st);

	if(sd) {
		switch(num) {
			case 0: id = sd->status.char_id;    break;
			case 1: id = sd->status.party_id;   break;
			case 2: id = sd->status.guild_id;   break;
			case 3: id = sd->status.account_id; break;
		}
	}
	push_val(st->stack,C_INT,id);

	return 0;
}

/*==========================================
 * wIDPT擾
 *------------------------------------------
 */
static char *buildin_getpartyname_sub(int party_id)
{
	struct party *p = party_search(party_id);

	if(p!=NULL){
		return (char *)aStrdup(p->name);
	}
	return NULL;
}

int buildin_getpartyname(struct script_state *st)
{
	char *name;
	int party_id;

	party_id=conv_num(st,& (st->stack->stack_data[st->start+2]));
	name=buildin_getpartyname_sub(party_id);
	if(name)
		push_str(st->stack,C_STR,name);
	else
		push_str(st->stack,C_CONSTSTR,"");

	return 0;
}

/*==========================================
 * wID̃LN^擾
 *------------------------------------------
 */
static char *buildin_getcharname_sub(int char_id)
{
	struct charid2nick *c = char_search(char_id);

	if(c!=NULL){
		return (char *)aStrdup(c->nick);
	}
	return NULL;
}

int buildin_getcharname(struct script_state *st)
{
	char *name;
	int char_id;

	char_id=conv_num(st,& (st->stack->stack_data[st->start+2]));
	name=buildin_getcharname_sub(char_id);

	if(name)
		push_str(st->stack,C_STR,name);
	else
		push_str(st->stack,C_CONSTSTR,"");

	return 0;
}

/*==========================================
 * wIDPTlƃo[ID擾
 *------------------------------------------
 */
int buildin_getpartymember(struct script_state *st)
{
	int j = 0;
	struct party *p = NULL;

	p = party_search(conv_num(st,& (st->stack->stack_data[st->start+2])));

	if(p) {
		int i,num;
		num = add_str("$@partymembername$");
		for(i=0; i<MAX_PARTY && j<128; i++) {
			if(p->member[i].account_id) {
				mapreg_setregstr(num+(j<<24),p->member[i].name,0);
				j++;
			}
		}
	}
	mapreg_setreg(add_str("$@partymembercount"),j,0);

	return 0;
}

/*==========================================
 * wID̃Mh擾
 *------------------------------------------
 */
static char *buildin_getguildname_sub(int guild_id)
{
	struct guild *g = guild_search(guild_id);

	if(g!=NULL){
		return (char *)aStrdup(g->name);
	}
	return NULL;
}

int buildin_getguildname(struct script_state *st)
{
	char *name;
	int guild_id=conv_num(st,& (st->stack->stack_data[st->start+2]));

	name=buildin_getguildname_sub(guild_id);
	if(name)
		push_str(st->stack,C_STR,name);
	else
		push_str(st->stack,C_CONSTSTR,"");

	return 0;
}

/*==========================================
 * wIDGuildMaster擾
 *------------------------------------------
 */
int buildin_getguildmaster(struct script_state *st)
{
	int guild_id = conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(guild_id > 0) {
		struct guild *g = guild_search(guild_id);
		if(g) {
			push_str(st->stack,C_STR,(unsigned char *)aStrdup(g->master));
			return 0;
		}
	}
	push_str(st->stack,C_CONSTSTR,"");
	return 0;
}

int buildin_getguildmasterid(struct script_state *st)
{
	int guild_id = conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(guild_id > 0) {
		struct guild *g = guild_search(guild_id);
		if(g) {
			struct map_session_data *sd = map_nick2sd(g->master);
			if(sd) {
				push_val(st->stack,C_INT,sd->status.char_id);
				return 0;
			}
		}
	}
	push_val(st->stack,C_INT,0);
	return 0;
}

/*==========================================
 * LN^̖O
 *------------------------------------------
 */
int buildin_strcharinfo(struct script_state *st)
{
	struct map_session_data *sd;
	int num;
	char *buf;

	sd=script_rid2sd(st);
	num=conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(sd) {
		switch (num) {
		case 0:
			push_str(st->stack,C_STR,(unsigned char *)aStrdup(sd->status.name));
			return 0;
		case 1:
			buf=buildin_getpartyname_sub(sd->status.party_id);
			if(buf)
				push_str(st->stack,C_STR,buf);
			else
				push_str(st->stack,C_CONSTSTR,"");
			return 0;
		case 2:
			buf=buildin_getguildname_sub(sd->status.guild_id);
			if(buf)
				push_str(st->stack,C_STR,buf);
			else
				push_str(st->stack,C_CONSTSTR,"");
			return 0;
		}
	}
	push_str(st->stack,C_CONSTSTR,"");
	return 0;
}

// pc.cequip_posƏԂقȂ邱Ƃɒ
static const unsigned int equip_pos[11]={0x0100,0x0010,0x0020,0x0002,0x0004,0x0040,0x0008,0x0080,0x0200,0x0001,0x8000};

/*==========================================
 * wʒȗiID擾
 *------------------------------------------
 */
int buildin_getequipid(struct script_state *st)
{
	int num, i = -1;
	struct map_session_data *sd = script_rid2sd(st);

	num = conv_num(st,& (st->stack->stack_data[st->start+2]));
	if(num > 0 && num <= 11)
		i = pc_checkequip(sd,equip_pos[num-1]);

	if(i >= 0) {
		if(sd->inventory_data[i]) {
			push_val(st->stack,C_INT,sd->inventory_data[i]->nameid);
			return 0;
		}
	}
	push_val(st->stack,C_INT,0);
	return 0;
}

/*==========================================
 * iBj[pj
 *------------------------------------------
 */
int buildin_getequipname(struct script_state *st)
{
	int num = conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(num > 0 && num <= 11) {
		struct map_session_data *sd = script_rid2sd(st);
		int i = pc_checkequip(sd,equip_pos[num-1]);
		char *buf;

		if(i >= 0 && sd->inventory_data[i]) {
			buf = (char *)aStrdup(sd->inventory_data[i]->jname);
		} else {
			buf = (char *)aMalloc(sizeof(refine_posword[0]) * 2 + 4);
			sprintf(buf,"%s-[%s]",refine_posword[num-1],refine_posword[10]);
		}
		push_str(st->stack,C_STR,buf);
	} else {
		push_str(st->stack,C_CONSTSTR,"");
	}

	return 0;
}

/*==========================================
 * `FbN
 *------------------------------------------
 */
int buildin_getequipisequiped(struct script_state *st)
{
	int num, i = -1;
	struct map_session_data *sd;

	num=conv_num(st,& (st->stack->stack_data[st->start+2]));
	sd=script_rid2sd(st);

	if(num > 0 && num <= 11)
		i=pc_checkequip(sd,equip_pos[num-1]);
	if(i >= 0){
		push_val(st->stack,C_INT,1);
	}else{
		push_val(st->stack,C_INT,0);
	}

	return 0;
}

/*==========================================
 * iB\`FbN
 *------------------------------------------
 */
int buildin_getequipisenableref(struct script_state *st)
{
	int num, i = -1;
	struct map_session_data *sd;

	num=conv_num(st,& (st->stack->stack_data[st->start+2]));
	sd=script_rid2sd(st);

	if(num > 0 && num <= 11)
		i=pc_checkequip(sd,equip_pos[num-1]);
	if(i >= 0 && sd->inventory_data[i] && sd->inventory_data[i]->refine != 0)
		push_val(st->stack,C_INT,1);
	else
		push_val(st->stack,C_INT,0);

	return 0;
}

/*==========================================
 * iӒ`FbN
 *------------------------------------------
 */
int buildin_getequipisidentify(struct script_state *st)
{
	int num, i = -1;
	struct map_session_data *sd;

	num=conv_num(st,& (st->stack->stack_data[st->start+2]));
	sd=script_rid2sd(st);
	if(num > 0 && num <= 11)
		i=pc_checkequip(sd,equip_pos[num-1]);
	if(i >= 0)
		push_val(st->stack,C_INT,sd->status.inventory[i].identify);
	else
		push_val(st->stack,C_INT,0);

	return 0;
}

/*==========================================
 * iBx
 *------------------------------------------
 */
int buildin_getequiprefinerycnt(struct script_state *st)
{
	int num, i = -1;
	struct map_session_data *sd;

	num=conv_num(st,& (st->stack->stack_data[st->start+2]));
	sd=script_rid2sd(st);
	if(num > 0 && num <= 11)
		i=pc_checkequip(sd,equip_pos[num-1]);
	if(i >= 0)
		push_val(st->stack,C_INT,sd->status.inventory[i].refine);
	else
		push_val(st->stack,C_INT,0);

	return 0;
}

/*==========================================
 * iLV
 *------------------------------------------
 */
int buildin_getequipweaponlv(struct script_state *st)
{
	int num, i = - 1;
	struct map_session_data *sd;

	num=conv_num(st,& (st->stack->stack_data[st->start+2]));
	sd=script_rid2sd(st);
	if(num > 0 && num <= 11)
		i=pc_checkequip(sd,equip_pos[num-1]);
	if(i >= 0 && sd->inventory_data[i])
		push_val(st->stack,C_INT,sd->inventory_data[i]->wlv);
	else
		push_val(st->stack,C_INT,0);

	return 0;
}

/*==========================================
 * iB
 *------------------------------------------
 */
int buildin_getequippercentrefinery(struct script_state *st)
{
	int num, i = -1;
	struct map_session_data *sd;

	num=conv_num(st,& (st->stack->stack_data[st->start+2]));
	sd=script_rid2sd(st);
	if(num > 0 && num <= 11)
		i=pc_checkequip(sd,equip_pos[num-1]);
	if(i >= 0)
		push_val(st->stack,C_INT,status_percentrefinery(sd,&sd->status.inventory[i]));
	else
		push_val(st->stack,C_INT,0);

	return 0;
}

/*==========================================
 * B
 *------------------------------------------
 */
int buildin_successrefitem(struct script_state *st)
{
	int num, i = -1;
	struct map_session_data *sd;

	num=conv_num(st,& (st->stack->stack_data[st->start+2]));
	sd=script_rid2sd(st);

	if(num > 0 && num <= 11)
		i=pc_checkequip(sd,equip_pos[num-1]);
	if(i >= 0) {
		int ep=sd->status.inventory[i].equip;
		pc_unequipitem(sd,i,1);
		skill_success_weaponrefine(sd,i);
		pc_equipitem(sd,i,ep);
	}

	return 0;
}

/*==========================================
 * Bs
 *------------------------------------------
 */
int buildin_failedrefitem(struct script_state *st)
{
	int num, i = -1;
	struct map_session_data *sd;

	num=conv_num(st,& (st->stack->stack_data[st->start+2]));
	sd=script_rid2sd(st);

	if(num > 0 && num <= 11)
		i=pc_checkequip(sd,equip_pos[num-1]);
	if(i >= 0)
		skill_fail_weaponrefine(sd,i);

	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_statusup(struct script_state *st)
{
	unsigned short type;

	type = (unsigned short)conv_num(st, &(st->stack->stack_data[st->start+2]));
	pc_statusup(script_rid2sd(st),type);

	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_statusup2(struct script_state *st)
{
	int type,val;

	type = conv_num(st,& (st->stack->stack_data[st->start+2]));
	val  = conv_num(st,& (st->stack->stack_data[st->start+3]));
	pc_statusup2(script_rid2sd(st),type,val);

	return 0;
}

/*==========================================
 * iɂ\͒l{[iX
 *------------------------------------------
 */
int buildin_bonus(struct script_state *st)
{
	int type,val;

	type = conv_num(st,& (st->stack->stack_data[st->start+2]));
	val  = conv_num(st,& (st->stack->stack_data[st->start+3]));

	pc_bonus(script_rid2sd(st),type,val);
	return 0;
}

/*==========================================
 * iɂ\͒l{[iX2
 *------------------------------------------
 */
int buildin_bonus2(struct script_state *st)
{
	int type,type2,val;

	type  = conv_num(st,& (st->stack->stack_data[st->start+2]));
	type2 = conv_num(st,& (st->stack->stack_data[st->start+3]));
	val   = conv_num(st,& (st->stack->stack_data[st->start+4]));

	pc_bonus2(script_rid2sd(st),type,type2,val);
	return 0;
}

/*==========================================
 * iɂ\͒l{[iX3
 *------------------------------------------
 */
int buildin_bonus3(struct script_state *st)
{
	int type,type2,type3,val;

	type  = conv_num(st,& (st->stack->stack_data[st->start+2]));
	type2 = conv_num(st,& (st->stack->stack_data[st->start+3]));
	type3 = conv_num(st,& (st->stack->stack_data[st->start+4]));
	val   = conv_num(st,& (st->stack->stack_data[st->start+5]));

	pc_bonus3(script_rid2sd(st),type,type2,type3,val);
	return 0;
}

/*==========================================
 * iɂ\͒l{[iX4
 *------------------------------------------
 */
int buildin_bonus4(struct script_state *st)
{
	int type,type2,type3,type4;
	unsigned long val;

	type  = conv_num(st,& (st->stack->stack_data[st->start+2]));
	type2 = conv_num(st,& (st->stack->stack_data[st->start+3]));
	type3 = conv_num(st,& (st->stack->stack_data[st->start+4]));
	type4 = conv_num(st,& (st->stack->stack_data[st->start+5]));
	val   = (unsigned long)conv_num(st,& (st->stack->stack_data[st->start+6]));

	pc_bonus4(script_rid2sd(st),type,type2,type3,type4,val);
	return 0;
}

/*==========================================
 * XL擾
 *------------------------------------------
 */
int buildin_skill(struct script_state *st)
{
	int id,level,flag=1;

	id    = conv_num(st,& (st->stack->stack_data[st->start+2]));
	level = conv_num(st,& (st->stack->stack_data[st->start+3]));
	if(st->end > st->start+4)
		flag = conv_num(st,&(st->stack->stack_data[st->start+4]) );

	pc_skill(script_rid2sd(st),id,level,flag);
	return 0;
}

/*==========================================
 * MhXL擾
 *------------------------------------------
 */
int buildin_guildskill(struct script_state *st)
{
	int id,level,flag=1;
	struct map_session_data *sd = script_rid2sd(st);

	id    = conv_num(st,& (st->stack->stack_data[st->start+2]));
	level = conv_num(st,& (st->stack->stack_data[st->start+3]));
	if(st->end > st->start+4)
		flag = conv_num(st,& (st->stack->stack_data[st->start+4]));

	if(level <= 0)
		level = -1;

	guild_skillup(sd,id,level,flag);

	return 0;
}

/*==========================================
 * XLx擾
 *------------------------------------------
 */
int buildin_getskilllv(struct script_state *st)
{
	int id=conv_num(st,& (st->stack->stack_data[st->start+2]));

	push_val(st->stack,C_INT, pc_checkskill2( script_rid2sd(st) ,id) );
	return 0;
}

/*==========================================
 * getgdskilllv(Guild_ID, Skill_ID);
 *------------------------------------------
 */
int buildin_getgdskilllv(struct script_state *st)
{
	int guild_id=conv_num(st,& (st->stack->stack_data[st->start+2]));
	int skill_id=conv_num(st,& (st->stack->stack_data[st->start+3]));
	struct guild *g=guild_search(guild_id);

	push_val(st->stack,C_INT, (g==NULL)? -1: guild_checkskill(g,skill_id) );
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_basicskillcheck(struct script_state *st)
{
	push_val(st->stack,C_INT,battle_config.basic_skill_check);
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_getgmlevel(struct script_state *st)
{
	push_val(st->stack,C_INT, pc_isGM(script_rid2sd(st)));
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_end(struct script_state *st)
{
	st->state = END;
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_checkoption(struct script_state *st)
{
	unsigned int type;
	struct map_session_data *sd = script_rid2sd(st);

	type=(unsigned int)conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(sd && sd->sc.option & type){
		push_val(st->stack,C_INT,1);
	} else {
		push_val(st->stack,C_INT,0);
	}

	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_setoption(struct script_state *st)
{
	unsigned int type;

	type = (unsigned int)conv_num(st,& (st->stack->stack_data[st->start+2]));
	pc_setoption(script_rid2sd(st),type);

	return 0;
}

/*==========================================
 * J[gt
 *------------------------------------------
 */
int buildin_setcart(struct script_state *st)
{
	pc_setcart( script_rid2sd(st),1 );
	return 0;
}

/*==========================================
 * t
 *------------------------------------------
 */
int buildin_setfalcon(struct script_state *st)
{
	pc_setfalcon( script_rid2sd(st) );
	return 0;
}

/*==========================================
 * yRyR
 *------------------------------------------
 */
int buildin_setriding(struct script_state *st)
{
	pc_setriding( script_rid2sd(st) );
	return 0;
}

/*==========================================
 * Z[u|Cg̕ۑ
 *------------------------------------------
 */
int buildin_savepoint(struct script_state *st)
{
	int x,y;
	char *str;

	str=conv_str(st,& (st->stack->stack_data[st->start+2]));
	x=conv_num(st,& (st->stack->stack_data[st->start+3]));
	y=conv_num(st,& (st->stack->stack_data[st->start+4]));
	pc_setsavepoint(script_rid2sd(st),str,x,y);
	return 0;
}

/*==========================================
 * GetTimeTick(0: System Tick, 1: Time Second Tick)
 *------------------------------------------
 */
int buildin_gettimetick(struct script_state *st)	/* Asgard Version */
{
	int type;

	type=conv_num(st,& (st->stack->stack_data[st->start+2]));

	switch(type){
	case 0:
		// System Ticks
		push_val(st->stack,C_INT,gettick());
		break;
	case 1:
		// Second Ticks: 0-86399, 00:00:00-23:59:59
		{
			time_t timer;
			struct tm *t;
			time(&timer);
			t=localtime(&timer);
			push_val(st->stack,C_INT,((t->tm_hour)*3600+(t->tm_min)*60+t->tm_sec));
		}
		break;
	case 2:
		// Get the number of seconds elapsed since 00:00 hours, Jan 1, 1970 UTC from the system clock
		push_val(st->stack,C_INT,(int)time(NULL));
		break;
	default:
		push_val(st->stack,C_INT,0);
		break;
	}
	return 0;
}

/*==========================================
 * GetTime(Type);
 * 1: Sec	 2: Min	 3: Hour
 * 4: WeekDay	 5: MonthDay	 6: Month
 * 7: Year
 *------------------------------------------
 */
int buildin_gettime(struct script_state *st)	/* Asgard Version */
{
	int type;
	time_t timer;
	struct tm *t;

	type=conv_num(st,& (st->stack->stack_data[st->start+2]));

	time(&timer);
	t=localtime(&timer);

	switch(type){
	case 1:	// Sec(0~59)
		push_val(st->stack,C_INT,t->tm_sec);
		break;
	case 2:	// Min(0~59)
		push_val(st->stack,C_INT,t->tm_min);
		break;
	case 3:	// Hour(0~23)
		push_val(st->stack,C_INT,t->tm_hour);
		break;
	case 4:	// WeekDay(0~6)
		push_val(st->stack,C_INT,t->tm_wday);
		break;
	case 5:	// MonthDay(01~31)
		push_val(st->stack,C_INT,t->tm_mday);
		break;
	case 6:	// Month(01~12)
		push_val(st->stack,C_INT,t->tm_mon+1);
		break;
	case 7:	// Year(20xx)
		push_val(st->stack,C_INT,t->tm_year+1900);
		break;
	default:
		push_val(st->stack,C_INT,-1);
		break;
	}
	return 0;
}

/*==========================================
 * GetTimeStr("TimeFMT", Length);
 *------------------------------------------
 */
int buildin_gettimestr(struct script_state *st)
{
	char *tmpstr, *fmtstr;
	int maxlen;
	time_t now = time(NULL);

	fmtstr = conv_str(st,& (st->stack->stack_data[st->start+2]));
	maxlen = conv_num(st,& (st->stack->stack_data[st->start+3]));

	tmpstr = (char *)aCalloc(maxlen+1,sizeof(char));
	if( !strftime(tmpstr,maxlen+1,fmtstr,localtime(&now)) ) {
		printf("buildin_gettimestr: maxlen is too short!!\n");
	}
	tmpstr[maxlen] = '\0';

	push_str(st->stack,C_STR,tmpstr);
	return 0;
}

/*==========================================
 * JvqɂJ
 *------------------------------------------
 */
int buildin_openstorage(struct script_state *st)
{
	storage_storageopen(script_rid2sd(st));
	return 0;
}

/*==========================================
 * MhqɂJ
 *------------------------------------------
 */
int buildin_guildopenstorage(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);

	if(sd == NULL) {
		st->state = END;
		return 0;
	}

	if(sd->state.menu_or_input == 0) {
		if(sd->status.guild_id <= 0) {	// Mh
			push_val(st->stack,C_INT,2);
			return 0;
		}
		if(sd->state.gstorage_lockreq != 0) {
			push_val(st->stack,C_INT,3);
			return 0;
		}
		st->state = RERUNLINE;
		sd->state.menu_or_input = 1;
		sd->state.gstorage_lockreq = 1;	// scriptoRtO
		intif_trylock_guild_storage(sd,st->oid);
	} else {
		sd->state.menu_or_input = 0;

		if(sd->npc_menu) {	// bN擾
			if(sd->npc_menu == 2) {
				// LbV폜ă[h
				storage_guild_delete(sd->status.guild_id);
			}
			sd->npc_menu = 0;
			push_val(st->stack,C_INT,storage_guild_storageopen(sd));
		} else {		// bN擾s
			push_val(st->stack,C_INT,1);
		}
	}

	return 0;
}

/*==========================================
 * ACeɂXL
 *------------------------------------------
 */
int buildin_itemskill(struct script_state *st)
{
	int id,lv,flag = 0;
	char *str;
	struct map_session_data *sd = script_rid2sd(st);

	nullpo_retr(0, sd);

	id  = conv_num(st,& (st->stack->stack_data[st->start+2]));
	lv  = conv_num(st,& (st->stack->stack_data[st->start+3]));
	str = conv_str(st,& (st->stack->stack_data[st->start+4]));
	if(st->end > st->start+5)
		flag = conv_num(st,& (st->stack->stack_data[st->start+5]));

	// rɃXLACe͎gpłȂ
	if(sd->ud.skilltimer != -1)
		return 0;

	sd->skillitem   = id;
	sd->skillitemlv = lv;
	sd->skillitem_flag = (flag)? 1: 0;
	clif_item_skill(sd,id,lv,str);

	return 0;
}

/*==========================================
 * ACe쐬
 *------------------------------------------
 */
int buildin_produce(struct script_state *st)
{
	struct map_session_data *sd=script_rid2sd(st);

	nullpo_retr(0, sd);

	if(sd->state.produce_flag != 1) {
		int trigger=conv_num(st,& (st->stack->stack_data[st->start+2]));
		clif_skill_produce_mix_list(sd,trigger);
	}
	return 0;
}

/*==========================================
 * NPCŃybg
 *------------------------------------------
 */
int buildin_makepet(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	int id,pet_id;

	id = conv_num(st,& (st->stack->stack_data[st->start+2]));

	pet_id = search_petDB_index(id, PET_CLASS);

	if(pet_id < 0)
		pet_id = search_petDB_index(id, PET_EGG);
	if(pet_id >= 0 && sd) {
		sd->catch_target_class = pet_db[pet_id].class_;
		intif_create_pet(
			sd->status.account_id, sd->status.char_id,
			pet_db[pet_id].class_, mob_db[pet_db[pet_id].class_].lv,
			pet_db[pet_id].EggID, 0, pet_db[pet_id].intimate,
			100, 0, 1, pet_db[pet_id].jname
		);
	}

	return 0;
}

/*==========================================
 * X^[
 *------------------------------------------
 */
int buildin_monster(struct script_state *st)
{
	int mob_id,amount,m,x,y,guild_id=0,id;
	char *str,*mapname,*mobname;
	const char *event = "";
	struct mob_data *md;

	mapname = conv_str(st,& (st->stack->stack_data[st->start+2]));
	x       = conv_num(st,& (st->stack->stack_data[st->start+3]));
	y       = conv_num(st,& (st->stack->stack_data[st->start+4]));
	str     = conv_str(st,& (st->stack->stack_data[st->start+5]));
	mobname = conv_str(st,& (st->stack->stack_data[st->start+6]));

	if((mob_id = atoi(mobname)) == 0)
		mob_id = mobdb_searchname(mobname);
	if(mob_id >= 0 && !mobdb_checkid(mob_id))
		return 0;

	amount = conv_num(st,& (st->stack->stack_data[st->start+7]));
	if(st->end > st->start+8) {
		event = conv_str(st,& (st->stack->stack_data[st->start+8]));
	}
	if(st->end > st->start+9) {	// Guild_ID
		guild_id = conv_num(st,& (st->stack->stack_data[st->start+9]));
	}

	if((m = script_mapname2mapid(st,mapname)) < 0)
		return 0;

	id = mob_once_spawn(map_id2sd(st->rid),m,x,y,str,mob_id,amount,event);

	if((md = map_id2md(id)) != NULL)
	{
		md->guardup_lv = 0;
		if(guild_id > 0) {
			struct guild *g = guild_search(guild_id);
			if(g) {
				// K[fBAȂMhXLKp
				md->guild_id   = guild_id;
				md->guardup_lv = guild_checkskill(g,GD_GUARDUP);
			}
		}

		// _҂ȂȂhbv
		if(mob_id == -1) {
			if(md->mode&0x20) {	// 蔲{X
				md->state.nodrop = battle_config.branch_boss_no_drop;
				md->state.noexp  = battle_config.branch_boss_no_exp;
				md->state.nomvp  = battle_config.branch_boss_no_mvp;
			} else {
				md->state.nodrop = battle_config.branch_mob_no_drop;
				md->state.noexp  = battle_config.branch_mob_no_exp;
				md->state.nomvp  = battle_config.branch_mob_no_mvp;
			}
		}
	}

	return 0;
}

/*==========================================
 * X^[GAw
 *------------------------------------------
 */
int buildin_areamonster(struct script_state *st)
{
	int mob_id,amount,m,x0,y0,x1,y1;
	char *str,*mapname,*mobname;
	const char *event = "";

	mapname = conv_str(st,& (st->stack->stack_data[st->start+2]));
	x0      = conv_num(st,& (st->stack->stack_data[st->start+3]));
	y0      = conv_num(st,& (st->stack->stack_data[st->start+4]));
	x1      = conv_num(st,& (st->stack->stack_data[st->start+5]));
	y1      = conv_num(st,& (st->stack->stack_data[st->start+6]));
	str     = conv_str(st,& (st->stack->stack_data[st->start+7]));
	mobname = conv_str(st,& (st->stack->stack_data[st->start+8]));

	if((mob_id = atoi(mobname)) == 0)
		mob_id = mobdb_searchname(mobname);

	amount = conv_num(st,& (st->stack->stack_data[st->start+9]));
	if(st->end > st->start+10)
		event = conv_str(st,& (st->stack->stack_data[st->start+10]));

	m = script_mapname2mapid(st,mapname);
	if(m >= 0)
		mob_once_spawn_area(map_id2sd(st->rid), m, x0, y0, x1, y1, str, mob_id, amount, event);

	return 0;
}

/*==========================================
 * X^[폜
 *------------------------------------------
 */
static int buildin_killmonster_sub(struct block_list *bl,va_list ap)
{
	struct mob_data *md = NULL;
	char *event=va_arg(ap,char *);
	int allflag=va_arg(ap,int);

	nullpo_retr(0, bl);
	nullpo_retr(0, md = (struct mob_data *)bl);

	if(!allflag) {
		if(strcmp(event,md->npc_event) == 0)
			unit_remove_map(bl,1,0);
	} else {
		if(md->spawndelay1 == -1 && md->spawndelay2 == -1)
			unit_remove_map(bl,1,0);
	}
	return 0;
}

int buildin_killmonster(struct script_state *st)
{
	char *mapname,*event;
	int m,allflag=0;

	mapname=conv_str(st,& (st->stack->stack_data[st->start+2]));
	event=conv_str(st,& (st->stack->stack_data[st->start+3]));

	if((m = script_mapname2mapid(st,mapname)) < 0)
		return 0;

	if(strcmp(event,"All")==0)
		allflag = 1;
	map_foreachinarea(buildin_killmonster_sub,
		m,0,0,map[m].xs,map[m].ys,BL_MOB, event ,allflag);
	return 0;
}

/*==========================================
 * MAPX^[S폜
 *------------------------------------------
 */
static int buildin_killmonsterall_sub(struct block_list *bl,va_list ap)
{
	unit_remove_map(bl,1,0);
	return 0;
}

int buildin_killmonsterall(struct script_state *st)
{
	char *mapname;
	int m;

	mapname=conv_str(st,& (st->stack->stack_data[st->start+2]));

	m = script_mapname2mapid(st,mapname);
	if(m >= 0)
		map_foreachinarea(buildin_killmonsterall_sub,m,0,0,map[m].xs,map[m].ys,BL_MOB);
	return 0;
}

/*==========================================
 * X^[폜GAw
 *------------------------------------------
 */
int buildin_areakillmonster(struct script_state *st)
{
	char *mapname;
	int m,x0,y0,x1,y1;

	mapname=conv_str(st,& (st->stack->stack_data[st->start+2]));
	x0=conv_num(st,& (st->stack->stack_data[st->start+3]));
	y0=conv_num(st,& (st->stack->stack_data[st->start+4]));
	x1=conv_num(st,& (st->stack->stack_data[st->start+5]));
	y1=conv_num(st,& (st->stack->stack_data[st->start+6]));

	m = script_mapname2mapid(st,mapname);
	if(m >= 0)
		map_foreachinarea(buildin_killmonsterall_sub,m,x0,y0,x1,y1,BL_MOB);
	return 0;
}

/*==========================================
 * Cxgs
 *------------------------------------------
 */
int buildin_doevent(struct script_state *st)
{
	char *event = conv_str(st,& (st->stack->stack_data[st->start+2]));

	npc_event(script_rid2sd(st),event);
	return 0;
}

/*==========================================
 * NPC̃Cxgs
 *------------------------------------------
 */
int buildin_donpcevent(struct script_state *st)
{
	char *event= conv_str(st,& (st->stack->stack_data[st->start+2]));

	npc_event_do(event);
	return 0;
}

/*==========================================
 * Cxg^C}[ǉ
 *------------------------------------------
 */
int buildin_addtimer(struct script_state *st)
{
	int tick    = conv_num(st,& (st->stack->stack_data[st->start+2]));
	char *event = conv_str(st,& (st->stack->stack_data[st->start+3]));

	pc_addeventtimer(script_rid2sd(st),tick,event);
	return 0;
}

/*==========================================
 * Cxg^C}[폜
 *------------------------------------------
 */
int buildin_deltimer(struct script_state *st)
{
	char *event = conv_str(st,& (st->stack->stack_data[st->start+2]));

	pc_deleventtimer(script_rid2sd(st),event);
	return 0;
}

/*==========================================
 * Cxg^C}[̃JEglǉ
 *------------------------------------------
 */
int buildin_addtimercount(struct script_state *st)
{
	char *event = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int tick    = conv_num(st,& (st->stack->stack_data[st->start+3]));

	pc_addeventtimercount(script_rid2sd(st),event,tick);
	return 0;
}

/*==========================================
 * Cxg^C}[̎c莞Ԏ擾
 *------------------------------------------
 */
int buildin_gettimerdiff(struct script_state *st)
{
	char *event;
	int ret;

	event = conv_str(st,& (st->stack->stack_data[st->start+2]));
	ret = pc_geteventtimerdiff(script_rid2sd(st),event);

	push_val(st->stack,C_INT,ret);
	return 0;
}

/*==========================================
 * NPC^C}[
 *------------------------------------------
 */
int buildin_initnpctimer(struct script_state *st)
{
	struct npc_data *nd;

	if( st->end > st->start+2 )
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+2])));
	else
		nd = map_id2nd(st->oid);

	npc_settimerevent_tick(nd,0);
	npc_timerevent_start(nd);
	return 0;
}

/*==========================================
 * NPC^C}[Jn
 *------------------------------------------
 */
int buildin_startnpctimer(struct script_state *st)
{
	struct npc_data *nd;

	if( st->end > st->start+2 )
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+2])));
	else
		nd = map_id2nd(st->oid);

	npc_timerevent_start(nd);
	return 0;
}

/*==========================================
 * NPC^C}[~
 *------------------------------------------
 */
int buildin_stopnpctimer(struct script_state *st)
{
	struct npc_data *nd;

	if( st->end > st->start+2 )
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+2])));
	else
		nd = map_id2nd(st->oid);

	npc_timerevent_stop(nd);
	return 0;
}

/*==========================================
 * NPC^C}[擾
 *------------------------------------------
 */
int buildin_getnpctimer(struct script_state *st)
{
	struct npc_data *nd;
	int type, val = 0;

	type = conv_num(st,& (st->stack->stack_data[st->start+2]));
	if(st->end > st->start+3)
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+3])));
	else
		nd = map_id2nd(st->oid);

	if(nd) {
		switch(type){
			case 0: val = npc_gettimerevent_tick(nd);       break;
			case 1: val = (nd->u.scr.nexttimer >= 0)? 1: 0; break;
			case 2: val = nd->u.scr.timeramount;            break;
		}
	}
	push_val(st->stack,C_INT,val);
	return 0;
}

/*==========================================
 * NPC^C}[lݒ
 *------------------------------------------
 */
int buildin_setnpctimer(struct script_state *st)
{
	int tick;
	struct npc_data *nd;

	tick=conv_num(st,& (st->stack->stack_data[st->start+2]));
	if( st->end > st->start+3 )
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+3])));
	else
		nd = map_id2nd(st->oid);

	npc_settimerevent_tick(nd,tick);
	return 0;
}

/*==========================================
 * V̐AiEX
 *------------------------------------------
 */
int buildin_announce(struct script_state *st)
{
	char *str = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int flag  = conv_num(st,& (st->stack->stack_data[st->start+3]));
	size_t len;
	char *color = NULL;

	if(st->end > st->start+4)
		color = conv_str(st,& (st->stack->stack_data[st->start+4]));
	len = strlen(str)+1;

	if(flag&0x07) {
		struct block_list *bl = NULL;
		if(flag&0x08) {
			bl = map_id2bl(st->oid);
		} else {
			struct map_session_data *sd = script_rid2sd(st);
			if(sd)
				bl = &sd->bl;
		}
		if(bl == NULL)
			return 0;

		if(color)
			clif_announce(bl,str,len,strtoul(color,NULL,0),flag);
		else
			clif_GMmessage(bl,str,len,flag);
	} else {
		if(color)
			intif_announce(str,len,strtoul(color,NULL,0));
		else
			intif_GMmessage(str,len,flag);
	}
	return 0;
}

/*==========================================
 * V̐AiEXi}bvj
 *------------------------------------------
 */
static int buildin_mapannounce_sub(struct block_list *bl,va_list ap)
{
	char *str,*color;
	size_t len;
	int flag;

	nullpo_retr(0, bl);

	str   = va_arg(ap,char *);
	len   = va_arg(ap,size_t);
	flag  = va_arg(ap,int);
	color = va_arg(ap,char *);

	if(color)
		clif_announce(bl,str,len,strtoul(color,NULL,0),flag|3);
	else
		clif_GMmessage(bl,str,len,flag|3);
	return 0;
}

int buildin_mapannounce(struct script_state *st)
{
	char *mapname,*str,*color=NULL;
	int flag,m;

	mapname=conv_str(st,& (st->stack->stack_data[st->start+2]));
	str=conv_str(st,& (st->stack->stack_data[st->start+3]));
	flag=conv_num(st,& (st->stack->stack_data[st->start+4]));
	if (st->end>st->start+5)
		color=conv_str(st,& (st->stack->stack_data[st->start+5]));

	m = script_mapname2mapid(st,mapname);
	if(m >= 0)
		map_foreachinarea(buildin_mapannounce_sub,m,0,0,map[m].xs,map[m].ys,BL_PC,str,strlen(str)+1,flag&0x10,color);
	return 0;
}

/*==========================================
 * V̐AiEXiGAj
 *------------------------------------------
 */
int buildin_areaannounce(struct script_state *st)
{
	char *mapname,*str,*color = NULL;
	int flag,m;
	int x0,y0,x1,y1;

	mapname = conv_str(st,& (st->stack->stack_data[st->start+2]));
	x0      = conv_num(st,& (st->stack->stack_data[st->start+3]));
	y0      = conv_num(st,& (st->stack->stack_data[st->start+4]));
	x1      = conv_num(st,& (st->stack->stack_data[st->start+5]));
	y1      = conv_num(st,& (st->stack->stack_data[st->start+6]));
	str     = conv_str(st,& (st->stack->stack_data[st->start+7]));
	flag    = conv_num(st,& (st->stack->stack_data[st->start+8]));
	if (st->end>st->start+9)
		color = conv_str(st,& (st->stack->stack_data[st->start+9]));

	m = script_mapname2mapid(st,mapname);
	if(m >= 0)
		map_foreachinarea(buildin_mapannounce_sub,m,x0,y0,x1,y1,BL_PC,str,strlen(str)+1,flag&0x10,color);
	return 0;
}

/*==========================================
 * [U[擾
 *------------------------------------------
 */
int buildin_getusers(struct script_state *st)
{
	int flag=conv_num(st,& (st->stack->stack_data[st->start+2]));
	int val = -1;

	switch(flag&0x07){
		case 0:
		{
			struct block_list *bl=map_id2bl( (flag&0x08)? st->oid: st->rid );
			if(bl)
				val=map[bl->m].users;
			break;
		}
		case 1:
			val=map_getusers();
			break;
	}
	push_val(st->stack,C_INT,val);
	return 0;
}

/*==========================================
 * qł郆[U[̑S̖O擾
 *------------------------------------------
 */
int buildin_getusersname(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	struct map_session_data *pl_sd = NULL;
	int i=0,disp_num=1;

	nullpo_retr(0, sd);

	for (i=0;i<fd_max;i++) {
		if(session[i] && (pl_sd = (struct map_session_data *)session[i]->session_data) && pl_sd->state.auth){
			if( !(battle_config.hide_GM_session && pc_isGM(pl_sd)) ){
				if((disp_num++)%10==0)
					clif_scriptnext(sd,st->oid);
				clif_scriptmes(sd,st->oid,pl_sd->status.name);
			}
		}
	}
	return 0;
}

/*==========================================
 * }bvw胆[U[擾
 *------------------------------------------
 */
int buildin_getmapusers(struct script_state *st)
{
	char *str;
	int m;

	str=conv_str(st,& (st->stack->stack_data[st->start+2]));

	m = script_mapname2mapid(st,str);
	if(m < 0)
		push_val(st->stack,C_INT,-1);
	else
		push_val(st->stack,C_INT,map[m].users);
	return 0;
}

/*==========================================
 * GAw胆[U[擾
 *------------------------------------------
 */
static int buildin_getareausers_sub(struct block_list *bl,va_list ap)
{
	int *users=va_arg(ap,int *);

	(*users)++;
	return 0;
}

int buildin_getareausers(struct script_state *st)
{
	char *str;
	int m,x0,y0,x1,y1,users=0;

	str=conv_str(st,& (st->stack->stack_data[st->start+2]));
	x0=conv_num(st,& (st->stack->stack_data[st->start+3]));
	y0=conv_num(st,& (st->stack->stack_data[st->start+4]));
	x1=conv_num(st,& (st->stack->stack_data[st->start+5]));
	y1=conv_num(st,& (st->stack->stack_data[st->start+6]));

	m = script_mapname2mapid(st,str);
	if(m < 0) {
		push_val(st->stack,C_INT,-1);
		return 0;
	}
	map_foreachinarea(buildin_getareausers_sub,m,x0,y0,x1,y1,BL_PC,&users);
	push_val(st->stack,C_INT,users);
	return 0;
}

/*==========================================
 * GAwhbvACe擾
 *------------------------------------------
 */
static int buildin_getareadropitem_sub(struct block_list *bl,va_list ap)
{
	int item=va_arg(ap,int);
	int *amount=va_arg(ap,int *);
	struct flooritem_data *fitem;

	nullpo_retr(0, bl);
	nullpo_retr(0, fitem = (struct flooritem_data *)bl);

	if(fitem->item_data.nameid == item)
		(*amount) += fitem->item_data.amount;

	return 0;
}

int buildin_getareadropitem(struct script_state *st)
{
	char *str;
	int m,x0,y0,x1,y1,item=0,amount=0;
	struct script_data *data;

	str = conv_str(st,& (st->stack->stack_data[st->start+2]));
	x0  = conv_num(st,& (st->stack->stack_data[st->start+3]));
	y0  = conv_num(st,& (st->stack->stack_data[st->start+4]));
	x1  = conv_num(st,& (st->stack->stack_data[st->start+5]));
	y1  = conv_num(st,& (st->stack->stack_data[st->start+6]));

	data = &(st->stack->stack_data[st->start+7]);
	get_val(st,data);
	if( isstr(data) ) {
		const char *name = conv_str(st,data);
		struct item_data *item_data = itemdb_searchname(name);
		if(item_data)
			item = item_data->nameid;
	} else {
		item = conv_num(st,data);
	}

	m = script_mapname2mapid(st,str);
	if(m < 0) {
		push_val(st->stack,C_INT,-1);
		return 0;
	}
	if(item > 0) {
		map_foreachinarea(buildin_getareadropitem_sub,m,x0,y0,x1,y1,BL_ITEM,item,&amount);
	}
	push_val(st->stack,C_INT,amount);

	return 0;
}

/*==========================================
 * NPC̗L
 *------------------------------------------
 */
int buildin_enablenpc(struct script_state *st)
{
	struct npc_data *nd;

	if(st->end > st->start+2) {
		char *name = conv_str(st,& (st->stack->stack_data[st->start+2]));
		nd = npc_name2id(name);
		if(nd == NULL) {
			return 0;
		}
	} else {
		nd = map_id2nd(st->oid);
		if(nd == NULL) {
			printf("buildin_enablenpc: fatal error: npc not attached\n");
			return 0;
		}
	}
	npc_enable(nd->exname,1);
	return 0;
}
/*==========================================
 * NPC̖
 *------------------------------------------
 */
int buildin_disablenpc(struct script_state *st)
{
	struct npc_data *nd;

	if(st->end > st->start+2) {
		char *name = conv_str(st,& (st->stack->stack_data[st->start+2]));
		nd = npc_name2id(name);
		if(nd == NULL) {
			return 0;
		}
	} else {
		nd = map_id2nd(st->oid);
		if(nd == NULL) {
			printf("buildin_disablenpc: fatal error: npc not attached\n");
			return 0;
		}
	}
	npc_enable(nd->exname,0);
	return 0;
}

/*==========================================
 * BĂNPC̕\
 *------------------------------------------
 */
int buildin_hideoffnpc(struct script_state *st)
{
	struct npc_data *nd;

	if(st->end > st->start+2) {
		char *name = conv_str(st,& (st->stack->stack_data[st->start+2]));
		nd = npc_name2id(name);
		if(nd == NULL) {
			return 0;
		}
	} else {
		nd = map_id2nd(st->oid);
		if(nd == NULL) {
			printf("buildin_hideoffnpc: fatal error: npc not attached\n");
			return 0;
		}
	}
	npc_enable(nd->exname,2);
	return 0;
}

/*==========================================
 * NPCnCfBO
 *------------------------------------------
 */
int buildin_hideonnpc(struct script_state *st)
{
	struct npc_data *nd;

	if(st->end > st->start+2) {
		char *name = conv_str(st,& (st->stack->stack_data[st->start+2]));
		nd = npc_name2id(name);
		if(nd == NULL) {
			return 0;
		}
	} else {
		nd = map_id2nd(st->oid);
		if(nd == NULL) {
			printf("buildin_hideonnpc: fatal error: npc not attached\n");
			return 0;
		}
	}
	npc_enable(nd->exname,4);
	return 0;
}

/*==========================================
 * Ԉُɂ
 *------------------------------------------
 */
int buildin_sc_start(struct script_state *st)
{
	struct block_list *bl;
	int type,tick,val1;

	type=conv_num(st,& (st->stack->stack_data[st->start+2]));
	tick=conv_num(st,& (st->stack->stack_data[st->start+3]));
	val1=conv_num(st,& (st->stack->stack_data[st->start+4]));
	if( st->end>st->start+5 )	// w肵LԈُɂ
		bl = map_id2bl(conv_num(st,& (st->stack->stack_data[st->start+5])));
	else
		bl = map_id2bl(st->rid);

	if(bl && bl->type == BL_PC) {
		struct map_session_data *sd = (struct map_session_data *)bl;
		if(sd && sd->state.potionpitcher_flag)
			bl = map_id2bl(sd->ud.skilltarget);
	}
	if(bl && !unit_isdead(bl))
		status_change_start(bl,type,val1,0,0,0,tick,0);
	return 0;
}

/*==========================================
 * Ԉُɂ(mw)
 *------------------------------------------
 */
int buildin_sc_start2(struct script_state *st)
{
	struct block_list *bl;
	int type,tick,val1,per;

	type=conv_num(st,& (st->stack->stack_data[st->start+2]));
	tick=conv_num(st,& (st->stack->stack_data[st->start+3]));
	val1=conv_num(st,& (st->stack->stack_data[st->start+4]));
	per=conv_num(st,& (st->stack->stack_data[st->start+5]));
	if( st->end>st->start+6 )	// w肵LԈُɂ
		bl = map_id2bl(conv_num(st,& (st->stack->stack_data[st->start+6])));
	else
		bl = map_id2bl(st->rid);

	if(bl && bl->type == BL_PC) {
		struct map_session_data *sd = (struct map_session_data *)bl;
		if(sd && sd->state.potionpitcher_flag)
			bl = map_id2bl(sd->ud.skilltarget);
	}
	if(bl && !unit_isdead(bl))
		if(atn_rand() % 10000 < status_change_rate(bl,type,per,0))
			status_change_start(bl,type,val1,0,0,0,tick,0);
	return 0;
}

/*==========================================
 * Ԉُɂ(ČvZȂ)
 *------------------------------------------
 */
int buildin_sc_starte(struct script_state *st)
{
	struct block_list *bl;
	int type,tick,val1;

	type=conv_num(st,& (st->stack->stack_data[st->start+2]));
	tick=conv_num(st,& (st->stack->stack_data[st->start+3]));
	val1=conv_num(st,& (st->stack->stack_data[st->start+4]));
	if( st->end>st->start+5 )	// w肵LԈُɂ
		bl = map_id2bl(conv_num(st,& (st->stack->stack_data[st->start+5])));
	else
		bl = map_id2bl(st->rid);

	if(bl && bl->type == BL_PC) {
		struct map_session_data *sd = (struct map_session_data *)bl;
		if(sd && sd->state.potionpitcher_flag)
			bl = map_id2bl(sd->ud.skilltarget);
	}
	if(bl && !unit_isdead(bl))
		status_change_start(bl,type,val1,0,0,0,tick,4);
	return 0;
}

/*==========================================
 * Ԉُɂ(ڍ)
 *------------------------------------------
 */
int buildin_sc_start3(struct script_state *st)
{
	struct block_list *bl;
	int type,tick,val1,val2,val3,val4,flag;

	type=conv_num(st,& (st->stack->stack_data[st->start+2]));
	val1=conv_num(st,& (st->stack->stack_data[st->start+3]));
	val2=conv_num(st,& (st->stack->stack_data[st->start+4]));
	val3=conv_num(st,& (st->stack->stack_data[st->start+5]));
	val4=conv_num(st,& (st->stack->stack_data[st->start+6]));
	tick=conv_num(st,& (st->stack->stack_data[st->start+7]));
	flag=conv_num(st,& (st->stack->stack_data[st->start+8]));
	if( st->end>st->start+9 )	// w肵LԈُɂ
		bl = map_id2bl(conv_num(st,& (st->stack->stack_data[st->start+9])));
	else
		bl = map_id2bl(st->rid);

	if(bl && bl->type == BL_PC) {
		struct map_session_data *sd = (struct map_session_data *)bl;
		if(sd && sd->state.potionpitcher_flag)
			bl = map_id2bl(sd->ud.skilltarget);
	}
	if(bl && !unit_isdead(bl))
		if(flag&8 || status_change_rate(bl,type,10000,0) > 0)
			status_change_start(bl,type,val1,val2,val3,val4,tick,flag);
	return 0;
}

/*==========================================
 * Ԉُ킪
 *------------------------------------------
 */
int buildin_sc_end(struct script_state *st)
{
	struct block_list *bl;
	int type;

	type=conv_num(st,& (st->stack->stack_data[st->start+2]));
	if( st->end>st->start+3 )	// w肵L̏Ԉُ
		bl = map_id2bl(conv_num(st,& (st->stack->stack_data[st->start+3])));
	else
		bl = map_id2bl(st->rid);

	if(bl && bl->type == BL_PC) {
		struct map_session_data *sd = (struct map_session_data *)bl;
		if(sd && sd->state.potionpitcher_flag)
			bl = map_id2bl(sd->ud.skilltarget);
	}
	if(bl) {
		if(type == SC_DANCING) {
			skill_stop_dancing(bl,0);
		} else {
			if(type == SC_BASILICA)
				skill_basilica_cancel(bl);
			status_change_end(bl,type,-1);
		}
	}
	return 0;
}

/*==========================================
 * Ԉُ풆ǂԂ
 *------------------------------------------
 */
int buildin_sc_ison(struct script_state *st)
{
	struct block_list *bl = map_id2bl(st->rid);
	int type;

	type = conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(type < 0 || type >= MAX_STATUSCHANGE) {
		printf("buildin_sc_ison: invaild type %d\n", type);
		push_val(st->stack,C_INT,0);
		return 0;
	}

	if(bl && bl->type == BL_PC) {
		struct map_session_data *sd = (struct map_session_data *)bl;
		if(sd && sd->state.potionpitcher_flag)
			bl = map_id2bl(sd->ud.skilltarget);
	}
	if(bl) {
		struct status_change *sc = status_get_sc(bl);
		if(sc && sc->data[type].timer != -1) {
			push_val(st->stack,C_INT,1);
			return 0;
		}
	}
	push_val(st->stack,C_INT,0);
	return 0;
}

/*==========================================
 * ԈُϐvZmԂ
 *------------------------------------------
 */
int buildin_getscrate(struct script_state *st)
{
	struct block_list *bl;
	int sc_def=100;
	int type,rate=0;

	type=conv_num(st,& (st->stack->stack_data[st->start+2]));
	rate=conv_num(st,& (st->stack->stack_data[st->start+3]));
	if( st->end>st->start+4 )	// w肵L̑ϐvZ
		bl = map_id2bl(conv_num(st,& (st->stack->stack_data[st->start+6])));
	else
		bl = map_id2bl(st->rid);

	if(bl) {
		int luk = status_get_luk(bl);
		switch (type) {
			case SC_STONE:
			case SC_FREEZE:
				sc_def = 100 - (3 + status_get_mdef(bl) + luk/3);
				break;
			case SC_STUN:
			case SC_POISON:
			case SC_SILENCE:
			case SC_BLEED:
				sc_def = 100 - (3 + status_get_vit(bl) + luk/3);
				break;
			case SC_SLEEP:
			case SC_CONFUSION:
			case SC_BLIND:
				sc_def = 100 - (3 + status_get_int(bl) + luk/3);
				break;
			case SC_CURSE:
				sc_def = 100 - (3 + luk);
				break;
		}
	}
	rate=rate*sc_def/100;
	push_val(st->stack,C_INT,rate);

	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_debugmes(struct script_state *st)
{
	char *str = conv_str(st,& (st->stack->stack_data[st->start+2]));

	printf("script debug : %d %d : %s\n",st->rid,st->oid,str);
	return 0;
}

/*==========================================
 * ߊlACegp
 *------------------------------------------
 */
int buildin_catchpet(struct script_state *st)
{
	int pet_id = conv_num(st,& (st->stack->stack_data[st->start+2]));

	pet_catch_process1(script_rid2sd(st),pet_id);
	return 0;
}

/*==========================================
 * gїz@gp
 *------------------------------------------
 */
int buildin_birthpet(struct script_state *st)
{
	clif_sendegg( script_rid2sd(st) );
	return 0;
}

/*==========================================
 * Xe[^XZbg
 *------------------------------------------
 */
int buildin_resetstatus(struct script_state *st)
{
	pc_resetstate( script_rid2sd(st) );
	return 0;
}

/*==========================================
 * XLZbg
 *------------------------------------------
 */
int buildin_resetskill(struct script_state *st)
{
	int flag = -1;

	if(st->end > st->start+2)
		flag = conv_num(st,& (st->stack->stack_data[st->start+2]));

	pc_resetskill(script_rid2sd(st), flag);
	return 0;
}

/*==========================================
 * ڂ̐EƕύX
 *------------------------------------------
 */
int buildin_changebase(struct script_state *st)
{
	struct map_session_data *sd;
	int job, upper = -1, vclass;

	job = conv_num(st,& (st->stack->stack_data[st->start+2]));
	if( st->end>st->start+3 )
		upper = conv_num(st,& (st->stack->stack_data[st->start+3]));

	if( st->end>st->start+4 )
		sd=map_id2sd(conv_num(st,& (st->stack->stack_data[st->start+4])));
	else
		sd=script_rid2sd(st);

	if(sd == NULL)
		return 0;

	if(upper < 0)
		upper = sd->s_class.upper;

	vclass = pc_calc_class_job(job, upper);

	if(vclass == 22 && !battle_config.wedding_modifydisplay)
		return 0;

	//if(vclass == 22 || vclass == 26 || vclass == 27)
	//	pc_unequipitem(sd,sd->equip_index[9],1);	// O

	sd->view_class = vclass;
	clif_changelook(&sd->bl, LOOK_BASE, vclass);
	clif_send_clothcolor(&sd->bl);
#if PACKETVER > 14
	clif_skillinfoblock(sd); 
#endif
	return 0;
}

/*==========================================
 * ʕϊ
 *------------------------------------------
 */
int buildin_changesex(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);

	nullpo_retr(0, sd);

	if(sd->sex==0){
		sd->sex=1;
		if(sd->s_class.job == 20)
			sd->status.class_ -= 1;
	} else {
		sd->sex=0;
		if(sd->s_class.job == 19)
			sd->status.class_ += 1;
	}
	chrif_changesex(sd->status.account_id,sd->sex);
	chrif_save(sd,0);
	return 0;
}

/*==========================================
 * npc`bg쐬
 *------------------------------------------
 */
int buildin_waitingroom(struct script_state *st)
{
	char *name;
	const char *ev = "";
	int limit, trigger = 0,pub=1;
	int zeny=0,lowlv=0,highlv=MAX_LEVEL;
	unsigned int job = 0xFFFFFFFF;
	int upper = 0;

	name  = conv_str(st,& (st->stack->stack_data[st->start+2]));
	limit = conv_num(st,& (st->stack->stack_data[st->start+3]));
	if(limit == 0)
		pub = 3;

	if(st->end > st->start+4) {
		ev = conv_str(st,& (st->stack->stack_data[st->start+4]));
	}
	if(st->end > st->start+5) {
		trigger = conv_num(st,& (st->stack->stack_data[st->start+5]));
	}
	if(st->end > st->start+8) {
		zeny   = conv_num(st,& (st->stack->stack_data[st->start+6]));
		lowlv  = conv_num(st,& (st->stack->stack_data[st->start+7]));
		highlv = conv_num(st,& (st->stack->stack_data[st->start+8]));
	}
	if(st->end > st->start+10) {
		job   = (unsigned int)conv_num(st,& (st->stack->stack_data[st->start+9]));
		upper = conv_num(st,& (st->stack->stack_data[st->start+10]));
	}
	chat_createnpcchat(map_id2nd(st->oid),
		limit,pub,trigger,name,strlen(name)+1,ev,zeny,lowlv,highlv,job,upper);
	return 0;
}

/*==========================================
 * NPCI[v`bgŔ
 *------------------------------------------
 */
int buildin_globalmes(struct script_state *st)
{
	char *mes, *name = NULL;

	mes = conv_str(st,& (st->stack->stack_data[st->start+2]));

	if(st->end > st->start+3) {	// NPC̎擾(123#456)
		name = conv_str(st,& (st->stack->stack_data[st->start+3]));
	} else {
		struct npc_data *nd = map_id2nd(st->oid);
		if(nd)
			name = nd->exname;
	}
	if(name)
		npc_globalmessage(name,mes);	// O[obZ[WM

	return 0;
}

/*==========================================
 * npc`bg폜
 *------------------------------------------
 */
int buildin_delwaitingroom(struct script_state *st)
{
	struct npc_data *nd;

	if( st->end > st->start+2 )
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+2])));
	else
		nd = map_id2nd(st->oid);
	chat_deletenpcchat(nd);
	return 0;
}

/*==========================================
 * `bgwvC[Ro
 *------------------------------------------
 */
int buildin_kickwaitingroom(struct script_state *st)
{
	struct map_session_data *sd;

	sd = map_id2sd(conv_num(st,& (st->stack->stack_data[st->start+2])));

	if(sd && sd->chatID)
		chat_leavechat(sd,1);
	return 0;
}

/*==========================================
 * npc`bgSRo
 *------------------------------------------
 */
int buildin_kickwaitingroomall(struct script_state *st)
{
	struct npc_data *nd;
	struct chat_data *cd;

	if( st->end > st->start+2 )
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+2])));
	else
		nd = map_id2nd(st->oid);

	if( nd && (cd = map_id2cd(nd->chat_id)) )
		chat_npckickall(cd);
	return 0;
}

/*==========================================
 * npc`bgCxgL
 *------------------------------------------
 */
int buildin_enablewaitingroomevent(struct script_state *st)
{
	struct npc_data *nd;
	struct chat_data *cd;

	if( st->end > st->start+2 )
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+2])));
	else
		nd = map_id2nd(st->oid);

	if(nd && (cd = map_id2cd(nd->chat_id)) )
		chat_enableevent(cd);
	return 0;
}

/*==========================================
 * npc`bgCxg
 *------------------------------------------
 */
int buildin_disablewaitingroomevent(struct script_state *st)
{
	struct npc_data *nd;
	struct chat_data *cd;

	if( st->end > st->start+2 )
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+2])));
	else
		nd = map_id2nd(st->oid);

	if(nd && (cd = map_id2cd(nd->chat_id)) )
		chat_disableevent(cd);
	return 0;
}

/*==========================================
 * npc`bgԎ擾
 *------------------------------------------
 */
int buildin_getwaitingroomstate(struct script_state *st)
{
	struct npc_data *nd;
	struct chat_data *cd;
	int type, val = -1;
	char *str = NULL;

	type = conv_num(st,& (st->stack->stack_data[st->start+2]));
	if(st->end > st->start+3)
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+3])));
	else
		nd = map_id2nd(st->oid);

	if(nd == NULL || (cd = map_id2cd(nd->chat_id)) == NULL){
		push_val(st->stack,C_INT,-1);
		return 0;
	}

	switch(type) {
		case 0:  val = cd->users;                        break;
		case 1:  val = cd->limit;                        break;
		case 2:  val = cd->trigger&0x7f;                 break;
		case 3:  val = ((cd->trigger&0x80) > 0)? 1: 0;   break;
		case 6:  val = cd->zeny;                         break;
		case 7:  val = cd->lowlv;                        break;
		case 8:  val = cd->highlv;                       break;
		case 9:  val = cd->job;                          break;
		case 10: val = cd->upper;                        break;
		case 32: val = (cd->users >= cd->limit)? 1: 0;   break;
		case 33: val = (cd->users >= cd->trigger)? 1: 0; break;
		case 4:
			str = (char *)aStrdup(cd->title);
			break;
		case 5:
			// pass\0tĂȂꍇ
			str = (char *)aCalloc(sizeof(cd->pass)+1, sizeof(char));
			memcpy(str, cd->pass, sizeof(cd->pass));
			break;
		case 16:
			str = (char *)aStrdup(cd->npc_event);
			break;
	}
	if(str)
		push_str(st->stack,C_STR,str);
	else
		push_val(st->stack,C_INT,val);
	return 0;
}

/*==========================================
 * `bgo[(Kl)[v
 *------------------------------------------
 */
int buildin_warpwaitingpc(struct script_state *st)
{
	int x,y,i,j,n,num;
	char *str;
	struct npc_data *nd = map_id2nd(st->oid);
	struct chat_data *cd;

	if(nd == NULL || (cd = map_id2cd(nd->chat_id)) == NULL)
		return 0;

	str = conv_str(st,& (st->stack->stack_data[st->start+2]));
	x   = conv_num(st,& (st->stack->stack_data[st->start+3]));
	y   = conv_num(st,& (st->stack->stack_data[st->start+4]));

	if(st->end > st->start+5)
		n = conv_num(st,& (st->stack->stack_data[st->start+5]));
	else
		n = cd->trigger&0x7f;

	num = add_str("$@warpwaitingpc");
	for(i=0, j=0; i<n && i<128; i++) {
		struct map_session_data *sd = cd->usersd[0];	// Xg擪PCXɁB

		if(sd == NULL)
			continue;
		mapreg_setreg(num+(j<<24),sd->bl.id,0);
		j++;
		script_warp(sd, str, x, y);
	}
	mapreg_setreg(add_str("$@warpwaitingpcnum"),j,0);
	return 0;
}

/*==========================================
 * NPC`bgɋPCIDXgAbv
 *------------------------------------------
 */
int buildin_getwaitingpcid(struct script_state *st)
{
	int i,j=0;
	struct map_session_data *sd = NULL;
	struct npc_data *nd;
	struct chat_data *cd;
	int num;
	char *name;
	char prefix, postfix;

	if( st->stack->stack_data[st->start+2].type != C_NAME ) {
		printf("buildin_getwaitingpcid: param not name\n");
		return 0;
	}
	num     = st->stack->stack_data[st->start+2].u.num;
	name    = str_buf+str_data[num&0x00ffffff].str;
	prefix  = *name;
	postfix = name[strlen(name)-1];

	if( prefix != '$' && prefix != '@' && prefix != '\'' ) {
		printf("buildin_getwaitingpcid: illegal scope !\n");
		return 0;
	}
	if( prefix != '$' && prefix != '\'' )
		sd = script_rid2sd(st);

	if(st->end > st->start+3)
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+3])));
	else
		nd = map_id2nd(st->oid);

	if(nd == NULL || (cd = map_id2cd(nd->chat_id)) == NULL)
		return 0;

	for(i=0; i<cd->users; i++) {
		struct map_session_data *pl_sd;
		pl_sd=cd->usersd[i];
		if(pl_sd) {
			void *v;
			if( postfix == '$' ) {
				char str[16];
				sprintf(str,"%d",pl_sd->bl.id);
				v = (void*)str;
			} else {
				v = (void*)pl_sd->bl.id;
			}
			set_reg(st,sd,num+(j<<24),name,v,st->stack->stack_data[st->start+2].ref);
			j++;
			if((num>>24)+j >= 128)
				break;
		}
	}
	return 0;
}

/*==========================================
 * RID̃A^b`
 *------------------------------------------
 */
int buildin_attachrid(struct script_state *st)
{
	struct map_session_data *sd;

	st->rid = conv_num(st,& (st->stack->stack_data[st->start+2]));

	sd = map_id2sd(st->rid);
	if(sd && sd->npc_id == 0) {
		sd->npc_id = st->oid;
	}
	push_val(st->stack,C_INT,(sd ? 1: 0));

	return 0;
}

/*==========================================
 * RID̃f^b`
 *------------------------------------------
 */
int buildin_detachrid(struct script_state *st)
{
	struct map_session_data *sd = map_id2sd(st->rid);

	st->rid = 0;
	if(sd && sd->npc_id == st->oid) {
		sd->npc_id = 0;
	}
	return 0;
}

/*==========================================
 * ݃`FbN
 *------------------------------------------
 */
int buildin_isloggedin(struct script_state *st)
{
	int id = conv_num(st,& (st->stack->stack_data[st->start+2]));

	push_val(st->stack,C_INT,(map_id2sd(id) ? 1: 0));
	return 0;
}

/*==========================================
 * }bvtOindexAhXԂ
 *------------------------------------------
 */
static int* script_conv_mapflag(int m,int type)
{
	if(m < 0)
		return NULL;

	switch(type) {
		case MF_NOSAVE:             return &map[m].flag.nosave;
		case MF_NOMEMO:             return &map[m].flag.nomemo;
		case MF_NOTELEPORT:         return &map[m].flag.noteleport;
		case MF_NOPORTAL:           return &map[m].flag.noportal;
		case MF_NORETURN:           return &map[m].flag.noreturn;
		case MF_MONSTER_NOTELEPORT: return &map[m].flag.monster_noteleport;
		case MF_NOBRANCH:           return &map[m].flag.nobranch;
		case MF_NOPENALTY:          return &map[m].flag.nopenalty;
		case MF_PVP:                return &map[m].flag.pvp;
		case MF_PVP_NOPARTY:        return &map[m].flag.pvp_noparty;
		case MF_PVP_NOGUILD:        return &map[m].flag.pvp_noguild;
		case MF_PVP_NIGHTMAREDROP:  return &map[m].flag.pvp_nightmaredrop;
		case MF_PVP_NOCALCRANK:     return &map[m].flag.pvp_nocalcrank;
		case MF_GVG:                return &map[m].flag.gvg;
		case MF_GVG_NOPARTY:        return &map[m].flag.gvg_noparty;
		case MF_GVG_NIGHTMAREDROP:  return &map[m].flag.gvg_nightmaredrop;
		case MF_NOZENYPENALTY:      return &map[m].flag.nozenypenalty;
		case MF_NOTRADE:            return &map[m].flag.notrade;
		case MF_NOSKILL:            return &map[m].flag.noskill;
		case MF_NOABRA:             return &map[m].flag.noabra;
		case MF_NODROP:             return &map[m].flag.nodrop;
		case MF_SNOW:               return &map[m].flag.snow;
		case MF_FOG:                return &map[m].flag.fog;
		case MF_SAKURA:             return &map[m].flag.sakura;
		case MF_LEAVES:             return &map[m].flag.leaves;
		case MF_RAIN:               return &map[m].flag.rain;
		case MF_FIREWORKS:          return &map[m].flag.fireworks;
		case MF_CLOUD1:             return &map[m].flag.cloud1;
		case MF_CLOUD2:             return &map[m].flag.cloud2;
		case MF_CLOUD3:             return &map[m].flag.cloud3;
		case MF_BASEEXP_RATE:       return &map[m].flag.base_exp_rate;
		case MF_JOBEXP_RATE:        return &map[m].flag.job_exp_rate;
		case MF_PK:                 return &map[m].flag.pk;
		case MF_PK_NOPARTY:         return &map[m].flag.pk_noparty;
		case MF_PK_NOGUILD:         return &map[m].flag.pk_noguild;
		case MF_PK_NIGHTMAREDROP:   return &map[m].flag.pk_nightmaredrop;
		case MF_PK_NOCALCRANK:      return &map[m].flag.pk_nocalcrank;
		case MF_NOICEWALL:          return &map[m].flag.noicewall;
		case MF_TURBO:              return &map[m].flag.turbo;
		case MF_NOREVIVE:           return &map[m].flag.norevive;
		case MF_NOCOMMAND:          return &map[m].flag.nocommand;
		case MF_NOJUMP:             return &map[m].flag.nojump;
	}
	return NULL;
}

/*==========================================
 * }bvtOݒ肷
 *------------------------------------------
 */
int buildin_setmapflag(struct script_state *st)
{
	char *mapname = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int type = conv_num(st,& (st->stack->stack_data[st->start+3]));
	int m;
	int *flag;

	if((m = script_mapname2mapid(st,mapname)) < 0)
		return 0;
	flag = script_conv_mapflag(m, type);

	if(flag)
	{
		*flag = 1;
		switch (type) {		// ꏈKvȃ}bvtO
		case MF_NOSAVE:
			if(st->end > st->start+6) {
				char *str = conv_str(st,& (st->stack->stack_data[st->start+4]));
				int x     = conv_num(st,& (st->stack->stack_data[st->start+5]));
				int y     = conv_num(st,& (st->stack->stack_data[st->start+6]));
				strncpy(map[m].save.map, str, 16);
				map[m].save.map[15] = '\0';	// force \0 terminal
				map[m].save.x = x;
				map[m].save.y = y;
			}
			break;
		case MF_BASEEXP_RATE:
		case MF_JOBEXP_RATE:
		case MF_NOCOMMAND:
			if(st->end > st->start+4) {
				*flag = conv_num(st,& (st->stack->stack_data[st->start+4]));
			}
			break;
		case MF_PVP_NIGHTMAREDROP:
		case MF_GVG_NIGHTMAREDROP:
		case MF_PK_NIGHTMAREDROP:
			if(st->end > st->start+6) {
				char buf[128];
				char *arg1 = conv_str(st,& (st->stack->stack_data[st->start+4]));
				char *arg2 = conv_str(st,& (st->stack->stack_data[st->start+5]));
				int  per   = conv_num(st,& (st->stack->stack_data[st->start+6]));
				snprintf(buf, sizeof(buf), "%s,%s,%d", arg1, arg2, per);
				npc_set_mapflag_sub(m, buf, type);
			}
			break;
		}
		map_field_setting();
	}
	return 0;
}

/*==========================================
 * }bvtO폜
 *------------------------------------------
 */
int buildin_removemapflag(struct script_state *st)
{
	char *mapname = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int type = conv_num(st,& (st->stack->stack_data[st->start+3]));
	int m;
	int *flag;

	if((m = script_mapname2mapid(st,mapname)) < 0)
		return 0;
	flag = script_conv_mapflag(m, type);

	if(flag)
	{
		*flag = 0;
		switch (type) {		// ꏈKvȃ}bvtO
		case MF_PVP_NIGHTMAREDROP:
		case MF_GVG_NIGHTMAREDROP:
		case MF_PK_NIGHTMAREDROP:
			{
				int i,j;
				for(i=0; i<MAX_DROP_PER_MAP; i++) {	// ỸhbvXg폜ċ󂫂l߂
					if(map[m].drop_list[i].drop_id == 0)
						break;
					if(map[m].drop_list[i].drop_flag != type)
						continue;
					for(j=i+1; j<MAX_DROP_PER_MAP && map[m].drop_list[j].drop_id != 0; j++);
					j--;
					if(i != j) {
						memcpy(&map[m].drop_list[i], &map[m].drop_list[j], sizeof(map[m].drop_list[0]));
					}
					memset(&map[m].drop_list[j], 0, sizeof(map[m].drop_list[0]));
					i--;
				}
			}
			break;
		}
		map_field_setting();
	}
	return 0;
}

/*==========================================
 * }bvtÕ`FbN
 *------------------------------------------
 */
int buildin_checkmapflag(struct script_state *st)
{
	char *mapname = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int type = conv_num(st,& (st->stack->stack_data[st->start+3]));
	int m;
	int *flag;

	if((m = script_mapname2mapid(st,mapname)) < 0) {
		push_val(st->stack,C_INT,-1);
		return 0;
	}

	flag = script_conv_mapflag(m ,type);

	push_val(st->stack,C_INT,((flag)? *flag: -1));
	return 0;
}

/*==========================================
 * PvPI
 *------------------------------------------
 */
int buildin_pvpon(struct script_state *st)
{
	char *str;
	int m;

	str = conv_str(st,& (st->stack->stack_data[st->start+2]));
	m = script_mapname2mapid(st,str);
	if(m >= 0 && !map[m].flag.pvp) {
		int i;
		struct map_session_data *pl_sd;

		map[m].flag.pvp = 1;
		clif_send0199(m,1);

		for(i=0; i<fd_max; i++) {	// l[v
			if(session[i] && (pl_sd = (struct map_session_data *)session[i]->session_data) && pl_sd->state.auth) {
				if(m == pl_sd->bl.m && pl_sd->pvp_timer == -1) {
					pl_sd->pvp_timer = add_timer(gettick()+200,pc_calc_pvprank_timer,pl_sd->bl.id,NULL);
					pl_sd->pvp_rank = 0;
					pl_sd->pvp_lastusers = 0;
					pl_sd->pvp_point = 5;
				}
			}
		}
		map_field_setting();
	}
	return 0;
}

/*==========================================
 * PvPIt
 *------------------------------------------
 */
int buildin_pvpoff(struct script_state *st)
{
	char *str;
	int m;

	str = conv_str(st,& (st->stack->stack_data[st->start+2]));
	m = script_mapname2mapid(st,str);
	if(m >= 0 && map[m].flag.pvp) {
		int i;
		struct map_session_data *pl_sd;

		map[m].flag.pvp = 0;
		clif_send0199(m,0);

		for(i=0; i<fd_max; i++) {	// l[v
			if(session[i] && (pl_sd = (struct map_session_data *)session[i]->session_data) && pl_sd->state.auth) {
				if(m == pl_sd->bl.m) {
					clif_pvpset(pl_sd,0,0,2);
					if(pl_sd->pvp_timer != -1) {
						delete_timer(pl_sd->pvp_timer,pc_calc_pvprank_timer);
						pl_sd->pvp_timer = -1;
					}
				}
			}
		}
		map_field_setting();
	}

	return 0;
}

/*==========================================
 * GvGI
 *------------------------------------------
 */
int buildin_gvgon(struct script_state *st)
{
	char *str;
	int m;

	str = conv_str(st,& (st->stack->stack_data[st->start+2]));
	m = script_mapname2mapid(st,str);
	if(m >= 0 && !map[m].flag.gvg) {
		map[m].flag.gvg = 1;
		clif_send0199(m,3);
		map_field_setting();
	}

	return 0;
}

/*==========================================
 * GvGIt
 *------------------------------------------
 */
int buildin_gvgoff(struct script_state *st)
{
	char *str;
	int m;

	str = conv_str(st,& (st->stack->stack_data[st->start+2]));
	m = script_mapname2mapid(st,str);
	if(m >= 0 && map[m].flag.gvg) {
		map[m].flag.gvg = 0;
		clif_send0199(m,0);
		map_field_setting();
	}

	return 0;
}

/*==========================================
 * NPCG[V
 *------------------------------------------
 */
int buildin_emotion(struct script_state *st)
{
	struct npc_data *nd;
	int type;

	type = conv_num(st,& (st->stack->stack_data[st->start+2]));
	if( st->end > st->start+3 )
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+3])));
	else
		nd = map_id2nd(st->oid);

	if(nd) {
		clif_emotion(&nd->bl,type);
	} else {
		struct block_list *bl = map_id2bl(st->rid);
		if(bl)
			clif_emotion(bl,type);
	}
	return 0;
}

/*==========================================
 * ΏۂړAUp
 *------------------------------------------
 */
static int buildin_maprespawnguildid_sub(struct block_list *bl,va_list ap)
{
	int g_id = va_arg(ap,int);
	int flag = va_arg(ap,int);

	nullpo_retr(0, bl);

	if(bl->type == BL_PC && flag&3) {
		struct map_session_data *sd = (struct map_session_data *)bl;
		if(sd) {
			int match = (g_id > 0 && sd->status.guild_id == g_id);
			if( (flag&1 && match) || (flag&2 && !match) )
				pc_setpos(sd,sd->status.save_point.map,sd->status.save_point.x,sd->status.save_point.y,3);
		}
	}
	else if(bl->type == BL_MOB && flag&4) {
		struct mob_data *md = (struct mob_data *)bl;
		if(md) {
			if(md->class_ < 1285 || md->class_ > 1288)
				unit_remove_map(&md->bl,1,0);
		}
	}
	return 0;
}

int buildin_maprespawnguildid(struct script_state *st)
{
	char *mapname = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int g_id      = conv_num(st,& (st->stack->stack_data[st->start+3]));
	int flag      = conv_num(st,& (st->stack->stack_data[st->start+4]));
	int m;

	m = script_mapname2mapid(st,mapname);
	if(m >= 0)
		map_foreachinarea(buildin_maprespawnguildid_sub,m,0,0,map[m].xs,map[m].ys,BL_PC|BL_MOB,g_id,flag);
	return 0;
}

/*==========================================
 * UJn
 *------------------------------------------
 */
int buildin_agitstart(struct script_state *st)
{
	if(!agit_flag) {
		agit_flag = 1;
		guild_agit_start();
	}
	return 0;
}

/*==========================================
 * UI
 *------------------------------------------
 */
int buildin_agitend(struct script_state *st)
{
	if(agit_flag) {
		agit_flag = 0;
		guild_agit_end();
	}
	return 0;
}

/*==========================================
 * U풆ǂ
 *------------------------------------------
 */
int buildin_agitcheck(struct script_state *st)
{
	push_val(st->stack,C_INT,agit_flag);
	return 0;
}

/*==========================================
 * NPC̃MhGuύX
 *------------------------------------------
 */
int buildin_flagemblem(struct script_state *st)
{
	int g_id = conv_num(st,& (st->stack->stack_data[st->start+2]));
	struct npc_data *nd = map_id2nd(st->oid);

	if(nd && g_id > 0)
		nd->u.scr.guild_id = g_id;

	return 0;
}

/*==========================================
 * AWg̖OԂ
 *------------------------------------------
 */
int buildin_getcastlename(struct script_state *st)
{
	char *mapname = conv_str(st,& (st->stack->stack_data[st->start+2]));
	struct guild_castle *gc;
	int i,type = 0;

	if(st->end>st->start+3)
		type = conv_num(st,& (st->stack->stack_data[st->start+3]));

	if(strcmp(mapname,"this") == 0) {
		int m = script_mapname2mapid(st,mapname);
		if(m < 0) {
			push_str(st->stack,C_CONSTSTR,"");
			return 0;
		}
		mapname = map[m].name;
	}

	for(i=0; i<MAX_GUILDCASTLE; i++) {
		if((gc = guild_castle_search(i)) != NULL) {
			if(strcmp(mapname,gc->map_name) == 0) {
				push_str(st->stack,C_STR,(unsigned char *)aStrdup(type? gc->castle_name: gc->area_name));
				return 0;
			}
		}
	}
	push_str(st->stack,C_CONSTSTR,"");

	return 0;
}

/*==========================================
 * AWg̃f[^擾
 *------------------------------------------
 */
int buildin_getcastledata(struct script_state *st)
{
	char *mapname = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int idx       = conv_num(st,& (st->stack->stack_data[st->start+3]));
	struct guild_castle *gc;
	int i,val = 0;

	if(strcmp(mapname,"this") == 0) {
		int m = script_mapname2mapid(st,mapname);
		if(m < 0) {
			push_val(st->stack,C_INT,0);
			return 0;
		}
		mapname = map[m].name;
	}

	if(idx == 0) {
		int j = -1;
		for(i=0; i<MAX_GUILDCASTLE; i++) {
			if((gc = guild_castle_search(i)) != NULL && strcmp(mapname,gc->map_name) == 0)
				j = i;
		}
		if(j >= 0) {
			if(st->end > st->start+4) {
				char *event = conv_str(st,& (st->stack->stack_data[st->start+4]));
				guild_addcastleinfoevent(j,17,event);
			}
			for(i=1; i<10; i++) {
				guild_castledataload(gc->castle_id,i);
			}
		}
	} else {
		for(i=0; i<MAX_GUILDCASTLE; i++) {
			if((gc = guild_castle_search(i)) != NULL && strcmp(mapname,gc->map_name) == 0) {
				switch(idx) {
					case 1:  val = gc->guild_id;    break;
					case 2:  val = gc->economy;     break;
					case 3:  val = gc->defense;     break;
					case 4:  val = gc->triggerE;    break;
					case 5:  val = gc->triggerD;    break;
					case 6:  val = gc->nextTime;    break;
					case 7:  val = gc->payTime;     break;
					case 8:  val = gc->createTime;  break;
					case 9:  val = gc->visibleC;    break;
					case 10: val = gc->guardian[0].visible; break;
					case 11: val = gc->guardian[1].visible; break;
					case 12: val = gc->guardian[2].visible; break;
					case 13: val = gc->guardian[3].visible; break;
					case 14: val = gc->guardian[4].visible; break;
					case 15: val = gc->guardian[5].visible; break;
					case 16: val = gc->guardian[6].visible; break;
					case 17: val = gc->guardian[7].visible; break;
				}
				break;
			}
		}
	}
	push_val(st->stack,C_INT,val);
	return 0;
}

/*==========================================
 * AWg̃f[^ύX
 *------------------------------------------
 */
int buildin_setcastledata(struct script_state *st)
{
	char *mapname = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int idx       = conv_num(st,& (st->stack->stack_data[st->start+3]));
	int value     = conv_num(st,& (st->stack->stack_data[st->start+4]));
	struct guild_castle *gc;
	int i;

	if(strcmp(mapname,"this") == 0) {
		int m = script_mapname2mapid(st,mapname);
		if(m < 0)
			return 0;
		mapname = map[m].name;
	}

	for(i=0; i<MAX_GUILDCASTLE; i++) {
		if((gc = guild_castle_search(i)) != NULL && strcmp(mapname,gc->map_name) == 0) {
			switch(idx) {
				case 1:  gc->guild_id    = value; break;
				case 2:  gc->economy     = value; break;
				case 3:  gc->defense     = value; break;
				case 4:  gc->triggerE    = value; break;
				case 5:  gc->triggerD    = value; break;
				case 6:  gc->nextTime    = value; break;
				case 7:  gc->payTime     = value; break;
				case 8:  gc->createTime  = value; break;
				case 9:  gc->visibleC    = value; break;
				case 10: gc->guardian[0].visible = value; break;
				case 11: gc->guardian[1].visible = value; break;
				case 12: gc->guardian[2].visible = value; break;
				case 13: gc->guardian[3].visible = value; break;
				case 14: gc->guardian[4].visible = value; break;
				case 15: gc->guardian[5].visible = value; break;
				case 16: gc->guardian[6].visible = value; break;
				case 17: gc->guardian[7].visible = value; break;
				default: return 0;
			}
			guild_castledatasave(gc->castle_id,idx,value);
			break;
		}
	}
	return 0;
}

/*==========================================
 * Mhv
 *------------------------------------------
 */
int buildin_requestguildinfo(struct script_state *st)
{
	int guild_id=conv_num(st,& (st->stack->stack_data[st->start+2]));
	char *event=NULL;

	if( st->end>st->start+3 )
		event=conv_str(st,& (st->stack->stack_data[st->start+3]));

	if(guild_id>0)
		guild_npc_request_info(guild_id,event);
	return 0;
}

/*==========================================
 * J[h̐𓾂
 *------------------------------------------
 */
int buildin_getequipcardcnt(struct script_state *st)
{
	int num, i = -1;
	struct map_session_data *sd;

	num=conv_num(st,& (st->stack->stack_data[st->start+2]));
	sd=script_rid2sd(st);

	if(num > 0 && num <= 11)
		i=pc_checkequip(sd,equip_pos[num-1]);

	if(i >= 0) {
		int n,c=0;
		if(itemdb_isspecial(sd->status.inventory[i].card[0])){ // EO̓J[hȂ
			push_val(st->stack,C_INT,0);
			return 0;
		}
		for(n=0; n<sd->inventory_data[i]->slot; n++) {
			short card_id = sd->status.inventory[i].card[n];
			if(card_id > 0 && itemdb_type(card_id) == 6)
				c++;
		}
		push_val(st->stack,C_INT,c);
		return 0;
	}
	push_val(st->stack,C_INT,0);
	return 0;
}

/*=================================================================
 * J[hO
 * type=0: A1:J[hA2:A3:A4:
 *-----------------------------------------------------------------
 */
static int removecards_sub(struct map_session_data *sd,int i,int typefail,int pos)
{
	struct item item_tmp;
	int j,n,flag,slot,removed_flag=0;
	short card_set[4] = { 0,0,0,0 };

	if(i < 0 || i >= MAX_INVENTORY)
		return 0;
	if(itemdb_isspecial(sd->status.inventory[i].card[0]))	// EO͏Ȃ
		return 0;

	slot = sd->inventory_data[i]->slot;
	if(slot > 4)
		slot = 4;
	if(pos < 0 || pos > slot)
		return 0;

	for(n=0,j=0; n < slot; n++) {
		short card_id = sd->status.inventory[i].card[n];
		if(card_id <= 0 || itemdb_type(card_id) != 6)
			continue;
		if(pos == 0 || pos-1 == n) {			// w|WVƈvꍇ
			removed_flag = 1;
			if(typefail == 2 || typefail == 4) {	// J[hԋp
				memset(&item_tmp, 0, sizeof(item_tmp));
				item_tmp.nameid = card_id;
				item_tmp.identify = 1;
				if( (flag = pc_additem(sd,&item_tmp,1)) ) {	// ĂȂȂhbv
					clif_additem(sd,0,0,flag);
					map_addflooritem(&item_tmp,1,sd->bl.m,sd->bl.x,sd->bl.y,0,0,0,0);
				}
			}
		} else {
			card_set[j++] = card_id;	// OΏۊÕJ[hۑ
		}
	}

	if(removed_flag) {
		if(typefail == 0 || typefail == 2) {		// ďI
			pc_delitem(sd,i,1,0);
		}
		else if(typefail == 1 || typefail == 4) {	// wJ[h菜ACe擾
			flag = 0;
			memset(&item_tmp, 0, sizeof(item_tmp));
			item_tmp.nameid    = sd->status.inventory[i].nameid;
			item_tmp.identify  = 1;
			item_tmp.refine    = sd->status.inventory[i].refine;
			item_tmp.attribute = sd->status.inventory[i].attribute;
			memcpy(&item_tmp.card, &card_set, sizeof(card_set));
			pc_delitem(sd,i,1,0);
			if( (flag=pc_additem(sd,&item_tmp,1)) ) {	// ĂȂȂhbv
				clif_additem(sd,0,0,flag);
				map_addflooritem(&item_tmp,1,sd->bl.m,sd->bl.x,sd->bl.y,0,0,0,0);
			}
		}
		return 1;	// 
	}
	return 0;
}

/*==========================================
 * J[hO
 *------------------------------------------
 */
int buildin_successremovecards(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	int i=-1,num,pos=0;

	nullpo_retr(0, sd);

	num = conv_num(st,& (st->stack->stack_data[st->start+2]));
	if(num > 0 && num <= 11)
		i = pc_checkequip(sd,equip_pos[num-1]);

	if(st->end > st->start+3) {
		pos = conv_num(st,& (st->stack->stack_data[st->start+3]));
	}

	if(removecards_sub(sd, i, 4, pos)) {	// failtype=4Ƃ
		// GtFNg
		clif_misceffect2(&sd->bl,154);
	}
	return 0;
}

/*==========================================
 * J[hOs
 *------------------------------------------
 */
int buildin_failedremovecards(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	int i=-1,num,typefail,pos=0;

	nullpo_retr(0, sd);

	num = conv_num(st,& (st->stack->stack_data[st->start+2]));
	typefail = conv_num(st,& (st->stack->stack_data[st->start+3]));

	if(typefail < 0 || typefail > 3)
		return 0;

	if(num > 0 && num <= 11)
		i = pc_checkequip(sd,equip_pos[num-1]);

	if(st->end > st->start+4)
		pos = conv_num(st,& (st->stack->stack_data[st->start+4]));

	if(removecards_sub(sd, i, typefail, pos)) {
		// sGtFNg
		clif_misceffect2(&sd->bl,155);
	}
	return 0;
}

/*==========================================
 * 
 *------------------------------------------
 */
int buildin_marriage(struct script_state *st)
{
	char *partner=conv_str(st,& (st->stack->stack_data[st->start+2]));
	struct map_session_data *sd=script_rid2sd(st);
	struct map_session_data *p_sd=map_nick2sd(partner);

	if(sd==NULL || p_sd==NULL || pc_isbaby(sd) || pc_marriage(sd,p_sd) < 0)
		push_val(st->stack,C_INT,0);
	else
		push_val(st->stack,C_INT,sd->status.partner_id);
	return 0;
}

/*==========================================
 * p̃GtFNg
 *------------------------------------------
 */
int buildin_wedding_effect(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);

	if(sd)
		clif_wedding_effect(&sd->bl);
	return 0;
}

/*==========================================
 * 
 *------------------------------------------
 */
int buildin_divorce(struct script_state *st)
{
	int num, partner_id;
	struct map_session_data *sd = script_rid2sd(st);

	if(sd == NULL) {
		push_val(st->stack,C_INT,0);
		return 0;
	}
	partner_id = sd->status.partner_id;

	num = conv_num(st,& (st->stack->stack_data[st->start+2]));
	if(num == 0) {
		if(pc_divorce(sd) < 0) {
			push_val(st->stack,C_INT,0);
			return 0;
		}
	}
	push_val(st->stack,C_INT,partner_id);

	return 0;
}

/*==========================================
 * {q
 *------------------------------------------
 */
int buildin_adoption(struct script_state *st)
{
	int ret;
	char *name = conv_str(st,& (st->stack->stack_data[st->start+2]));
	struct map_session_data *sd = script_rid2sd(st);
	struct map_session_data *p1 = map_nick2sd(name);

	if(st->end > st->start+3) {
		char *name2 = conv_str(st,& (st->stack->stack_data[st->start+3]));
		struct map_session_data *p2 = map_nick2sd(name2);
		ret = pc_adoption_sub(sd,p1,p2);
	} else {
		ret = pc_adoption(sd,p1);
	}
	push_val(st->stack,C_INT,ret);
	return 0;
}

/*==========================================
 * {q
 *------------------------------------------
 */
int buildin_breakadoption(struct script_state *st)
{
	struct map_session_data *sd;

	if( st->end>st->start+2 ) {
		char *name = conv_str(st,& (st->stack->stack_data[st->start+2]));
		sd = map_nick2sd(name);
	} else {
		sd = script_rid2sd(st);
	}

	if(sd == NULL) {
		push_val(st->stack,C_INT,0);
		return 0;
	}

	push_val(st->stack,C_INT,pc_break_adoption(sd));
	return 0;
}

/*==========================================
 * IDItem
 *------------------------------------------
 */
int buildin_getitemname(struct script_state *st)
{
	int item_id;
	struct item_data *item;

	item_id = conv_num(st,& (st->stack->stack_data[st->start+2]));

	item = itemdb_exists(item_id);
	if(item)
		push_str(st->stack,C_STR,(unsigned char *)aStrdup(item->jname));
	else
		push_str(st->stack,C_CONSTSTR,"");

	return 0;
}

/*==========================================
 * PC̏iǂݎ
 *------------------------------------------
 */
int buildin_getinventorylist(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	int i,j=0;

	if(sd == NULL)
		return 0;

	for(i=0; i<MAX_INVENTORY && j<128; i++) {
		if(sd->status.inventory[i].nameid > 0 && sd->status.inventory[i].amount > 0) {
			pc_setreg(sd,add_str("@inventorylist_index")+(j<<24),i);
			pc_setreg(sd,add_str("@inventorylist_id")+(j<<24),sd->status.inventory[i].nameid);
			pc_setreg(sd,add_str("@inventorylist_amount")+(j<<24),sd->status.inventory[i].amount);
			pc_setreg(sd,add_str("@inventorylist_equip")+(j<<24),sd->status.inventory[i].equip);
			pc_setreg(sd,add_str("@inventorylist_refine")+(j<<24),sd->status.inventory[i].refine);
			pc_setreg(sd,add_str("@inventorylist_identify")+(j<<24),sd->status.inventory[i].identify);
			pc_setreg(sd,add_str("@inventorylist_attribute")+(j<<24),sd->status.inventory[i].attribute);
			pc_setreg(sd,add_str("@inventorylist_card1")+(j<<24),sd->status.inventory[i].card[0]);
			pc_setreg(sd,add_str("@inventorylist_card2")+(j<<24),sd->status.inventory[i].card[1]);
			pc_setreg(sd,add_str("@inventorylist_card3")+(j<<24),sd->status.inventory[i].card[2]);
			pc_setreg(sd,add_str("@inventorylist_card4")+(j<<24),sd->status.inventory[i].card[3]);
			pc_setreg(sd,add_str("@inventorylist_limit")+(j<<24),sd->status.inventory[i].limit);
			j++;
		}
	}
	pc_setreg(sd,add_str("@inventorylist_count"),j);
	return 0;
}

/*==========================================
 * PC̃J[giǂݎ
 *------------------------------------------
 */
int buildin_getcartlist(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	int i,j=0;

	if(sd == NULL)
		return 0;

	for(i=0; i<MAX_CART && j<128; i++) {
		if(sd->status.cart[i].nameid > 0 && sd->status.cart[i].amount > 0) {
			pc_setreg(sd,add_str("@cartlist_index")+(j<<24),i);
			pc_setreg(sd,add_str("@cartlist_id")+(j<<24),sd->status.cart[i].nameid);
			pc_setreg(sd,add_str("@cartlist_amount")+(j<<24),sd->status.cart[i].amount);
			pc_setreg(sd,add_str("@cartlist_refine")+(j<<24),sd->status.cart[i].refine);
			pc_setreg(sd,add_str("@cartlist_identify")+(j<<24),sd->status.cart[i].identify);
			pc_setreg(sd,add_str("@cartlist_attribute")+(j<<24),sd->status.cart[i].attribute);
			pc_setreg(sd,add_str("@cartlist_card1")+(j<<24),sd->status.cart[i].card[0]);
			pc_setreg(sd,add_str("@cartlist_card2")+(j<<24),sd->status.cart[i].card[1]);
			pc_setreg(sd,add_str("@cartlist_card3")+(j<<24),sd->status.cart[i].card[2]);
			pc_setreg(sd,add_str("@cartlist_card4")+(j<<24),sd->status.cart[i].card[3]);
			pc_setreg(sd,add_str("@cartlist_limit")+(j<<24),sd->status.cart[i].limit);
			j++;
		}
	}
	pc_setreg(sd,add_str("@cartlist_count"),j);
	return 0;
}

/*==========================================
 * PC̃XLǂݎ
 *------------------------------------------
 */
int buildin_getskilllist(struct script_state *st)
{
	struct map_session_data *sd=script_rid2sd(st);
	int i,j=0,k=0;

	if(!sd)
		return 0;
	for(i=0;i<MAX_SKILL;i++){
		if(sd->status.skill[i].id <= 0 || sd->status.skill[i].lv <= 0)
			continue;
		if(j >= 128) {
			j=0;
			k++;
		}
		if(k == 0) {
			pc_setreg(sd,add_str("@skilllist_id")+(j<<24),sd->status.skill[i].id);
			pc_setreg(sd,add_str("@skilllist_lv")+(j<<24),sd->status.skill[i].lv);
			pc_setreg(sd,add_str("@skilllist_flag")+(j<<24),sd->status.skill[i].flag);
		} else {
			char buf[32];
			sprintf(buf,"@skilllist_id[%d]",k);
			pc_setreg(sd,add_str(buf)+(j<<24),sd->status.skill[i].id);
			sprintf(buf,"@skilllist_lv[%d]",k);
			pc_setreg(sd,add_str(buf)+(j<<24),sd->status.skill[i].lv);
			sprintf(buf,"@skilllist_flag[%d]",k);
			pc_setreg(sd,add_str(buf)+(j<<24),sd->status.skill[i].flag);
		}
		j++;
	}
	pc_setreg(sd,add_str("@skilllist_count"),k*128+j);
	return 0;
}

/*==========================================
 * ACȇS폜
 *------------------------------------------
 */
int buildin_clearitem(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	int i;

	if(sd == NULL)
		return 0;
	for (i=0; i<MAX_INVENTORY; i++) {
		if (sd->status.inventory[i].amount) {
			if (sd->status.inventory[i].card[0] == (short)0xff00)
				intif_delete_petdata(*((long *)(&sd->status.inventory[i].card[1])));
			pc_delitem(sd, i, sd->status.inventory[i].amount, 0);
		}
	}
	return 0;
}

/*==========================================
 * J[gACȇS폜
 *------------------------------------------
 */
int buildin_clearcartitem(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	int i;

	if(sd == NULL)
		return 0;
	for (i=0; i<MAX_CART; i++) {
		if (sd->status.cart[i].amount) {
			if (sd->status.cart[i].card[0] == (short)0xff00)
				intif_delete_petdata(*((long *)(&sd->status.cart[i].card[1])));
			pc_cart_delitem(sd, i, sd->status.cart[i].amount, !pc_iscarton(sd));
		}
	}
	return 0;
}

/*==========================================
 * C\ACe𐔂
 *------------------------------------------
 */
int buildin_getrepairableitemcount(struct script_state *st)
{
	struct map_session_data *sd=script_rid2sd(st);
	int i,c=0;

	if(sd){
		for(i=0;i<MAX_INVENTORY;i++){
			if(sd->status.inventory[i].nameid > 0 && sd->status.inventory[i].amount > 0 && sd->status.inventory[i].attribute){
				c++;
			}
		}
	}
	push_val(st->stack,C_INT,c);
	return 0;
}

/*==========================================
 * C\ACeC
 *------------------------------------------
 */
int buildin_repairitem(struct script_state *st)
{
	struct map_session_data *sd=script_rid2sd(st);
	int i,c=0;

	if(sd){
		for(i=0;i<MAX_INVENTORY;i++){
			if(sd->status.inventory[i].nameid > 0 && sd->status.inventory[i].amount > 0 && sd->status.inventory[i].attribute){
				sd->status.inventory[i].attribute = 0;
				c++;
			}
		}
		if(c > 0) {
			clif_itemlist(sd);
			clif_equiplist(sd);
		}
	}
	return 0;
}

/*==========================================
 * NPCNX`FW
 * class͕ς肽class
 * type͒ʏ0Ȃ̂ȁH
 *------------------------------------------
 */
int buildin_classchange(struct script_state *st)
{
	int class_ = conv_num(st,& (st->stack->stack_data[st->start+2]));
	int type   = conv_num(st,& (st->stack->stack_data[st->start+3]));
	struct block_list *bl;

	if((bl = map_id2bl(st->oid)) != NULL)
		clif_class_change(bl,class_,type);

	return 0;
}

/*==========================================
 * NPC甭GtFNg
 *------------------------------------------
 */
int buildin_misceffect(struct script_state *st)
{
	struct npc_data *nd;
	int type;

	type = conv_num(st,& (st->stack->stack_data[st->start+2]));
	if(st->end > st->start+3)
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+3])));
	else
		nd = map_id2nd(st->oid);

	if(nd) {
		clif_misceffect2(&nd->bl,type);
	} else {
		struct block_list *bl = map_id2bl(st->rid);
		if(bl)
			clif_misceffect2(bl,type);
	}
	return 0;
}

/*==========================================
 * GAPCɔGtFNg
 *------------------------------------------
 */
static int buildin_misceffect_sub(struct block_list *bl,va_list ap)
{
	struct map_session_data *sd;
	int type = va_arg(ap,int);

	nullpo_retr(0, bl);
	nullpo_retr(0, sd = (struct map_session_data *)bl);

	clif_misceffect3(sd->fd, sd->bl.id, type);
	return 0;
}

int buildin_areamisceffect(struct script_state *st)
{
	char *str;
	int type,m,x0,y0,x1,y1;

	str  = conv_str(st,& (st->stack->stack_data[st->start+2]));
	x0   = conv_num(st,& (st->stack->stack_data[st->start+3]));
	y0   = conv_num(st,& (st->stack->stack_data[st->start+4]));
	x1   = conv_num(st,& (st->stack->stack_data[st->start+5]));
	y1   = conv_num(st,& (st->stack->stack_data[st->start+6]));
	type = conv_num(st,& (st->stack->stack_data[st->start+7]));

	m = script_mapname2mapid(st,str);
	if(m >= 0)
		map_foreachinarea(buildin_misceffect_sub,m,x0,y0,x1,y1,BL_PC,type);
	return 0;
}

/*==========================================
 * TEhGtFNg
 *------------------------------------------
 */
int buildin_soundeffect(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	char *name;
	int type, interval = 0;

	name = conv_str(st,& (st->stack->stack_data[st->start+2]));
	type = conv_num(st,& (st->stack->stack_data[st->start+3]));
	if(st->end > st->start+4)
		interval = conv_num(st,& (st->stack->stack_data[st->start+4]));

	if(sd) {
		if(st->oid)
			clif_soundeffect(sd,map_id2bl(st->oid),name,type,interval);
		else
			clif_soundeffect(sd,&sd->bl,name,type,interval);
	}
	return 0;
}

/*==========================================
 * ͈͎wTEhGtFNg
 *------------------------------------------
 */
static int buildin_soundeffect_sub(struct block_list *bl,va_list ap)
{
	struct map_session_data *sd;
	char *name   = va_arg(ap,char *);
	int type     = va_arg(ap,int);
	int interval = va_arg(ap,int);

	nullpo_retr(0, bl);
	nullpo_retr(0, sd = (struct map_session_data *)bl);

	clif_soundeffect(sd,bl,name,type,interval);

	return 0;
}

int buildin_areasoundeffect(struct script_state *st)
{
	char *name,*str;
	int type,m,x0,y0,x1,y1;
	int interval = 0;

	str  = conv_str(st,& (st->stack->stack_data[st->start+2]));
	x0   = conv_num(st,& (st->stack->stack_data[st->start+3]));
	y0   = conv_num(st,& (st->stack->stack_data[st->start+4]));
	x1   = conv_num(st,& (st->stack->stack_data[st->start+5]));
	y1   = conv_num(st,& (st->stack->stack_data[st->start+6]));
	name = conv_str(st,& (st->stack->stack_data[st->start+7]));
	type = conv_num(st,& (st->stack->stack_data[st->start+8]));
	if(st->end > st->start+9)
		interval = conv_num(st,& (st->stack->stack_data[st->start+9]));

	m = script_mapname2mapid(st,str);
	if(m >= 0)
		map_foreachinarea(buildin_soundeffect_sub,m,x0,y0,x1,y1,BL_PC,name,type,interval);
	return 0;
}

/*==========================================
 * gmcommand
 * suggested on the forums...
 *------------------------------------------
 */
int buildin_gmcommand(struct script_state *st)
{
	char *str;

	str = conv_str(st,& (st->stack->stack_data[st->start+2]));

	if(st->rid) {
		struct map_session_data *sd = map_id2sd(st->rid);
		if(sd) {
			is_atcommand_sub(sd->fd, sd, str, 99);
			return 0;
		}
	}
	if(st->oid) {
		// lȂCxgnNPCŎgꍇ̂߂̃_~[map_session_data
		struct map_session_data dummy_sd;
		struct block_list *bl = map_id2bl(st->oid);

		if(bl) {
			memset(&dummy_sd, 0, sizeof(struct map_session_data));
			memcpy(&dummy_sd.bl, bl, sizeof(struct block_list));
			dummy_sd.fd = 0;
			dummy_sd.bl.prev = NULL;
			dummy_sd.bl.next = NULL;
			if(bl->type == BL_NPC) {
				char *p;
				strncpy(dummy_sd.status.name, ((struct npc_data *)bl)->name, 24);
				if((p = strchr(dummy_sd.status.name,'#')) != NULL)
					*p = 0;
			} else {
				strncpy(dummy_sd.status.name, "dummy", 24);
			}
			is_atcommand_sub(dummy_sd.fd, &dummy_sd, str, 99);
			return 0;
		}
	}

	return 0;
}

/*==========================================
 * RgɃbZ[W\
 *------------------------------------------
 */
int buildin_dispbottom(struct script_state *st)
{
	struct map_session_data *sd=script_rid2sd(st);
	char *message;

	message=conv_str(st,& (st->stack->stack_data[st->start+2]));
	if(sd)
		clif_disp_onlyself(sd->fd,message);
	return 0;
}

/*==========================================
 * T[o[̑SS(h+HP/SPS)
 *------------------------------------------
 */
int buildin_recovery(struct script_state *st)
{
	struct map_session_data *sd;
	int i;

	for(i=0; i<fd_max; i++) {
		if(session[i] && (sd = (struct map_session_data *)session[i]->session_data) && sd->state.auth) {
			sd->status.hp = sd->status.max_hp;
			sd->status.sp = sd->status.max_sp;
			clif_updatestatus(sd, SP_HP);
			clif_updatestatus(sd, SP_SP);
			if(unit_isdead(&sd->bl)) {
				pc_setstand(sd);
				clif_resurrection(&sd->bl, 1);
			}
		}
	}
	return 0;
}

/*==========================================
 * zĘAĂybg̏擾
 * 0:pet_id 1:pet_class 2:pet_name
 * 3:friendly 4:hungry
 *------------------------------------------
 */
int buildin_getpetinfo(struct script_state *st)
{
	struct map_session_data *sd=script_rid2sd(st);
	int type=conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(sd && sd->status.pet_id){
		switch(type){
			case 0:
				push_val(st->stack,C_INT,sd->status.pet_id);
				break;
			case 1:
				if(sd->pet.class_)
					push_val(st->stack,C_INT,sd->pet.class_);
				else
					push_val(st->stack,C_INT,0);
				break;
			case 2:
				if(sd->pet.name[0])
					push_str(st->stack,C_STR,(unsigned char *)aStrdup(sd->pet.name));
				else
					push_str(st->stack,C_CONSTSTR,"");
				break;
			case 3:
				push_val(st->stack,C_INT,sd->pet.intimate);
				break;
			case 4:
				push_val(st->stack,C_INT,sd->pet.hungry);
				break;
			default:
				push_val(st->stack,C_INT,0);
				break;
		}
	}else{
		push_val(st->stack,C_INT,0);
	}
	return 0;
}

/*==========================================
 * AĂzNX̏擾
 * 0:homun_id 1:homun_base_lv 2:homun_name
 * 3:friendly 4:hungry 5:homun_class
 *------------------------------------------
 */
int buildin_gethomuninfo(struct script_state *st)
{
	struct map_session_data *sd=script_rid2sd(st);
	int type=conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(sd && sd->status.homun_id){
		switch(type){
			case 0:
				push_val(st->stack,C_INT,sd->status.homun_id);
				break;
			case 1:
				if(sd->hd && sd->hd->status.base_level)
					push_val(st->stack,C_INT,sd->hd->status.base_level);
				else
					push_val(st->stack,C_INT,0);
				break;
			case 2:
				if(sd->hd && sd->hd->status.name[0])
					push_str(st->stack,C_STR,(unsigned char *)aStrdup(sd->hd->status.name));
				else
					push_str(st->stack,C_CONSTSTR,"");
				break;
			case 3:
				if( sd->hd )
					push_val(st->stack,C_INT,sd->hd->intimate);
				else
					push_val(st->stack,C_INT,0);
				break;
			case 4:
				if( sd->hd )
					push_val(st->stack,C_INT,sd->hd->status.hungry);
				else
					push_val(st->stack,C_INT,0);
				break;
			case 5:
				if( sd->hd )
					push_val(st->stack,C_INT,sd->hd->status.class_);
				else
					push_val(st->stack,C_INT,0);
				break;
			default:
				push_val(st->stack,C_INT,0);
				break;
		}
	}else{
		push_val(st->stack,C_INT,0);
	}
	return 0;
}

/*==========================================
 * wID̃J[htȂ(1,0Ԃ)
 *------------------------------------------
 */
int buildin_checkequipedcard(struct script_state *st)
{
	struct map_session_data *sd=script_rid2sd(st);
	int n,i,c;

	c=conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(sd){
		for(i=0;i<MAX_INVENTORY;i++){
			if(sd->status.inventory[i].nameid > 0 && sd->status.inventory[i].amount){
				if(itemdb_isspecial(sd->status.inventory[i].card[0]))
					continue;
				for(n=0;n<4;n++){
					if(sd->status.inventory[i].card[n]==c){
						push_val(st->stack,C_INT,1);
						return 0;
					}
				}
			}
		}
	}
	push_val(st->stack,C_INT,0);
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_jump_zero(struct script_state *st)
{
	int sel = conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(!sel) {
		if(st->stack->stack_data[st->start+3].type != C_POS) {
			printf("buildin_jump_zero: not label !\n");
			st->state = END;
			return 0;
		}
		st->pos = conv_num(st,& (st->stack->stack_data[st->start+3]));
		st->state = GOTO;
	}
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_select(struct script_state *st)
{
	int i;
	struct map_session_data *sd = script_rid2sd(st);

	if(sd == NULL) {	// LZɂ
		st->state = END;
		return 0;
	}

	if(sd->state.menu_or_input == 0) {
		char *buf;
		st->state = RERUNLINE;
		sd->state.menu_or_input = 1;

		if(st->end > st->start+3) {
			size_t len = 0;
			for(i=st->start+2; i<st->end; i++) {
				conv_str(st,& (st->stack->stack_data[i]));
				len += strlen(st->stack->stack_data[i].u.str) + 1;
			}
			buf = (char *)aCalloc(len + 1, sizeof(char));
			for(i=st->start+2; i<st->end; i++) {
				if(st->stack->stack_data[i].u.str[0]) {
					if(buf[0]) {
						strcat(buf,":");
					}
					strcat(buf,st->stack->stack_data[i].u.str);
				}
			}
			clif_scriptmenu(sd,st->oid,buf);
			aFree(buf);
		} else {
			buf = conv_str(st,& (st->stack->stack_data[st->start+2]));
			clif_scriptmenu(sd,st->oid,buf);
		}
	} else if(sd->npc_menu <= 0 || sd->npc_menu == 0xff) {	// invalid or cancel
		sd->state.menu_or_input = 0;
		st->state = END;
	} else {
		int count = 0;
		sd->state.menu_or_input = 0;
		// 󕶎 : ̃j[␳
		for(i=st->start+2; count < sd->npc_menu && i<st->end; i++) {
			char *p  = conv_str(st,& (st->stack->stack_data[i]));
			char *np = NULL;
			count++;
			if(*p == '\0') {
				sd->npc_menu++;
				continue;
			}
			while(count <= sd->npc_menu && (np = strchr(p,':')) != NULL) {
				count++;
				if(np == p) {
					sd->npc_menu++;
				}
				p = np + 1;
			}
		}
		if(count >= sd->npc_menu) {
			pc_setreg(sd,add_str("@menu"),sd->npc_menu);
			push_val(st->stack,C_INT,sd->npc_menu);
		} else {
			// sȒlȂ̂ŃLZ
			st->state = END;
		}
	}
	return 0;
}

/*==========================================
 * }bvwmob擾
 *------------------------------------------
 */
static int buildin_getmapmobs_sub(struct block_list *bl,va_list ap)
{
	int *count=va_arg(ap,int *);

	(*count)++;
	return 0;
}

int buildin_getmapmobs(struct script_state *st)
{
	char *str;
	int m,count=0;

	str=conv_str(st,& (st->stack->stack_data[st->start+2]));

	m = script_mapname2mapid(st,str);
	if(m < 0) {
		push_val(st->stack,C_INT,-1);
		return 0;
	}
	map_foreachinarea(buildin_getmapmobs_sub,m,0,0,map[m].xs,map[m].ys,BL_MOB,&count);
	push_val(st->stack,C_INT,count);
	return 0;
}

/*==========================================
 * GAwmob擾
 *------------------------------------------
 */
int buildin_getareamobs(struct script_state *st)
{
	char *str;
	int m,x0,y0,x1,y1,count=0;

	str=conv_str(st,& (st->stack->stack_data[st->start+2]));
	x0=conv_num(st,& (st->stack->stack_data[st->start+3]));
	y0=conv_num(st,& (st->stack->stack_data[st->start+4]));
	x1=conv_num(st,& (st->stack->stack_data[st->start+5]));
	y1=conv_num(st,& (st->stack->stack_data[st->start+6]));

	m = script_mapname2mapid(st,str);
	if(m < 0) {
		push_val(st->stack,C_INT,-1);
		return 0;
	}
	map_foreachinarea(buildin_getmapmobs_sub,m,x0,y0,x1,y1,BL_MOB,&count);
	push_val(st->stack,C_INT,count);
	return 0;
}

/*==========================================
 * Mhm̊֌W𒲂ׂ
 *------------------------------------------
 */
int buildin_getguildrelation(struct script_state *st)
{
	int gld1, gld2, result = 0;
	struct guild *g = NULL;

	gld1 = conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(gld1 <= 0 || (g = guild_search(gld1)) == NULL) {
		push_val(st->stack,C_INT,-1);
		return 0;
	}

	if(st->end > st->start+3) {
		gld2 = conv_num(st,& (st->stack->stack_data[st->start+3]));
	} else {
		struct map_session_data *sd = script_rid2sd(st);
		if(sd == NULL) {
			push_val(st->stack,C_INT,-1);
			return 0;
		}
		gld2 = sd->status.guild_id;
	}

	if(gld2 <= 0 || (g = guild_search(gld2)) == NULL) {
		push_val(st->stack,C_INT,-1);
		return 0;
	}

	if(gld1 == gld2)
		result += 1;
	if(guild_check_alliance(gld1, gld2, 0))
		result += 2;
	if(guild_check_alliance(gld1, gld2, 1))
		result += 4;
	if(guild_check_alliance(gld2, gld1, 1))
		result += 8;

	push_val(st->stack,C_INT,result);
	return 0;
}

/*==========================================
 * 
 *------------------------------------------
 */
int buildin_unequip(struct script_state *st)
{
	struct map_session_data *sd=script_rid2sd(st);
	int i,num = -1;

	nullpo_retr(0, sd);

	if( st->end>st->start+2 )
		num = conv_num(st,& (st->stack->stack_data[st->start+2]));

	if( num > 0 && num <= 11 ) {
		i = pc_checkequip(sd,equip_pos[num-1]);
		if(i >= 0)
			pc_unequipitem(sd,i,0);
	} else {
		for(i=0;i<11;i++) {
			if(sd->equip_index[i] >= 0 )
				pc_unequipitem(sd,sd->equip_index[i],0);
		}
	}
	return 0;
}

/*==========================================
 * ACegp
 *------------------------------------------
 */
int buildin_allowuseitem(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	int i = 0;

	nullpo_retr(0, sd);

	if(st->end > st->start+2) {
		struct script_data *data = &(st->stack->stack_data[st->start+2]);

		get_val(st,data);
		if( isstr(data) ) {
			const char *name = conv_str(st,data);
			struct item_data *item_data = itemdb_searchname(name);
			if(item_data)
				i = item_data->nameid;
			else
				i = -1;
		} else {
			i = conv_num(st,data);
		}
	}
	sd->npc_allowuseitem = i;
	return 0;
}

/*==========================================
 * wL̋}bv擾
 *------------------------------------------
 */
int buildin_getmapname(struct script_state *st)
{
	struct map_session_data *sd;
	char *char_name = conv_str(st,& (st->stack->stack_data[st->start+2]));

	if( strlen(char_name) >= 4 )
		sd = map_nick2sd(char_name);
	else
		sd = script_rid2sd(st);

	if(sd)
		push_str(st->stack,C_STR,(unsigned char *)aStrdup(sd->mapname));
	else
		push_str(st->stack,C_CONSTSTR,"");

	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_summon(struct script_state *st)
{
	int class_, id;
	char *str;
	const char *event = "";
	struct map_session_data *sd = script_rid2sd(st);
	struct mob_data *md;
	unsigned int tick = gettick();

	if(sd == NULL)
		return 0;

	str    = conv_str(st,& (st->stack->stack_data[st->start+2]));
	class_ = conv_num(st,& (st->stack->stack_data[st->start+3]));
	if( st->end>st->start+4 )
		event = conv_str(st,& (st->stack->stack_data[st->start+4]));

	id = mob_once_spawn(sd, sd->bl.m, 0, 0, str, class_, 1, event);
	if((md = map_id2md(id)) != NULL) {
		md->state.special_mob_ai = 1;
		md->master_id   = sd->bl.id;
		md->mode        = mob_db[md->class_].mode | 0x04;
		md->deletetimer = add_timer(tick+60000,mob_timer_delete,id,NULL);
		clif_misceffect2(&md->bl,344);
	}
	clif_skill_poseffect(&sd->bl,AM_CALLHOMUN,1,sd->bl.x,sd->bl.y,tick);

	return 0;
}

/*==========================================
 * wIuWFNg̈ʒu擾
 *------------------------------------------
 */
int buildin_getmapxy(struct script_state *st)
{
	struct map_session_data *sd = NULL;
	struct npc_data         *nd = NULL;
	struct pet_data         *pd = NULL;
	struct homun_data       *hd = NULL;
	int num;
	char *name;
	char prefix, postfix;
	int x,y,type;
	char mapname[24];
	void *v;

	if( st->stack->stack_data[st->start+2].type != C_NAME ||
	    st->stack->stack_data[st->start+3].type != C_NAME ||
	    st->stack->stack_data[st->start+4].type != C_NAME )
	{
		printf("buildin_getmapxy: param not name\n");
		push_val(st->stack,C_INT,-1);
		return 0;
	}

	type=conv_num(st,& (st->stack->stack_data[st->start+5]));

	memset(mapname, 0, sizeof(mapname));
	switch (type){
		case 0:		// Get Character Position
			if( st->end>st->start+6 )
				sd=map_nick2sd(conv_str(st,& (st->stack->stack_data[st->start+6])));
			else
				sd=script_rid2sd(st);

			if ( sd==NULL ) {	// wrong char name or char offline
				push_val(st->stack,C_INT,-1);
				return 0;
			}
			x=sd->bl.x;
			y=sd->bl.y;
			memcpy(mapname,sd->mapname,24);
			break;
		case 1:		// Get NPC Position
			if( st->end > st->start+6 )
				nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+6])));
			else
				nd = map_id2nd(st->oid);
			if ( nd==NULL ) {	// wrong npc name
				push_val(st->stack,C_INT,-1);
				return 0;
			}
			x=nd->bl.x;
			y=nd->bl.y;
			if(nd->bl.m == -1)
				mapname[0] = '-';	// }bvɔzuȂNPC
			else
				memcpy(mapname,map[nd->bl.m].name,24);
			break;
		case 2:		// Get Pet Position
			if( st->end>st->start+6 )
				sd=map_nick2sd(conv_str(st,& (st->stack->stack_data[st->start+6])));
			else
				sd=script_rid2sd(st);
			if ( sd==NULL ) {	// wrong char name or char offline
				push_val(st->stack,C_INT,-1);
				return 0;
			}
			pd=sd->pd;
			if ( pd==NULL ){	// pet data not found
				push_val(st->stack,C_INT,-1);
				return 0;
			}
			x=pd->bl.x;
			y=pd->bl.y;
			memcpy(mapname,map[pd->bl.m].name,24);
			break;
		case 3:		// Get Mob Position
			push_val(st->stack,C_INT,-1);
			return 0;
		case 4:		// Get Homun Position
			if( st->end>st->start+6 )
				sd=map_nick2sd(conv_str(st,& (st->stack->stack_data[st->start+6])));
			else
				sd=script_rid2sd(st);
			if ( sd==NULL ) {	// wrong char name or char offline
				push_val(st->stack,C_INT,-1);
				return 0;
			}
			hd=sd->hd;
			if ( hd==NULL ){	// homun data not found
				push_val(st->stack,C_INT,-1);
				return 0;
			}
			x=hd->bl.x;
			y=hd->bl.y;
			memcpy(mapname,map[hd->bl.m].name,24);
			break;
		default:	// Wrong type parameter
			push_val(st->stack,C_INT,-1);
			return 0;
	}

	// Set MapName$
	num     = st->stack->stack_data[st->start+2].u.num;
	name    = str_buf+str_data[num&0x00ffffff].str;
	prefix  = *name;
	postfix = name[strlen(name)-1];

	sd = (prefix != '$' && prefix != '\'')? script_rid2sd(st): NULL;
	if(postfix == '$') {
		v = (void*)mapname;
	} else {
		v = (void*)atoi(mapname);
	}
	set_reg(st,sd,num,name,v,st->stack->stack_data[st->start+2].ref);

	// Set MapX
	num     = st->stack->stack_data[st->start+3].u.num;
	name    = str_buf+str_data[num&0x00ffffff].str;
	prefix  = *name;
	postfix = name[strlen(name)-1];

	sd = (prefix != '$' && prefix != '\'')? script_rid2sd(st): NULL;
	if(postfix == '$') {
		char str[16];
		sprintf(str, "%d", x);
		v = (void*)str;
	} else {
		v = (void*)x;
	}
	set_reg(st,sd,num,name,v,st->stack->stack_data[st->start+3].ref);

	// Set MapY
	num     = st->stack->stack_data[st->start+4].u.num;
	name    = str_buf+str_data[num&0x00ffffff].str;
	prefix  = *name;
	postfix = name[strlen(name)-1];

	sd = (prefix != '$' && prefix != '\'')? script_rid2sd(st): NULL;
	if(postfix == '$') {
		char str[16];
		sprintf(str, "%d", y);
		v = (void*)str;
	} else {
		v = (void*)y;
	}
	set_reg(st,sd,num,name,v,st->stack->stack_data[st->start+4].ref);

	// Return Success value
	push_val(st->stack,C_INT,0);
	return 0;
}

/*==========================================
 * Checkcart [Valaris]
 *------------------------------------------
 */
int buildin_checkcart(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);

	push_val(st->stack,C_INT,(pc_iscarton(sd)) ? 1 : 0);

	return 0;
}

/*==========================================
 * checkfalcon [Valaris]
 *------------------------------------------
 */
int buildin_checkfalcon(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);

	push_val(st->stack,C_INT,(pc_isfalcon(sd)) ? 1 : 0);

	return 0;
}

/*==========================================
 * Checkriding [Valaris]
 *------------------------------------------
 */
int buildin_checkriding(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);

	push_val(st->stack,C_INT,(pc_isriding(sd)) ? 1 : 0);

	return 0;
}

/*==========================================
 * Ă邩ǂ
 *------------------------------------------
 */
int buildin_checksit(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);

	push_val(st->stack,C_INT,(pc_issit(sd)) ? 1 : 0);

	return 0;
}

/*==========================================
 * SĂ邩ǂ
 *------------------------------------------
 */
int buildin_checkdead(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);

	push_val(st->stack,C_INT,(sd && unit_isdead(&sd->bl)) ? 1 : 0);

	return 0;
}

/*==========================================
 * rĂ邩ǂ
 *------------------------------------------
 */
int buildin_checkcasting(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);

	push_val(st->stack,C_INT,(sd && sd->ud.skilltimer != -1) ? 1 : 0);

	return 0;
}

/*==========================================
 * pet attack skills [Valaris] //Rewritten by [Skotlex]
 *------------------------------------------
 */
int buildin_petskillattack(struct script_state *st)
{
	struct pet_data *pd;
	struct map_session_data *sd = script_rid2sd(st);

	if(sd == NULL || sd->pd == NULL)
		return 0;

	pd = sd->pd;
	if(pd->a_skill == NULL)
		pd->a_skill = (struct pet_skill_attack *)aCalloc(1, sizeof(struct pet_skill_attack));

	pd->a_skill->id = conv_num(st,& (st->stack->stack_data[st->start+2]));
	if(pd->a_skill->id == -1) {
		// remove pet skills
		aFree(pd->a_skill);
		pd->a_skill = NULL;
	} else {
		pd->a_skill->lv        = conv_num(st,& (st->stack->stack_data[st->start+3]));
		pd->a_skill->rate      = conv_num(st,& (st->stack->stack_data[st->start+4]));
		pd->a_skill->bonusrate = conv_num(st,& (st->stack->stack_data[st->start+5]));
		pd->a_skill->div_      = 0;
	}
	return 0;
}

/*==========================================
 * pet support skills [Skotlex]
 *------------------------------------------
 */
int buildin_petskillsupport(struct script_state *st)
{
	struct pet_data *pd;
	struct map_session_data *sd = script_rid2sd(st);

	if(sd == NULL || sd->pd == NULL)
		return 0;

	pd = sd->pd;
	if(pd->s_skill) {	// Clear previous skill
		if(pd->s_skill->timer != -1)
			delete_timer(pd->s_skill->timer, pet_skill_support_timer);
	} else {		// init memory
		pd->s_skill = (struct pet_skill_support *)aCalloc(1, sizeof(struct pet_skill_support));
	}

	pd->s_skill->id = conv_num(st,& (st->stack->stack_data[st->start+2]));
	if(pd->s_skill->id == -1) {
		// remove pet skills
		aFree(pd->s_skill);
		pd->s_skill = NULL;
	} else {
		pd->s_skill->lv    = conv_num(st,& (st->stack->stack_data[st->start+3]));
		pd->s_skill->delay = conv_num(st,& (st->stack->stack_data[st->start+4]));
		pd->s_skill->hp    = conv_num(st,& (st->stack->stack_data[st->start+5]));
		pd->s_skill->sp    = conv_num(st,& (st->stack->stack_data[st->start+6]));

		pd->s_skill->timer = add_timer(gettick()+pd->s_skill->delay*1000, pet_skill_support_timer ,sd->bl.id, NULL);
	}
	return 0;
}

/*==========================================
 * ybg̃[g^Cv̕ύX
 *------------------------------------------
 */
int buildin_changepettype(struct script_state *st)
{
	struct map_session_data *sd;
	int type = conv_num(st,& (st->stack->stack_data[st->start+2]));

	sd = script_rid2sd(st);

	if(sd && sd->status.pet_id) {
		if(sd->pd) {
			if(type > 2 || type < 0)
				type = 0;
			sd->pd->loottype = (short)type;
		}
	}
	return 0;
}

/*==========================================
 * p
 *------------------------------------------
 */
int buildin_making(struct script_state *st)
{
	struct map_session_data *sd;
	int makeid=conv_num(st,& (st->stack->stack_data[st->start+2]));

	sd=script_rid2sd(st);
	if(sd){
		clif_skill_produce_mix_list(sd,makeid);
		sd->making_base_success_per = conv_num(st,& (st->stack->stack_data[st->start+3]));
	}
	return 0;
}

/*==========================================
 * w}bvpvp,gvgtO𒲂ׂ
 *------------------------------------------
 */
int buildin_getpkflag(struct script_state *st)
{
	char *str;
	int m,count=0;

	str = conv_str(st,& (st->stack->stack_data[st->start+2]));

	m = script_mapname2mapid(st,str);
	if(m < 0) {
		push_val(st->stack,C_INT,-1);
		return 0;
	}

	if(map[m].flag.pvp)
		count += 1;
	if(map[m].flag.gvg)
		count += 2;

	push_val(st->stack,C_INT,count);
	return 0;
}

/*==========================================
 * Gain guild exp [Celest]
 * guildgetexp <exp>
 *------------------------------------------
 */
int buildin_guildgetexp(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	int exp;

	exp = conv_num(st,& (st->stack->stack_data[st->start+2]));

	// Exp͕̒lłǂ
	if(sd && sd->status.guild_id > 0)
		guild_getexp(sd, exp);

	return 0;
}

/*==========================================
 * flagname [Sapientia]
 *------------------------------------------
 */
int buildin_flagname(struct script_state *st)
{
	struct npc_data *nd = map_id2nd(st->oid);
	char *name;

	name = conv_str(st,& (st->stack->stack_data[st->start+2]));

	if(nd) {
		strncpy(nd->name, name, 24);
		nd->name[23] = '\0';
	}
	return 0;
}

/*==========================================
 * getnpcposition [Sapientia]
 *------------------------------------------
 */
int buildin_getnpcposition(struct script_state *st)
{
	struct npc_data *nd = map_id2nd(st->oid);
	char *str;

	str = conv_str(st,& (st->stack->stack_data[st->start+2]));

	if(nd) {
		strncpy(nd->position, str, 24);
		nd->position[23] = '\0';
	}
	return 0;
}

#ifndef NO_CSVDB_SCRIPT
/*==========================================
 * CSVANZX/֐Q
 *------------------------------------------
 */
static struct dbt* script_csvdb;

static int script_csvinit(void)
{
	script_csvdb = strdb_init(0);
	return 0;
}

static int script_csvfinal_sub(void *key, void *data, va_list ap)
{
	struct csvdb_data *csv = (struct csvdb_data *)data;

	csvdb_close( csv );
	return 0;
}

static int script_csvfinal(void)
{
	strdb_final( script_csvdb, script_csvfinal_sub );
	return 0;
}

// t@CÓȂ̂`FbN
static int script_csvfilename_check(const char *file, const char *func)
{
	const unsigned char *p = file;

	while(*p) {
		if( !isalnum(*p) && *p != '/' && *p != '_' ) {
			printf("%s: invalid file name %s\n", func, file);
			return 0;
		}
		p++;
	}
	return 1;
}

static struct csvdb_data* script_csvload(const char *file)
{
	struct csvdb_data *csv = (struct csvdb_data *)strdb_search(script_csvdb, file);

	if( csv == NULL ) {
		// t@CɕςȂ̂ĂȂmF
		int i;
		for(i = 0; file[i]; i++) {
			switch(file[i]) {
			case '.':
				if(file[i+1] != '.') break;
				// fall through
			case '<':
			case '>':
			case '|':
				printf("script_csvload: invalid file name %s\n", file);
				return NULL;
			}
		}
		csv = csvdb_open( file, 0 );
		if( csv ) {
			printf("script_csvload: %s load successfully\n", file);
			strdb_insert( script_csvdb, csv->file, csv );
		}
	}
	return csv;
}

int buildin_csvgetrows(struct script_state *st)
{
	char *file = conv_str(st,& (st->stack->stack_data[st->start+2]));
	struct csvdb_data *csv = script_csvload( file );

	if( csv == NULL ) {
		push_val(st->stack,C_INT,-1);
	} else {
		push_val(st->stack,C_INT,csvdb_get_rows(csv));
	}
	return 0;
}

int buildin_csvgetcols(struct script_state *st)
{
	char *file = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int  row   = conv_num(st,& (st->stack->stack_data[st->start+3]));
	struct csvdb_data *csv = script_csvload( file );

	if( csv == NULL ) {
		push_val(st->stack,C_INT,-1);
	} else {
		push_val(st->stack,C_INT,csvdb_get_columns(csv, row));
	}
	return 0;
}

// csvread <file>, <row>, <cow>
int buildin_csvread(struct script_state *st)
{
	char *file = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int  row   = conv_num(st,& (st->stack->stack_data[st->start+3]));
	int  col   = conv_num(st,& (st->stack->stack_data[st->start+4]));
	const char *v;
	struct csvdb_data *csv = script_csvload( file );

	v = (csv ? csvdb_get_str(csv, row, col) : NULL);

	if( v ) {
		push_str(st->stack,C_STR,(unsigned char *)aStrdup(v));
	} else {
		push_str(st->stack,C_CONSTSTR,"");
	}
	return 0;
}

// csvreadarray <file>, <row>, <array>
int buildin_csvreadarray(struct script_state *st)
{
	int i;
	struct map_session_data *sd = NULL;
	char *file = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int  row   = conv_num(st,& (st->stack->stack_data[st->start+3]));
	int  num;
	char *name;
	char prefix, postfix;
	struct csvdb_data* csv = script_csvload( file );

	if( st->stack->stack_data[st->start+4].type != C_NAME ) {
		printf("buildin_csvreadarray: param not name\n");
		return 0;
	}
	num     = st->stack->stack_data[st->start+4].u.num;
	name    = str_buf+str_data[num&0x00ffffff].str;
	prefix  = *name;
	postfix = name[strlen(name)-1];

	if( prefix != '$' && prefix != '@' && prefix != '\'' ) {
		printf("buildin_csvreadarray: illegal scope !\n");
		return 0;
	}
	if( prefix != '$' && prefix != '\'' ) {
		sd = script_rid2sd(st);
		if(sd == NULL)
			return 0;
	}

	// clear array
	for(i = getarraysize(st,num,postfix,st->stack->stack_data[st->start+4].ref) - (num >> 24) - 1;i >= 0;i--) {
		set_reg(st,sd,num+(i<<24),name,(postfix == '$')? "": 0,st->stack->stack_data[st->start+4].ref);
	}

	if( csv ) {
		const void *v;
		int max = csvdb_get_columns( csv, row );
		if( max + (num >> 24) > 128 ) {
			max = 128 - (num>>24);
		}
		for( i = 0; i < max; i++ ) {
			if( postfix == '$' )
				v = (const void *)csvdb_get_str(csv, row, i);
			else
				v = (const void *)csvdb_get_num(csv, row, i);
			set_reg(st,sd,num+(i<<24),name,v,st->stack->stack_data[st->start+4].ref);
		}
	}
	return 0;
}

// csvfind <file>, <col>, <val>
int buildin_csvfind(struct script_state *st)
{
	char *file = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int   col  = conv_num(st,& (st->stack->stack_data[st->start+3]));
	struct csvdb_data *csv = script_csvload( file );

	if( !csv ) {
		push_val(st->stack,C_INT,-1);
	} else if( isstr(&st->stack->stack_data[st->start+4])) {
		char *str = conv_str(st,& (st->stack->stack_data[st->start+4]));
		push_val(st->stack,C_INT,csvdb_find_str(csv, col, str));
	} else {
		int  val  = conv_num(st,& (st->stack->stack_data[st->start+4]));
		push_val(st->stack,C_INT,csvdb_find_num(csv, col, val));
	}
	return 0;
}

// csvwrite <file>, <row>, <col>, <val>
int buildin_csvwrite(struct script_state *st)
{
	char *file = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int  row   = conv_num(st,& (st->stack->stack_data[st->start+3]));
	int  col   = conv_num(st,& (st->stack->stack_data[st->start+4]));
	struct csvdb_data *csv;

	if( !script_csvfilename_check(file, "buildin_csvwrite") )
		return 0;
	csv = script_csvload( file );

	if( isstr(&st->stack->stack_data[st->start+5]) ) {
		char *str = conv_str(st,& (st->stack->stack_data[st->start+5]));
		csvdb_set_str(csv, row, col, str);
	} else {
		int   val = conv_num(st,& (st->stack->stack_data[st->start+5]));
		csvdb_set_num(csv, row, col, val);
	}
	return 0;
}

// csvwritearray <file>, <row>, <array>
int buildin_csvwritearray(struct script_state *st)
{
	char *file = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int  row   = conv_num(st,& (st->stack->stack_data[st->start+3]));
	int  num;
	char *name;
	char prefix, postfix;
	struct csvdb_data* csv;

	if( st->stack->stack_data[st->start+4].type != C_NAME ) {
		printf("buildin_csvwritearray: param not name\n");
		return 0;
	}
	num     = st->stack->stack_data[st->start+4].u.num;
	name    = str_buf+str_data[num&0x00ffffff].str;
	prefix  = *name;
	postfix = name[strlen(name)-1];

	if( prefix != '$' && prefix != '@' && prefix != '\'' ) {
		printf("buildin_csvwritearray: illegal scope !\n");
		return 0;
	}

	if( !script_csvfilename_check(file, "buildin_csvwritearray") )
		return 0;

	csv = script_csvload( file );
	if( csv ) {
		int max = getarraysize(st, num, postfix, st->stack->stack_data[st->start+4].ref) - (num >> 24);
		int i;

		csvdb_clear_row( csv, row );
		for( i = 0; i < max; i++ ) {
			if( postfix == '$' ) {
				csvdb_set_str(csv, row, i, (char*)get_val2(st, num+(i<<24),st->stack->stack_data[st->start+4].ref));
			} else {
				csvdb_set_num(csv, row, i, (int)get_val2(st, num+(i<<24),st->stack->stack_data[st->start+4].ref));
			}
		}
	}
	return 0;
}

// csvreload <file>
int buildin_csvreload(struct script_state *st)
{
	char *file = conv_str(st,& (st->stack->stack_data[st->start+2]));
	struct csvdb_data *csv = (struct csvdb_data *)strdb_erase( script_csvdb, file );

	if( csv ) {
		csvdb_close( csv );
	}
	return 0;
}

// csvinsert <file>, <row>
int buildin_csvinsert(struct script_state *st)
{
	char *file = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int   row  = conv_num(st,& (st->stack->stack_data[st->start+3]));
	struct csvdb_data *csv;

	if( !script_csvfilename_check(file, "buildin_csvinsert") )
		return 0;

	csv = script_csvload(file);
	if( csv ) {
		csvdb_insert_row(csv, row);
	}
	return 0;
}

// csvdelete <file>, <row>
int buildin_csvdelete(struct script_state *st)
{
	char *file = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int   row  = conv_num(st,& (st->stack->stack_data[st->start+3]));
	struct csvdb_data *csv;

	if( !script_csvfilename_check(file, "buildin_csvdelete") )
		return 0;

	csv = script_csvload( file );
	if( csv ) {
		csvdb_delete_row(csv, row);
	}
	return 0;
}

// csvsort <file>, <col>, <order>
int buildin_csvsort(struct script_state *st)
{
	char *file  = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int   col   = conv_num(st,& (st->stack->stack_data[st->start+3]));
	int   order = conv_num(st,& (st->stack->stack_data[st->start+4]));
	struct csvdb_data *csv;

	if( !script_csvfilename_check(file, "buildin_csvsort") )
		return 0;

	csv = script_csvload(file);
	if( csv ) {
		csvdb_sort(csv, col, order);
	}
	return 0;
}

// csvflush <file>
int buildin_csvflush(struct script_state *st)
{
	char *file = conv_str(st,& (st->stack->stack_data[st->start+2]));
	struct csvdb_data *csv;

	if( !script_csvfilename_check(file, "buildin_csvflush") )
		return 0;

	csv = (struct csvdb_data *)strdb_search( script_csvdb, file );
	if( csv ) {
		csvdb_flush( csv );
	}
	return 0;
}

#endif

/*==========================================
 *
 *------------------------------------------
 */
int buildin_sleep(struct script_state *st)
{
	int tick = conv_num(st,& (st->stack->stack_data[st->start+2]));
	struct map_session_data *sd = map_id2sd(st->rid);

	if(sd && sd->npc_id == st->oid) {
		sd->npc_id = 0;
	}
	st->rid = 0;
	if(tick <= 0) {
		// Ȃ
	} else if( !st->sleep.tick ) {
		// s
		st->state = RERUNLINE;
		st->sleep.tick = tick;
	} else {
		// s
		st->sleep.tick = 0;
	}
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_sleep2(struct script_state *st)
{
	int tick = conv_num(st,& (st->stack->stack_data[st->start+2]));

	if( tick <= 0 ) {
		// 0ms ̑ҋ@Ԃw肳ꂽ
		push_val(st->stack,C_INT,map_id2sd(st->rid) != NULL);
	} else if( !st->sleep.tick ) {
		// s
		st->state = RERUNLINE;
		st->sleep.tick = tick;
	} else {
		push_val(st->stack,C_INT,map_id2sd(st->rid) != NULL);
		st->sleep.tick = 0;
	}
	return 0;
}

/*==========================================
 * wNPC̑SĂsleepĊJ
 *------------------------------------------
 */
int buildin_awake(struct script_state *st)
{
	struct npc_data *nd;
	struct linkdb_node *node = (struct linkdb_node *)sleep_db;

	nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+2])));
	if(nd == NULL)
		return 0;

	while( node ) {
		if( (int)node->key == nd->bl.id) {
			struct script_state *tst    = (struct script_state *)node->data;
			struct map_session_data *sd = map_id2sd(tst->rid);

			if( tst->sleep.timer == -1 ) {
				node = node->next;
				continue;
			}
			if( sd && sd->status.char_id != tst->sleep.charid )
				tst->rid = 0;

			delete_timer(tst->sleep.timer, run_script_timer);
			node = script_erase_sleepdb(node);
			tst->sleep.timer = -1;
			run_script_main(tst);
		} else {
			node = node->next;
		}
	}
	return 0;
}

/*==========================================
 *
 *------------------------------------------
 */
int buildin_getvariableofnpc(struct script_state *st)
{
	if( st->stack->stack_data[st->start+2].type != C_NAME ) {
		// ϐȂ
		printf("buildin_getvariableofnpc: param not name\n");
		push_val(st->stack,C_INT,0);
	} else {
		int num = st->stack->stack_data[st->start+2].u.num;
		char *var_name = str_buf+str_data[num&0x00ffffff].str;
		char *npc_name = conv_str(st,& (st->stack->stack_data[st->start+3]));
		struct npc_data *nd = npc_name2id(npc_name);

		if( var_name[0] != '\'' || var_name[1] == '@' ) {
			// ' ϐȊO̓_
			printf("buildin_getvariableofnpc: invalid scope %s\n", var_name);
			push_val(st->stack,C_INT,0);
		} else if( nd == NULL || nd->subtype != SCRIPT || !nd->u.scr.script) {
			// NPC Ȃ or SCRIPTȊONPC
			printf("buildin_getvariableofnpc: can't find npc %s\n", npc_name);
			push_val(st->stack,C_INT,0);
		} else {
			push_val2(st->stack,C_NAME,num, &nd->u.scr.script->script_vars );
		}
	}
	return 0;
}

/*==========================================
 * ڃTCY̕ύX
 * type1=PC type2=PET type3=HOM type4=NPC
 *  size=-1  size=1
 *------------------------------------------
 */
int buildin_changeviewsize(struct script_state *st)
{
	struct block_list *bl = NULL;
	struct map_session_data *sd;
	int type=conv_num(st,& (st->stack->stack_data[st->start+2]));
	int size=conv_num(st,& (st->stack->stack_data[st->start+3]));

	sd = map_id2sd(st->rid);
	if(sd == NULL && type != 4)
		return 1;

	size = (size<0)? -1: (size>0)? 1: 0;

	switch (type) {
		case 1:
			bl = &sd->bl;
			break;
		case 2:
			if(sd->pd)
				bl = &sd->pd->bl;
			break;
		case 3:
			if(sd->hd)
				bl = &sd->hd->bl;
			break;
		case 4:
			bl = map_id2bl(st->oid);
			break;
		default:
			break;
	}
	if(bl)
		unit_changeviewsize(bl,size);

	return 0;
}

/*==========================================
 * ACegpAgACeID␻IDԂ
 * usediteminfo flag; 0=gpACeID 1=ID
 *------------------------------------------
 */
int buildin_usediteminfo(struct script_state *st)
{
	struct map_session_data *sd;
	int flag=conv_num(st,& (st->stack->stack_data[st->start+2]));
	int ret;

	if((sd=script_rid2sd(st))==NULL){
		push_val(st->stack,C_INT,-1);
		return -1;
	}
	if(flag==0)
		ret = sd->use_itemid;
	else
		ret = sd->use_nameditem;
	push_val(st->stack,C_INT,ret);
	return 0;
}

/*==========================================
 * ACeID擾
 *------------------------------------------
 */
int buildin_getitemid(struct script_state *st)
{
	struct item_data *item_data;
	char *item_name = conv_str(st,& (st->stack->stack_data[st->start+2]));

	item_data = itemdb_searchname(item_name);

	push_val(st->stack,C_INT,(item_data ? item_data->nameid: -1));
	return 0;
}

/*==========================================
 * Mh̃o[擾(ڑł͂Ȃo^)
 *------------------------------------------
 */
int buildin_getguildmembers(struct script_state *st)
{
	int gid,i,n=0;
	struct guild *g=NULL;

	gid = conv_num(st,& (st->stack->stack_data[st->start+2]));
	g = guild_search(gid);
	if(g == NULL){
		push_val(st->stack,C_INT,-1);
		return 0;
	}
	for(i=0;i<g->max_member;i++) {
		if(g->member[i].account_id!=0)
			n++;
	}

	push_val(st->stack,C_INT,n);
	return 0;
}

/*==========================================
 * NPCUXLGtFNg
 *------------------------------------------
 */
int buildin_npcskillattack(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	struct block_list *bl = map_id2bl(st->oid);
	int id=conv_num(st,& (st->stack->stack_data[st->start+2]));
	int lv=conv_num(st,& (st->stack->stack_data[st->start+3]));
	int damage=conv_num(st,& (st->stack->stack_data[st->start+4]));

	nullpo_retr(0, sd);

	if(bl == NULL || bl->type != BL_NPC)
		bl = &sd->bl;

	clif_skill_damage(bl,&sd->bl,gettick(),status_get_amotion(bl),status_get_dmotion(&sd->bl),
				damage, skill_get_num(id,lv), id, lv, skill_get_hit(id));
	return 0;
}

/*==========================================
 * NPCx/񕜃XLGtFNg
 *------------------------------------------
 */
int buildin_npcskillsupport(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	struct block_list *bl = map_id2bl(st->oid);
	int skillid=conv_num(st,& (st->stack->stack_data[st->start+2]));
	int heal=conv_num(st,& (st->stack->stack_data[st->start+3]));

	nullpo_retr(0, sd);

	if(bl == NULL || bl->type != BL_NPC)
		bl = &sd->bl;

	clif_skill_nodamage(bl,&sd->bl,skillid,heal,1);

	return 0;
}

/*==========================================
 * NPCꏊwXLGtFNg
 *------------------------------------------
 */
int buildin_npcskillpos(struct script_state *st)
{
	struct block_list *bl = map_id2bl(st->oid);
	int skillid=conv_num(st,& (st->stack->stack_data[st->start+2]));
	int skilllv=conv_num(st,& (st->stack->stack_data[st->start+3]));
	int x=conv_num(st,& (st->stack->stack_data[st->start+4]));
	int y=conv_num(st,& (st->stack->stack_data[st->start+5]));

	if(bl == NULL || bl->type != BL_NPC) {
		struct map_session_data *sd = script_rid2sd(st);
		if(sd == NULL)
			return 0;
		bl = &sd->bl;
	}
	clif_skill_poseffect(bl,skillid,skilllv,x,y,gettick());

	return 0;
}

/*==========================================
 * ĂяoNPC擾
 *------------------------------------------
 */
int buildin_strnpcinfo(struct script_state *st)
{
	struct npc_data *nd = map_id2nd(st->oid);
	int type;
	char *p, *name = NULL;

	type = conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(nd == NULL) {
		push_str(st->stack,C_CONSTSTR,"");
		return 0;
	}
	switch(type) {
		case 0:
			name = (char *)aStrdup(nd->name);
			break;
		case 1:
			name = (char *)aStrdup(nd->name);
			if((p = strchr(name,'#')) != NULL)
				*p = 0;
			break;
		case 2:
			if((p = strchr(nd->name,'#')) != NULL)
				name = (char *)aStrdup(p+1);
			break;
		case 3:
			name = (char *)aStrdup(nd->exname);
			break;
		case 4:
			name = (char *)aStrdup(nd->exname);
			if((p = strchr(name,'#')) != NULL)
				*p = 0;
			break;
		case 5:
			if((p = strchr(nd->exname,'#')) != NULL)
				name = (char *)aStrdup(p+1);
			break;
		case 6:
			name = (char *)aStrdup(nd->position);
			break;
	}
	if(name)
		push_str(st->stack,C_STR,name);
	else
		push_str(st->stack,C_CONSTSTR,"");
	return 0;
}

/*==========================================
 * wPTPT[_[̃LԂ
 *------------------------------------------
 */
int buildin_getpartyleader(struct script_state *st)
{
	int i,pt_id;
	struct party *p=NULL;

	pt_id=conv_num(st,& (st->stack->stack_data[st->start+2]));

	p=party_search(pt_id);
	if(p==NULL) {
		push_str(st->stack,C_CONSTSTR,"");
		return 0;
	}
	for(i=0;i<MAX_PARTY;i++){
		if(p->member[i].leader) {
			push_str(st->stack,C_STR,(unsigned char *)aStrdup(p->member[i].name));
			return 0;
		}
	}
	push_str(st->stack,C_CONSTSTR,"");	// [_[Ȃ
	return 0;
}

/*==========================================
 * 񒷂Ԃ
 *------------------------------------------
 */
int buildin_getstrlen(struct script_state *st)
{
	char *str = conv_str(st,& (st->stack->stack_data[st->start+2]));

	push_val(st->stack,C_INT,(int)strlen(str));

	return 0;
}

/*==========================================
 * ̔Cӂ̕擾
 *------------------------------------------
 */
int buildin_substr(struct script_state *st)
{
	char *str;
	int len, offset, count;

	str    = conv_str(st,& (st->stack->stack_data[st->start+2]));
	offset = conv_num(st,& (st->stack->stack_data[st->start+3]));

	len = (int)strlen(str);
	if(offset < 0)		// JnʒuȂ̂ŖʒuvZ
		offset += len;
	if(offset < 0 || offset >= len) {
		push_str(st->stack,C_CONSTSTR,"");
		return 0;
	}

	if(st->end > st->start+4) {
		count = conv_num(st,& (st->stack->stack_data[st->start+4]));
		if(count > 0 && offset + count > len)
			count = len - offset;		// 񒷂𒴂̂ŕ␳
		else if(count < 0)
			count = len - offset + count;	// Ȃ̂Ŗ
	} else {
		count = len - offset;			// ȗ͍Ō܂
	}

	if(count > 0) {
		char *buf = (char *)aCalloc(count+1, sizeof(char));
		memcpy(buf, str+offset, count);
		push_str(st->stack,C_STR,buf);
	} else {
		push_str(st->stack,C_CONSTSTR,"");
	}
	return 0;
}

/*==========================================
 * PCNPCԂ̋Ԃ
 *------------------------------------------
 */
int buildin_distance(struct script_state *st)
{
	struct map_session_data *sd;
	struct npc_data *nd;

	sd = map_id2sd(conv_num(st,& (st->stack->stack_data[st->start+2])));
	if(st->end > st->start+3)
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+3])));
	else
		nd = map_id2nd(st->oid);

	if(sd && nd && sd->bl.m == nd->bl.m)
		push_val(st->stack,C_INT,unit_distance2(&sd->bl,&nd->bl));
	else
		push_val(st->stack,C_INT,-1);
	return 0;
}

/*==========================================
 * zNX폜
 *------------------------------------------
 */
int buildin_homundel(struct script_state *st)
{
	homun_delete_data( script_rid2sd(st) );
	return 0;
}

/*==========================================
 * zNXl[
 *------------------------------------------
 */
int buildin_homunrename(struct script_state *st)
{
	char *homname;
	int flag=0;
	struct map_session_data *sd = script_rid2sd(st);

	homname	= conv_str(st,& (st->stack->stack_data[st->start+2]));
	if( st->end > st->start+3 ){
		flag = conv_num(st,& (st->stack->stack_data[st->start+3]));
	}

	if(sd && sd->hd) {
		if(flag == 0 && sd->hd->status.rename_flag == 0) {
			// ύXȌԂɖ߂
			flag = 1;
		}
		sd->hd->status.rename_flag = 0;
		homun_change_name(sd,homname);

		// ύX\ɂ
		if(flag == 1)
			sd->hd->status.rename_flag = 0;
	}
	return 0;
}

/*==========================================
 * zNXi
 *------------------------------------------
 */
int buildin_homunevolution(struct script_state *st)
{
	homun_change_class(script_rid2sd(st), -1);
	return 0;
}

/*==========================================
 * Xe[^XČvZ
 *------------------------------------------
 */
int buildin_recalcstatus(struct script_state *st)
{
	struct map_session_data *sd;

	if( st->end>st->start+2 )
		sd = map_id2sd(conv_num(st,& (st->stack->stack_data[st->start+2])));
	else
		sd = script_rid2sd(st);

	if(sd)
		status_calc_pc(sd,0);
	return 0;
}

/*==========================================
 * SQLNGs
 *------------------------------------------
 */
int buildin_sqlquery(struct script_state *st)
{
#ifndef TXT_ONLY
	int count = -1, rc;
	MYSQL_RES* sql_res;
	char *query = conv_str(st,& (st->stack->stack_data[st->start+2]));

	// SQLNGpsA̓NGȂG[
	if(!sql_script_enable || strlen(query) >= sizeof(tmp_sql)) {
		push_val(st->stack,C_INT,-1);
		return 0;
	}
	rc = sqldbs_query(&mysql_handle_script, query);
	if(rc) {
		push_val(st->stack,C_INT,-1);
		return 0;
	}
	sql_res = sqldbs_store_result(&mysql_handle_script);

	// SELECTȊO͂Ŋ
	if(sql_res == NULL) {
		count = (int)sqldbs_affected_rows(&mysql_handle_script);
		push_val(st->stack,C_INT,count);
		return 0;
	}

	do {
		int num, len, elem, max;
		char *name, *var, *p;
		char prefix, postfix;
		struct map_session_data *sd = NULL;
		MYSQL_ROW sql_row = NULL;

		if(st->end <= st->start+3 || st->stack->stack_data[st->start+3].type != C_NAME) {
			printf("buildin_sqlquery: param not name\n");
			break;
		}
		num     = st->stack->stack_data[st->start+3].u.num;
		name    = str_buf+str_data[num&0x00ffffff].str;
		prefix  = *name;
		postfix = name[strlen(name)-1];

		if( prefix != '$' && prefix != '@' && prefix != '\'' ) {
			printf("buildin_sqlquery: illegal scope !\n");
			break;
		}
		if( prefix != '$' && prefix != '\'' ) {
			sd = script_rid2sd(st);
			if(sd == NULL)
				break;
		}

		var = (char *)aCalloc(strlen(name)+6,sizeof(char));	// [xxx] + \0 = 6
		strcpy(var,name);

		if((p = strrchr(var,'[')) != NULL) {
			elem = atoi(p+1);	// z̓񎟌ڂ̗vf擾
			len  = (int)(p - var);
		} else {
			elem = 0;
			len = (int)strlen(var);
			if(postfix == '$')
				len--;
		}

		max = sqldbs_num_fields(sql_res);
		if(max + (num >> 24) > 128) {
			max = 128 - (num>>24);
		}

		for(count = 0; elem < 128 && (sql_row = sqldbs_fetch(sql_res)); count++) {
			int i,tmp_num;

			if(count > 0) {	// ʃZbgŝŕϐ
				sprintf(var+len, "[%d]%s", elem, (postfix == '$')? "$": "");
				tmp_num = add_str(var) + (num&0xff000000);
			} else {
				tmp_num = num;
			}
			for(i=0; i<max; i++) {
				void *v = (postfix == '$')? sql_row[i]: (void*)atoi(sql_row[i]);
				set_reg(st,sd,tmp_num+(i<<24),var,v,st->stack->stack_data[st->start+3].ref);
			}
			elem++;
		}
		aFree(var);
	} while(0);

	sqldbs_free_result(sql_res);
	push_val(st->stack,C_INT,count);
#else
	// TXT͉Ȃ
	push_val(st->stack,C_INT,-1);
#endif
	return 0;
}

/*==========================================
 * SQLp̕GXP[v
 *------------------------------------------
 */
int buildin_strescape(struct script_state *st)
{
	char *str = conv_str(st,& (st->stack->stack_data[st->start+2]));

#ifndef TXT_ONLY
	char *buf = (char *)aCalloc(strlen(str)*2+1,sizeof(char));

	strecpy(buf,str);
	push_str(st->stack,C_STR,buf);
#else
	// TXT͉Ȃ
	push_str(st->stack,C_STR,(unsigned char *)aStrdup(str));
#endif
	return 0;
}

/*==========================================
 * ACehbv
 *------------------------------------------
 */
int buildin_dropitem(struct script_state *st)
{
	int nameid=0, flag=0, amount;
	int x,y,m,tick;
	unsigned int limit = 0;
	char *str;
	struct script_data *data;

	str = conv_str(st,& (st->stack->stack_data[st->start+2]));
	x   = conv_num(st,& (st->stack->stack_data[st->start+3]));
	y   = conv_num(st,& (st->stack->stack_data[st->start+4]));

	data = &(st->stack->stack_data[st->start+5]);
	get_val(st,data);
	if(isstr(data)) {
		const char *name = conv_str(st,data);
		struct item_data *item_data = itemdb_searchname(name);
		if(item_data)
			nameid = item_data->nameid;
	} else {
		nameid = conv_num(st,data);
	}

	amount = conv_num(st,& (st->stack->stack_data[st->start+6]));
	tick   = conv_num(st,& (st->stack->stack_data[st->start+7]));
	if(st->end > st->start+8)
		limit = (unsigned int)conv_num(st,& (st->stack->stack_data[st->start+8]));

	m = script_mapname2mapid(st,str);
	if(m < 0)
		return 0;

	if(nameid < 0) {
		nameid = itemdb_searchrandomid(-nameid);
		flag   = 1;
	}

	if(nameid > 0) {
		struct item item_tmp;
		memset(&item_tmp,0,sizeof(item_tmp));
		item_tmp.nameid = nameid;
		if(!flag)
			item_tmp.identify = 1;
		else
			item_tmp.identify = !itemdb_isequip3(nameid);

		item_tmp.limit = (limit > 0)? (unsigned int)time(NULL) + limit: 0;
		battle_config.flooritem_lifetime += tick;
		map_addflooritem(&item_tmp,amount,m,x,y,0,0,0,0);
		battle_config.flooritem_lifetime -= tick;
	}

	return 0;
}

/*==========================================
 * ol擾
 *------------------------------------------
 */
int buildin_getexp(struct script_state *st)
{
	int base,job;
	struct map_session_data *sd = script_rid2sd(st);

	nullpo_retr(0, sd);

	base = conv_num(st,& (st->stack->stack_data[st->start+2]));
	job  = conv_num(st,& (st->stack->stack_data[st->start+3]));

	if(base < 0 || job < 0)
		return 0;
	pc_gainexp(sd,NULL,base,job);

	return 0;
}

/*==========================================
 * ACef[^擾
 *------------------------------------------
 */
int buildin_getiteminfo(struct script_state *st)
{
	int num, val = -1;
	char *str = NULL;
	struct script_data *data;
	struct item_data *item = NULL;

	data = &(st->stack->stack_data[st->start+2]);
	get_val(st,data);
	if( isstr(data) ) {
		const char *name = conv_str(st,data);
		item = itemdb_searchname(name);
	} else {
		int nameid = conv_num(st,data);
		item = itemdb_exists(nameid);
	}
	num = conv_num(st,& (st->stack->stack_data[st->start+3]));

	if(item) {
		switch(num) {
			case 0:  val = item->nameid;           break;
			case 1:  str = item->name;             break;
			case 2:  str = item->jname;            break;
			case 3:  val = item->type;             break;
			case 4:  val = item->value_buy;        break;
			case 5:  val = item->value_sell;       break;
			case 6:  val = item->weight;           break;
			case 7:  val = item->atk;              break;
			case 8:  val = item->def;              break;
			case 9:  val = item->range;            break;
			case 10: val = item->slot;             break;
			case 11: val = item->class_;           break;
			case 12: val = item->sex;              break;
			case 13: val = item->look;             break;
			case 14: val = item->wlv;              break;
			case 15: val = item->elv;              break;
			case 16: val = item->view_id;          break;
			case 17: val = item->refine;           break;
			case 18: val = item->upper;            break;
			case 19: val = item->zone;             break;
			case 20: val = item->flag.dropable;    break;
			case 21: val = item->flag.storageable; break;
			case 22: val = item->flag.cartable;    break;
			case 23: val = item->delay;            break;
			case 24: val = item->flag.available;   break;
			case 25: val = item->flag.value_notdc; break;
			case 26: val = item->flag.value_notoc; break;
			case 27: val = item->group;            break;
			case 28: val = item->arrow_type;       break;
			case 29: str = item->cardillustname;   break;
		}
	}
	if(str)
		push_str(st->stack,C_STR,(unsigned char *)aStrdup(str));
	else
		push_val(st->stack,C_INT,val);
	return 0;
}

/*==========================================
 * p[eB[̐ڑl擾
 *------------------------------------------
 */
int buildin_getonlinepartymember(struct script_state *st)
{
	int count = 0;
	struct party *pt = NULL;

	if(st->end>st->start+2) {
		pt = party_search(conv_num(st,&(st->stack->stack_data[st->start+2])));
	} else {
		struct map_session_data *sd = script_rid2sd(st);
		if(sd)
			pt = party_search(sd->status.party_id);
	}
	if(pt != NULL) {
		int i;
		for(i=0; i<MAX_PARTY; i++) {
			if(pt->member[i].online && pt->member[i].sd != NULL)
				count++;
		}
	}
	push_val(st->stack,C_INT,count);

	return 0;
}

/*==========================================
 * Mh̐ڑl擾
 *------------------------------------------
 */
int buildin_getonlineguildmember(struct script_state *st)
{
	struct guild *g = NULL;

	if(st->end>st->start+2) {
		g = guild_search(conv_num(st,&(st->stack->stack_data[st->start+2])));
	} else {
		struct map_session_data *sd = script_rid2sd(st);
		if(sd)
			g = guild_search(sd->status.guild_id);
	}
	push_val(st->stack,C_INT,((g != NULL)? g->connect_member: 0));

	return 0;
}

/*==========================================
 * b쐬
 *------------------------------------------
 */
int buildin_makemerc(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	int merc_id;
	unsigned int limit;

	nullpo_retr(0, sd);

	merc_id = conv_num(st,& (st->stack->stack_data[st->start+2]));
	limit   = (unsigned int)conv_num(st,& (st->stack->stack_data[st->start+3]));

	merc_callmerc(sd,merc_id,limit);

	return 0;
}

/*==========================================
 * ǏEBhE̕\
 *------------------------------------------
 */
int buildin_openbook(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	int nameid, page = 1;

	nullpo_retr(0, sd);

	nameid = conv_num(st,& (st->stack->stack_data[st->start+2]));
	if(st->end>st->start+3)
		page = conv_num(st,& (st->stack->stack_data[st->start+3]));

	clif_openbook(sd,nameid,page);

	return 0;
}

/*==========================================
 * mbNobN
 *------------------------------------------
 */
int buildin_pushpc(struct script_state *st)
{
	struct map_session_data *sd = script_rid2sd(st);
	struct block_list *bl = map_id2bl(st->oid);
	int dir, count;
	
	nullpo_retr(0, sd);

	if(bl == NULL || bl->type != BL_NPC)
		bl = &sd->bl;

	dir = conv_num(st,& (st->stack->stack_data[st->start+2]));
	count = conv_num(st,& (st->stack->stack_data[st->start+3]));

	skill_blown(bl,&sd->bl,count|(dir<<20)|SAB_NODAMAGE|SAB_NOPATHSTOP);

	return 0;
}

/*==========================================
 * wZ̃^CvύX
 *------------------------------------------
 */
int buildin_setcell(struct script_state *st)
{
	int m,x,y,type;
	char *str;

	str  = conv_str(st,& (st->stack->stack_data[st->start+2]));
	x    = conv_num(st,& (st->stack->stack_data[st->start+3]));
	y    = conv_num(st,& (st->stack->stack_data[st->start+4]));
	type = conv_num(st,& (st->stack->stack_data[st->start+5]));

	m = script_mapname2mapid(st,str);
	if(m < 0) {
		return 0;
	}
	map_setcell(m,x,y,type);
	clif_changemapcell(m,x,y,type,1);

	return 0;
}

/*==========================================
 * w͈̓Z̃^CvύX
 *------------------------------------------
 */
int buildin_areasetcell(struct script_state *st)
{
	int m,x,y,x0,y0,x1,y1,type;
	char *str;

	str  = conv_str(st,& (st->stack->stack_data[st->start+2]));
	x0   = conv_num(st,& (st->stack->stack_data[st->start+3]));
	y0   = conv_num(st,& (st->stack->stack_data[st->start+4]));
	x1   = conv_num(st,& (st->stack->stack_data[st->start+5]));
	y1   = conv_num(st,& (st->stack->stack_data[st->start+6]));
	type = conv_num(st,& (st->stack->stack_data[st->start+7]));

	m = script_mapname2mapid(st,str);
	if(m < 0) {
		return 0;
	}

	for(y = y0; y <= y1; y++) {
		for(x = x0; x <= x1; x++) {
			map_setcell(m,x,y,type);
			clif_changemapcell(m,x,y,type,1);
		}
	}

	return 0;
}

/*==========================================
 * K[fBA
 *------------------------------------------
 */
int buildin_callguardian(struct script_state *st)
{
	int mob_id,amount,m,x,y,index=0,id;
	char *str,*mapname,*mobname;
	const char *event = "";
	struct mob_data *md;

	mapname = conv_str(st,& (st->stack->stack_data[st->start+2]));
	x       = conv_num(st,& (st->stack->stack_data[st->start+3]));
	y       = conv_num(st,& (st->stack->stack_data[st->start+4]));
	str     = conv_str(st,& (st->stack->stack_data[st->start+5]));
	mobname = conv_str(st,& (st->stack->stack_data[st->start+6]));

	if((mob_id = atoi(mobname)) == 0)
		mob_id = mobdb_searchname(mobname);
	if(mob_id >= 0 && !mobdb_checkid(mob_id))
		return 0;

	amount = conv_num(st,& (st->stack->stack_data[st->start+7]));
	if(st->end > st->start+8) {
		event = conv_str(st,& (st->stack->stack_data[st->start+8]));
	}
	if(st->end > st->start+9) {	// K[fBAindex
		index = conv_num(st,& (st->stack->stack_data[st->start+9]));
	}

	if((m = script_mapname2mapid(st,mapname)) < 0)
		return 0;

	id = mob_once_spawn(map_id2sd(st->rid),m,x,y,str,mob_id,amount,event);

	if((md = map_id2md(id)) != NULL)
	{
		struct guild_castle *gc = guild_mapname2gc(mapname);

		md->guardup_lv = 0;
		if(gc) {
			struct guild *g;

			// ԏɏ
			if(index > 0 && index <= MAX_GUILDGUARDIAN) {
				gc->guardian[index-1].visible = 1;
				gc->guardian[index-1].id = id;
				guild_castledatasave(gc->castle_id,index+9,1);
			}
			if((g = guild_search(gc->guild_id)) != NULL) {
				md->guild_id   = g->guild_id;
				md->guardup_lv = guild_checkskill(g,GD_GUARDUP);
			}
		}
	}

	return 0;
}

/*==========================================
 * K[fBȀ擾
 *------------------------------------------
 */
int buildin_getguardianinfo(struct script_state *st)
{
	struct guild_castle *gc;
	int val = 0;
	char *str = NULL;
	char *mapname = conv_str(st,& (st->stack->stack_data[st->start+2]));
	int index     = conv_num(st,& (st->stack->stack_data[st->start+3]));
	int type      = conv_num(st,& (st->stack->stack_data[st->start+4]));

	if(strcmp(mapname,"this") == 0) {
		int m = script_mapname2mapid(st,mapname);
		if(m < 0) {
			if(type == 2)
				push_str(st->stack,C_CONSTSTR,"");
			else
				push_val(st->stack,C_INT,0);
			return 0;
		}
		mapname = map[m].name;
	}

	if((gc = guild_mapname2gc(mapname)) != NULL) {
		struct mob_data *md = map_id2md(gc->guardian[index-1].id);
		if(md != NULL) {
			switch(type) {
			case 0:		// ID
				val = md->bl.id;
				break;
			case 1:		// Class
				val = md->class_;
				break;
			case 2:		// Name
				str = (char*)aStrdup(md->name);
				break;
			case 3:		// HP
				val = status_get_hp(&md->bl);
				break;
			case 4:		// MaxHP
				val = status_get_max_hp(&md->bl);
				break;
			default:
				val = 0;
				break;
			}
		}
	}

	if(str)
		push_str(st->stack,C_STR,str);
	else
		push_val(st->stack,C_INT,val);

	return 0;
}

/*==========================================
 * ID烂X^[
 *------------------------------------------
 */
int buildin_getmobname(struct script_state *st)
{
	int mob_class = conv_num(st,& (st->stack->stack_data[st->start+2]));

	if(!mobdb_checkid(mob_class))
		push_str(st->stack,C_CONSTSTR,"");
	else
		push_str(st->stack,C_STR,(unsigned char *)aStrdup(mob_db[mob_class].jname));

	return 0;
}

/*==========================================
 * ACeԂ
 *------------------------------------------
 */
int buildin_checkactiveitem(struct script_state *st)
{
	int id,i,ret = 0;
	struct map_session_data *sd = script_rid2sd(st);

	if(sd) {
		if(st->end>st->start+2)
			id = conv_num(st,& (st->stack->stack_data[st->start+2]));
		else
			id = current_equip_name_id;

		for(i=0; i < sd->activeitem.count; i++) {
			if(sd->activeitem.id[i] == id) {
				if(sd->activeitem_timer[i] != -1 && sd->activeitem_timertick[i] >= gettick()) {
					ret = 1;
					break;
				}
			}
		}
	}
	push_val(st->stack,C_INT,ret);

	return 0;
}

//  ++ NPCIX by Blaze
/*==========================================
 * NPCIXݒ
 *------------------------------------------
 */
int buildin_setvending(struct script_state *st)
{
	int idx, nameid, amount, value;
	int identify = 1, refine = 0, attribute = 0;
	int card[4] = { 0,0,0,0 };
	unsigned int limit=0;
	struct npc_data *nd;
	struct script_data *data;
	struct item_data *item_data;
	struct npc_vending vend_tmp;

	idx = conv_num(st,& (st->stack->stack_data[st->start+2]));
	data = &(st->stack->stack_data[st->start+3]);
	get_val(st,data);
	if( isstr(data) ) {
		const char *name = conv_str(st,data);
		struct item_data *item_data = itemdb_searchname(name);
		if(item_data)
			nameid = item_data->nameid;
	} else {
		nameid = conv_num(st,data);
	}
	if(nameid < 0)		// _
		nameid = itemdb_searchrandomid(-nameid);
	if(!itemdb_exists(nameid))
		return 0;

	amount = conv_num(st,& (st->stack->stack_data[st->start+4]));
	value = conv_num(st,& (st->stack->stack_data[st->start+5]));
	if(st->end > st->start+6)
		identify = conv_num(st,& (st->stack->stack_data[st->start+6]));
	if(st->end > st->start+7)
		refine = conv_num(st,& (st->stack->stack_data[st->start+7]));
	if(st->end > st->start+8)
		attribute = conv_num(st,& (st->stack->stack_data[st->start+8]));
	if(st->end > st->start+9)
		card[0] = conv_num(st,& (st->stack->stack_data[st->start+9]));
	if(st->end > st->start+10)
		card[1] = conv_num(st,& (st->stack->stack_data[st->start+10]));
	if(st->end > st->start+11)
		card[2] = conv_num(st,& (st->stack->stack_data[st->start+11]));
	if(st->end > st->start+12)
		card[3] = conv_num(st,& (st->stack->stack_data[st->start+12]));
	if(st->end > st->start+13)
		limit = (unsigned int)conv_num(st,& (st->stack->stack_data[st->start+13]));
	if(st->end > st->start+14)
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+14])));
	else
		nd = map_id2nd(st->oid);

	if(nd == NULL)
		return 0;

	memset(&vend_tmp,0,sizeof(vend_tmp));

	vend_tmp.item.nameid = nameid;
	vend_tmp.item.amount = amount;
	vend_tmp.value = value;

	item_data = itemdb_search(nameid);

	// 킩hH
	if(item_data->type == 4 || item_data->type == 5) {
		if(refine > MAX_REFINE)
			refine = MAX_REFINE;
	}
	// ybg̗H
	else if(item_data->type == 7) {
		identify = 1;
		refine = 0;
	}
	// ȊO
	else {
		identify = 1;
		refine = 0;
		attribute = 0;
	}

	vend_tmp.item.identify = identify? identify : !itemdb_isequip3(nameid);
	vend_tmp.item.refine    = refine;
	vend_tmp.item.attribute = attribute;
	vend_tmp.item.card[0]   = card[0];
	vend_tmp.item.card[1]   = card[1];
	vend_tmp.item.card[2]   = card[2];
	vend_tmp.item.card[3]   = card[3];
	vend_tmp.item.limit     = limit;

	vending_setnpcvending(nd,&vend_tmp,idx);

	return 0;
}

/*==========================================
 * NPCIX폜
 *------------------------------------------
 */
int buildin_delvending(struct script_state *st)
{
	int idx = 0;
	struct npc_data *nd;

	if(st->end > st->start+2)
		idx = conv_num(st,& (st->stack->stack_data[st->start+2]));
	if(st->end > st->start+3)
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+3])));
	else
		nd = map_id2nd(st->oid);

	if(nd == NULL)
		return 0;

	vending_delnpcvending(nd,idx);

	return 0;
}

/*==========================================
 * NPCIX擾
 *------------------------------------------
 */
int buildin_getvending(struct script_state *st)
{
	int i,count=0,open=0,num;
	struct npc_data *nd;

	if(st->end > st->start+3)
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+3])));
	else
		nd = map_id2nd(st->oid);

	if(nd == NULL)
		return 0;

	for(i = 0; i < MAX_VENDING; i++) {
		set_reg(st,NULL,
			num = add_str("\'@vending_itemid")+(i<<24),
			str_buf+str_data[num&0x00ffffff].str,
			(void*)nd->vending[i].item.nameid,
			st->stack->stack_data[st->start+2].ref
		);
		set_reg(st,NULL,
			num = add_str("\'@vending_amount")+(i<<24),
			str_buf+str_data[num&0x00ffffff].str,
			(void*)nd->vending[i].item.amount,
			st->stack->stack_data[st->start+2].ref
		);
		set_reg(st,NULL,
			num = add_str("\'@vending_value")+(i<<24),
			str_buf+str_data[num&0x00ffffff].str,
			(void*)nd->vending[i].value,
			st->stack->stack_data[st->start+2].ref
		);
		set_reg(st,NULL,
			num = add_str("\'@vending_identify")+(i<<24),
			str_buf+str_data[num&0x00ffffff].str,
			(void*)nd->vending[i].item.identify,
			st->stack->stack_data[st->start+2].ref
		);
		set_reg(st,NULL,
			num = add_str("\'@vending_refine")+(i<<24),
			str_buf+str_data[num&0x00ffffff].str,
			(void*)nd->vending[i].item.refine,
			st->stack->stack_data[st->start+2].ref
		);
		set_reg(st,NULL,
			num = add_str("\'@vending_attribute")+(i<<24),
			str_buf+str_data[num&0x00ffffff].str,
			(void*)nd->vending[i].item.attribute,
			st->stack->stack_data[st->start+2].ref
		);
		set_reg(st,NULL,
			num = add_str("\'@vending_card1")+(i<<24),
			str_buf+str_data[num&0x00ffffff].str,
			(void*)nd->vending[i].item.card[0],
			st->stack->stack_data[st->start+2].ref
		);
		set_reg(st,NULL,
			num = add_str("\'@vending_card2")+(i<<24),
			str_buf+str_data[num&0x00ffffff].str,
			(void*)nd->vending[i].item.card[1],
			st->stack->stack_data[st->start+2].ref
		);
		set_reg(st,NULL,
			num = add_str("\'@vending_card3")+(i<<24),
			str_buf+str_data[num&0x00ffffff].str,
			(void*)nd->vending[i].item.card[2],
			st->stack->stack_data[st->start+2].ref
		);
		set_reg(st,NULL,
			num = add_str("\'@vending_card4")+(i<<24),
			str_buf+str_data[num&0x00ffffff].str,
			(void*)nd->vending[i].item.card[3],
			st->stack->stack_data[st->start+2].ref
		);
		set_reg(st,NULL,
			num = add_str("\'@vending_limit")+(i<<24),
			str_buf+str_data[num&0x00ffffff].str,
			(void*)nd->vending[i].item.limit,
			st->stack->stack_data[st->start+2].ref
		);
		if(nd->vending[i].item.amount > 0)
			count++;
	}
	set_reg(st,NULL,
		num = add_str("\'@vending_num"),
		str_buf+str_data[num&0x00ffffff].str,
		(void*)count,
		st->stack->stack_data[st->start+2].ref
	);
	if(nd->vender_id != 0) {
		open = 1;
		set_reg(st,NULL,
			num = add_str("\'@vending_title$"),
			str_buf+str_data[num&0x00ffffff].str,
			(void*)nd->shop_title,
			st->stack->stack_data[st->start+2].ref
		);
	}
	set_reg(st,NULL,
		num = add_str("\'@vending_status"),
		str_buf+str_data[num&0x00ffffff].str,
		(void*)open,
		st->stack->stack_data[st->start+2].ref
	);

	return 0;
}

/*==========================================
 * NPCIXJ
 *------------------------------------------
 */
int buildin_openvending(struct script_state *st)
{
	char *title;
	const char *event = "";
	struct npc_data *nd;

	title = conv_str(st,& (st->stack->stack_data[st->start+2]));

	if( st->end>st->start+3 )
		event = conv_str(st,& (st->stack->stack_data[st->start+3]));

	if(st->end > st->start+4)
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+4])));
	else
		nd = map_id2nd(st->oid);

	vending_opennpcvending(map_id2nd(st->oid),title,event);

	return 0;
}

/*==========================================
 * NPCIX
 *------------------------------------------
 */
int buildin_closevending(struct script_state *st)
{
	struct npc_data *nd;

	if(st->end > st->start+2)
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+2])));
	else
		nd = map_id2nd(st->oid);

	if(nd == NULL)
		return 0;

	vending_closenpcvending(nd);

	return 0;
}

/*==========================================
 * IXJĂ邩
 *------------------------------------------
 */
int buildin_checkvending(struct script_state *st)
{
	int ret = 0;
	struct npc_data *nd;

	if(st->end > st->start+2)
		nd = npc_name2id(conv_str(st,& (st->stack->stack_data[st->start+2])));
	else
		nd = map_id2nd(st->oid);

	if(nd)
		ret = nd->vender_id? 1: 0;

	push_val(st->stack,C_INT,ret);

	return 0;
}
//  -- NPCIX by Blaze
